#include "header/conector.h"


// Reference to navigator
Navigator *navigator;

/************************************************************************
 * Class:     navmap_ConectorJava
 * Method:    getMap
 * Signature: ()[I
 */
JNIEXPORT jintArray JNICALL Java_navmap_ConectorJNI_getMap (JNIEnv *env, jobject obj)
{

	if (navigator == NULL) return NULL;

	// Local map

	int *localmap = navigator->makeMap->getMap();


	if (localmap == NULL) return NULL;

	jint* pix;

	jintArray result;

	// Matrix of Java colors
	pix = (jint*)malloc(300*300*4);

	if (pix == 0) printf("Nao ha memoria suficiente \n");

	int cor;
	for (int i=0; i<300; i++)
		for (int j=0; j<300; j++)
		{
			cor = localmap[(i*300) + j];
			switch(cor){

			//Clear cells
			case CELL_CLEAR: pix[(299-i)*300 + j] = JAVA_white;
					 break;

			//Obstacle cells
			case CELL_BUSY:  pix[(299-i)*300 + j] = JAVA_black;
					 break;

			//Unknown cells
			case CELL_UNKNO: pix[(299-i)*300 + j] = JAVA_gray;
				 	 break;

			//Tolerance cells
			case CELL_TOLER: pix[(299-i)*300 + j] = JAVA_green;
					 break;

			default: break;
			}

		}

	result = env->NewIntArray(300*300);
	env->SetIntArrayRegion(result, 0, 300*300, pix);
	free(pix);


    return result;

}



/************************************************************************
 * Class:     navmap_ConectorJava
 * Method:    getBatteryStatus
 * Signature: ()[F
 */
JNIEXPORT jfloatArray JNICALL Java_navmap_ConectorJNI_getBatteryStatus  (JNIEnv *env, jobject obj)
{
		jfloatArray result;

		jfloat *bat = (jfloat*)navigator->getBatteryStatus();

		result = env->NewFloatArray(4);
		env->SetFloatArrayRegion(result, 0, 4, bat);

		return result;

}

/*****************************************************************************
/*
 * Class:     navmap_ConectorJNI
 * Method:    getPosition
 * Signature: ()[J
 */
JNIEXPORT jfloatArray JNICALL Java_navmap_ConectorJNI_getPosition (JNIEnv *env, jobject obj)
{
		jfloatArray result;

		jfloat *position = (jfloat*)navigator->getPosition();

		result = env->NewFloatArray(3);
		env->SetFloatArrayRegion(result, 0, 3, position);

		return result;

}


/************************************************************************
/*
 * Class:     navmap_ConectorJava
 * Method:    startNomad
 * Signature: (IILjava/lang/String;)Z
 */

JNIEXPORT jboolean JNICALL Java_navmap_ConectorJNI_startNomad
        (JNIEnv *env, jobject obj, jint posx, jint posy, jstring nome_arquivo)
{

	if (navigator == NULL)
	{
		navigator = new Navigator();
	}

	const char *filename = env->GetStringUTFChars(nome_arquivo, NULL);

	if (filename == 0)
	{
		printf("Erro ao pegar nome do arquivo ou arquivo em Branco");
	}

	int result = navigator->start(posx, posy, (char*)filename);

	if (result != 0)
	{
		return JNI_FALSE;
	}
	return JNI_TRUE;


}





/************************************************************************
 * Class:     navmap_ConectorJava
 * Method:    finishNomad
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_navmap_ConectorJNI_finishNomad (JNIEnv *env, jobject obj)
{
	if (navigator == NULL) return false;
	if (navigator->finish() != 0)
	{
		return JNI_FALSE;
	}

	return JNI_TRUE;




}






/************************************************************************
 * Class:     navmap_ConectorJava
 * Method:    moveNomad
 * Signature: (II)Z;
 */
JNIEXPORT jboolean JNICALL Java_navmap_ConectorJNI_moveNomad
        (JNIEnv *env, jobject obj, jint coordX, jint coordY)
{
	jboolean result = navigator->move(coordX, coordY);

	return result;

}






/************************************************************************
 * Class:     navmap_ConectorJava
 * Method:    stopNomad
 * Signature: ()Z;
 */
JNIEXPORT jboolean JNICALL Java_navmap_ConectorJNI_stopNomad (JNIEnv *env, jobject obj)
{
       jboolean result = navigator->stop();

       return result;

}




/************************************************************************
 * Class:     navmap_ConectorJava
 * Method:    saveMap
 * Signature: (Ljava/lang/String;)Z;
 */
JNIEXPORT jboolean JNICALL Java_navmap_ConectorJNI_saveMap
        (JNIEnv *env, jobject obj, jstring nome_arquivo)
{
	const char *fname = env->GetStringUTFChars(nome_arquivo, NULL);
	jboolean result = navigator->makeMap->saveMap((char*)fname);

	return result;

}
