#include "header/navigator.h"

using namespace std;


Navigator::Navigator()
{
	nomad = new Nomad();
	cinematic = new Cinematic();
	makeMap = new MakeMap();
	baterias = (float*)malloc(4*4);
	posicoes = (float*)malloc(3*4);

}

Navigator::~Navigator()
{
	delete nomad;
	delete cinematic;
	delete state;
	delete makeMap;
	free(baterias);
	free(posicoes);

	printf("Navigator destruido sem problemas\n");

}

float * Navigator::getBatteryStatus()
{
	return baterias;

}

float * Navigator::getPosition()
{
	return posicoes;

}

int Navigator::start(int posx, int posy, char* map_filename)
{
	CREATE_SEMAPHORE;

	printf("Conectando com o ROBO\n");

	if (nomad->Connect() != N_NO_ERROR)
	{
		printf("Erro ao conectar com o Nomad\n");
		return -1;
	}

	nomad->Speak("Conectado!");
	state = nomad->GetRobotState();

	//Zeroing the robot Odometers;
	cinematic->ZeroRobot(state);

	//Setting the map
	int result;
	if (strcmp(map_filename, "") == 0)
	{
		result = makeMap->newMap();
	}
	else
	{
		result = makeMap->openMap(map_filename);
	}

	if (result != 0)
	{
		printf("An error occurs when create map\n");
		return -1;
	}



	//startXY defined in navigator.h
	startX = posx;
	startY = posy;

	// Creating threads objects
	tRefreshMap = new TRefreshMap();
	tGetBattery = new TGetBattery(baterias);
	tGetPosition = new TGetPosition(posicoes);


	// Configurando o mapa
	tRefreshMap->setMakemap(makeMap, posx, posy);

	//Passando as posicoes inciais do robo
	tGetPosition->setPosition(posx,posy);


	// Inicialize threads
	tptrGetPosition = tGetPosition->CreateThread();
	tGetPosition->mSleep(500);

	tptrRefreshMap = tRefreshMap->CreateThread();
	tRefreshMap->mSleep(500);

	tptrGetBattery = tGetBattery->CreateThread();
	tGetBattery->mSleep(500);



	// Start OK
	return 0;

}

int Navigator::finish()
{

	if (tptrRefreshMap != 0)
	{
		tRefreshMap->setContinua(false);
		tRefreshMap->JoinThread(tptrRefreshMap);

		delete tptrRefreshMap;
		delete tRefreshMap;
		printf("Fim da thread AtualizaMapa\n");
	}

	if (tptrGetBattery != 0)
	{
		tGetBattery->setContinua(false);
		tGetBattery->JoinThread(tptrGetBattery);

		delete tptrGetBattery;
		delete tGetBattery;
		printf("Fim da thread GetBaterias\n");
	}

	if (tptrGetPosition != 0)
	{
		tGetPosition->setContinua(false);
		tGetPosition->JoinThread(tptrGetPosition);

		delete tptrGetPosition;
		delete tGetPosition;
		printf("Fim da thread GetPosition\n");
	}

	REMOVE_SEMAPHORE;

	return 0;

}



int Navigator::move(int goal_x, int goal_y)
{

	// get actual position em relacao ao mapa global
	Coord start;

	ENTER_CRITICALSECTION;

	state = nomad->GetRobotState();
	Coord posInicial = cinematic->GetOdometry(state);

	start.x = (long)((posInicial.x + startX)/100);
	start.y = (long)((posInicial.y + startY)/100);
	start.rot = cinematic->GetOdometry(state).rot;

	LEAVE_CRITICALSECTION;

	// set the goal
	Coord goal;
	goal.x = goal_x;
	goal.y = goal_y;
	goal.rot = 0;


	printf("Start (%d, %d)\n", start.x, start.y);
	printf("Goal  (%d, %d)\n", goal.x, goal.y);

	// create the pathplanner
	pathPlanner = new PathPlanner(makeMap->getMap());

	// get the trajectory
	trajectory = pathPlanner->getTrajectory(start, goal);

	if (trajectory.empty())
	{
		printf("trajectory is empty\n");
		return 0;
	}


	// Setting the velocity[mm/s] and acceleration[mm/s^2]
	Coord vel, acc, pos;
	vel.x = vel.y = vel.rot = acc.rot = 300;
	acc.x = acc.y = 600;



	// run into list of points
	typeTrajectory::iterator found;
	found = trajectory.begin();

	pos.rot = 0;
	int i = 1;
	++found;
	Coord prev;

	//pegando posicoes iniciais
	prev.x = start.x;
	prev.y = start.y;

	bool inProgress = false;
	bool obstaculo = false;
	long distance;

	SenSonar *sonar2;
	SenIr *ir2;
	SenBumper *bumper;

	ENTER_CRITICALSECTION;

	sonar2 = new SenSonar();
	ir2 = new SenIr();
	bumper = new SenBumper();

	for (found; ( found != trajectory.end() && !obstaculo ); ++found)
	{

		Coord &E = *found;
		pos.x = E.x;
		pos.y = E.y;

		pos.x = (pos.x - prev.x)*100;
		pos.y = (pos.y - prev.y)*100;

			state = nomad->GetRobotState();
			cinematic->SetPosition(state, pos, vel, acc, N_AXIS_POSITION_RELATIVE);
			inProgress = cinematic->Moving(state);


			while (inProgress)
			{
				inProgress = cinematic->Moving(state);
				N_GetSonar(1);
				N_GetInfrared(1);
				N_GetBumper(1);

				if (bumper->Read(state).id >= 0)
				{
					printf("Colisao com o nomad\n");
					inProgress = false;
					obstaculo = true;
					cinematic->Stop(state);
					delete sonar2;
					delete ir2;
					delete bumper;
					LEAVE_CRITICALSECTION;
					return 0;
				}

				state = nomad->GetRobotState();
				for (int i=0; i< 24; i++)
				{
					distance = sonar2->Read(state,i) <? ir2->Read(state,i);
					if (distance < 300)
					{
						if (distance < 200)
						{
							printf("Obstaculo muito proximo do Nomad\n  Perigo ao movimentar o Nomad\n");
							inProgress = false;
							obstaculo = true;
							cinematic->Stop(state);
							delete sonar2;
							delete ir2;
							delete bumper;
							LEAVE_CRITICALSECTION;
							return 0;

						}
						printf("Encontrei obstaculo\n");
						inProgress = false;
						obstaculo = true;
						cinematic->Stop(state);
						break;
					}
				}
				makeMap->refreshMap2(startX, startY);

			}

		prev.x = prev.x + (long)(pos.x/100);
		prev.y = prev.y + (long)(pos.y/100);
		i++;

	}


	delete sonar2;
	delete ir2;
	delete bumper;


	LEAVE_CRITICALSECTION;


	// Tracando nova trajetoria;
	if (obstaculo)
	{
		printf("---> Calculando nova trajetoria\n");
		move(goal_x, goal_y);
	}
	else
	{
		printf("Cheguei no objetivo\n");
	}



}

int Navigator::stop()
{
	GET_SEMAPHORE;

	ENTER_CRITICALSECTION;
		state = nomad->GetRobotState();
		cinematic->Stop(state);
	LEAVE_CRITICALSECTION;

	return 0;

}



