#include "header/pathplanner.h"

// the search Map
int *mapa;

/* --------------------------------------------------------------
 * Function: 	getGridValue
 * Purpose: 	Get the value in a cell of GridMap
 * Arguments:	x - coordinate x of cell
				y - coordinate y of cell
 * Return:		the value in the cell(x,y)
 * --------------------------------------------------------------*/
int getGridValue(int x, int y)
{
	if( x < 0 ||
	    x >= 100 ||
		 y < 0 ||
		 y >= 100
	  )
	{
		return 9;
	}

	return mapa[(y*100)+x];
}


/* --------------------------------------------------------------
 * Function: 	PathPlanner::PathPlanner
 * Purpose: 	Constructor of PathPlanner
 * Arguments:	map - an matrix[300*300] of int
 * Return:		none
 * --------------------------------------------------------------*/
PathPlanner::PathPlanner(int map[300*300])
{

	mapa = (int*)malloc(100*100*4);

	if (mapa == NULL)
	{
		printf("Erro no malloc\n");
		exit(-1);
	}



	int pp= 0;
	int aux2 = 0;
	int indice = 0;


	for(int i=0; i <300; i+=3)
	{
		for (int j=0; j<300; j+=3)
		{
			pp = map[i*300 + j];
			for (int a=0; a < 3; a++)
			{
				for(int b=0; b<3; b++)
				{
					aux2 = map[(i+a)*300 + (j+b)];
					aux2 = (aux2 == -1)?9:aux2;
					pp = (aux2 > pp)?aux2:pp;
				}
			}
			mapa[indice] = pp;
			indice++;
		}
	}
}

PathPlanner::~PathPlanner()
{
	free(mapa);
}


/* --------------------------------------------------------------
 * Function: 	PathPlanner::getTrajectory
 * Purpose: 	Trace the trajectory to find the goal
 * Arguments:	start - the coordinate of start position
				goal - the coordinate of goal position
 * Return:		trajectory - a list of coordinates
 * --------------------------------------------------------------*/
typeTrajectory PathPlanner::getTrajectory(Coord start, Coord goal)
{

	// Create an instance of the search class...
	AStarSearch<MapSearchNode> astarsearch;

	// Create a start state
	MapSearchNode nodeStart;
	nodeStart.x = (int)(start.x/3);
	nodeStart.y = (int)(start.y/3);

	// Define the goal state
	MapSearchNode nodeEnd;
	nodeEnd.x = (int)(goal.x/3);
	nodeEnd.y = (int)(goal.y/3);

	// Set Start and goal states
	astarsearch.SetStartAndGoalStates( nodeStart, nodeEnd );

	unsigned int SearchState;
	unsigned int SearchSteps = 0;

	do
	{
		SearchState = astarsearch.SearchStep();

		SearchSteps++;

	} while( SearchState == AStarSearch<MapSearchNode>::SEARCH_STATE_SEARCHING );

	if( SearchState == AStarSearch<MapSearchNode>::SEARCH_STATE_SUCCEEDED )
	{
		MapSearchNode *node = astarsearch.GetSolutionStart();

		Coord nod;
		nod.x = node->x;
		nod.y = node->y;

		// Creating vectors
		double actual_dir, next_dir;

		for( ;; )
		{
			node = astarsearch.GetSolutionNext();
			if( !node )
			{
				break;
			}

			next_dir = atan(double(node->y - nod.y)/double(node->x - nod.x));

                	if (actual_dir != next_dir)
                	{
				nod.x = 3*nod.x;
				nod.y = 3*nod.y;
				
				trajectory.insert(trajectory.end(), nod);
			}

				actual_dir = next_dir;
				nod.x = node->x;
				nod.y = node->y;
		};

		trajectory.insert(trajectory.end(), goal);
		astarsearch.FreeSolutionNodes();

	}

	else if( SearchState == AStarSearch<MapSearchNode>::SEARCH_STATE_FAILED )
	{
		cout << "Search terminated. Did not find goal state\n";

	}


    return trajectory;


}

//**************************************************************************************
// MapSearchNode
//**************************************************************************************


bool MapSearchNode::IsSameState( MapSearchNode &rhs )
{

	// same state in a maze search is simply when (x,y) are the same
	if( (x == rhs.x) &&
		(y == rhs.y) )
	{
		return true;
	}
	else
	{
		return false;
	}

}


// Here's the heuristic function that estimates the distance from a Node
// to the Goal.

float MapSearchNode::GoalDistanceEstimate( MapSearchNode &nodeGoal )
{
	float xd = float( ( (float)x - (float)nodeGoal.x ) );
	float yd = float( ( (float)y - (float)nodeGoal.y) );

	return ((xd*xd) + (yd*yd));

}

bool MapSearchNode::IsGoal( MapSearchNode &nodeGoal )
{

	if( (x == nodeGoal.x) &&
		(y == nodeGoal.y) )
	{
		return true;
	}

	return false;
}

// This generates the successors to the given Node. It uses a helper function called
// AddSuccessor to give the successors to the AStar class. The A* specific initialisation
// is done for each node internally, so here you just set the state information that
// is specific to the application
bool MapSearchNode::GetSuccessors( AStarSearch<MapSearchNode> *astarsearch, MapSearchNode *parent_node )
{

	int parent_x = -1;
	int parent_y = -1;

	if( parent_node )
	{
		parent_x = parent_node->x;
		parent_y = parent_node->y;
	}


	MapSearchNode NewNode;

	// Procura por obstaculos no raio do robo

	// push each possible move except allowing the search to go backwards


	// Movimento na direcao Nordeste
	if( (getGridValue( x+1, y+1 ) < 8)
		&& !((parent_x == x+1) && (parent_y == y+1))
	  )
	{
		NewNode = MapSearchNode( x+1, y+1 );
		astarsearch->AddSuccessor( NewNode );
	}

	// Movimento na direcao Noroeste
	if( (getGridValue( x-1, y+1 ) < 8)
		&& !((parent_x == x-1) && (parent_y == y+1))
	  )
	{
		NewNode = MapSearchNode( x-1, y+1 );
		astarsearch->AddSuccessor( NewNode );
	}

	// Movimento na direcao Sudeste
	if( (getGridValue( x+1, y-1 ) < 8)
		&& !((parent_x == x+1) && (parent_y == y-1))
	  )
	{
		NewNode = MapSearchNode( x+1, y-1 );
		astarsearch->AddSuccessor( NewNode );
	}

	// Movimento na direcao Sudoeste
	if( (getGridValue( x-1, y-1 ) < 8)
		&& !((parent_x == x-1) && (parent_y == y-1))
	  )
	{
		NewNode = MapSearchNode( x-1, y-1 );
		astarsearch->AddSuccessor( NewNode );
	}


	if( (getGridValue( x-1, y ) < 8 )
		&& !((parent_x == x-1) && (parent_y == y))
	  )
	{
		NewNode = MapSearchNode( x-1, y );
		astarsearch->AddSuccessor( NewNode );
	}

	if( (getGridValue( x, y-1 ) < 8)
		&& !((parent_x == x) && (parent_y == y-1))
	  )
	{
		NewNode = MapSearchNode( x, y-1 );
		astarsearch->AddSuccessor( NewNode );
	}

	if( (getGridValue( x+1, y ) < 8)
		&& !((parent_x == x+1) && (parent_y == y))
	  )
	{
		NewNode = MapSearchNode( x+1, y );
		astarsearch->AddSuccessor( NewNode );
	}


	if( (getGridValue( x, y+1 ) < 8)
		&& !((parent_x == x) && (parent_y == y+1))
		)
	{
		NewNode = MapSearchNode( x, y+1 );
		astarsearch->AddSuccessor( NewNode );
	}


	return true;
}

// given this node, what does it cost to move to successor. In the case
// of our map the answer is the map terrain value at this node since that is
// conceptually where we're moving

float MapSearchNode::GetCost( MapSearchNode &successor )
{
	return (float) getGridValue( x, y );

}


