package navmap;

/**
 * Pattern Factory para gerenciar as instancias dos Conectores
 * Essa classe que determina qual tipo de conexao que a interface
 * ira estabelecer: Via Socket ou Via JNI, com a biblioteca libNavigator
 * <p>Title: NavMap</p>
 * <p>Description: Navegao por Mapeamento</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Graco - UnB</p>
 * @author Marcio Moreira de Sousa e Paulo Roberto Corra Dutra
 * @version 1.0
 */

public class ConectorFactory {

  /**
   * Via Socket
   */
  public static final int WEB=0;

  /**
   * Via JNI
   */
  public static final int JNI=1;

  /**
   * Interface Conector
   */
  private static IConector conector;

  /**
   * Implementacao do Pattern SingleTon
   */
  private static final ConectorFactory INSTANCE = new ConectorFactory();

  /**
   * Construtor
   */
  private ConectorFactory() {
  }


  /**
   * Inicializa a Factory com o tipo de conexao que ser estabelecida
   * @param tipoConector Tipo da Conexao
   * @return Instancia para a factory
   */
  public static ConectorFactory getInstance(int tipoConector){
    if (conector == null) {
      if (tipoConector == ConectorFactory.WEB)
        conector = new ConectorWEB();
     else
        conector = new ConectorJNI();
    }
    return INSTANCE;
  }

  /**
   * Pega a instancia da factory
   * @return Instancia para a factory
   */
  public static ConectorFactory getInstance() {
    return INSTANCE;
  }


  /**
   * Pega a instancia para o Conector instanciado pela Factory
   * @return Conector
   */
  public IConector getConector()
  {
    return conector;
  }

}