package navmap;


/**
 * Classe que faz a conexo entre os mtodos nativos da biblioteca C++
 * libNavigator e a interface da aplicaao NavMap em Java
 * <p>Title: NavMap</p>
 * <p>Description: Navegao por Mapeamento</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Graco - UnB</p>
 * @author Paulo Roberto Correa Dutra & Marcio Moreira de Sousa
 * @version 1.0
 */

public class ConectorJNI implements IConector{

  /**
   * Construtor da classe  responsvel por carregar a biblioteca
   * Nativa que contem os mtodos nativos.
   */
  public ConectorJNI() {
      try {
        System.out.println("Carregando a libNavigator");
        System.loadLibrary("Navigator");
        System.out.println("libNavigator loaded");

      } catch (Exception e){
        e.printStackTrace();
      }
    }



  /**
   * Mtodo Nativo que captura o Mapa feito em C++
   * @return Matriz com os dados do Mapa
   */
  public native int[] getMap();

  /**
   * Mtodo Nativo que captura o status das Bateris
   * @return Matriz com o valor da tenso nas 4 Baterias
   */
  public native float[] getBatteryStatus();

  /**
   * Mtodo Nativo para capturar a posicao do robo no mapa
   * @return Matriz com o valor da coordenada X e Y do robo
   */

   public native float[] getPosition();

  /**
   * Mtodo Nativo para iniciar a conexo e Mapeamento do Nomad
   * @param posx - coordenada X da posiao inicial do robo
   * @param posy - coordenada Y da posiao inicial do robo
   * @param nome_arquivo - Nome do Arquivo de Mapa
   * @return status
   */

  public native boolean startNomad(int posx, int posy, String nome_arquivo);

  /**
   * Mtodo Nativo para finalizar a conexo com o Nomad
   * @return status
   */
  public native boolean finishNomad();

  /**
   * Mtodo que envia o comando de movimento ao Nomad.
   * @param celx Coordenada X da posico de destino
   * @param cely Coordenada Y da posico de destino
   * @return Status do Comando
   */
  public native boolean moveNomad(int celx, int cely);

  /**
   * Mtodo que cancela o movimento do robo Nomad
   * @return Status do Comandos
   */
  public native boolean stopNomad();

  /**
   * Mtodo que salva um arquivo de mapa
   * @param nome_arquivo Nome do Arquivo com extenso .map
   * @return status do comando
   */
  public native boolean saveMap(String nome_arquivo);


  /**
   * Mtodo que nao  utilizado pelo conector jni apenas pelo conector web
   */
  public void close(){
  }

  /**
   * Envia texto para o sintetizador de Voz do robo
   * @param text Texto a ser enviado
   */
  public native void textToSpeech(String text);

}