package navmap;

import java.io.*;
import java.net.*;


/**
 * ConectorWeb - Classe que faz a conexao entre o servidor C++ e a
 * interface da aplicaao NavMap em Java via Socket;
 * Title:        NavMap
 * Description:  Navegao por Mapeamento
 * Copyright:    Copyright (c) 2002
 * Company:      Graco - UnB
 * @author Marcio Moreira de Sousa e Paulo Roberto Corra Dutra
 * @version 1.0
 *
 */

public class ConectorWEB implements IConector {

  public static final int CMD_StartNomad  = 1;
  public static final int CMD_FinishNomad = 2;
  public static final int CMD_MoveNomad   = 3;
  public static final int CMD_StopNomad   = 4;
  public static final int CMD_GetMap      = 5;
  public static final int CMD_GetBattery  = 6;
  public static final int CMD_GetPosition = 7;
  public static final int CMD_SaveMap     = 8;
  public static final int CMD_Speak       = 9;
  public static final int CMD_Close       = 10;


  /**
   * objeto que faz a conexao TCP/IP com o servidor
   */

  private Client client;


  private byte[] cmd = new byte[1];


  /**
   * Construtor da classe
   * Aqui  criada uma instancia para o clienteSocket
   */
  public ConectorWEB() {
    try{

      // Abre a conexao com o servidor navmapserver
      client = new Client(8091, 8093, "gospel.graco.unb.br", 1);

    }catch(IOException e){
      e.printStackTrace();
    }

  }

  /**
   * Mtodo que fecha a conexao com o servidor web
   */
  public synchronized void close(){
    try{
      cmd[0] = ConectorWEB.CMD_Close;

      client.send_bytes(cmd, 1);

    }catch(IOException e){
      e.printStackTrace();
    }

  }


    /**
   * Mtodo que captura o Mapa feito em C++ do Servidor
   * @return Matriz com os dados do Mapa
   */
  public synchronized int[] getMap(){
    int size = 300*300;
    int [] mapa = new  int[size];
	int [] mapa_ok = new int[1];

    try{
      cmd[0] = ConectorWEB.CMD_GetMap;

      client.send_bytes(cmd, 1);

	  client.recv_ints(mapa_ok, 1);

	  if (mapa_ok[0] == 1)
	  {
		  client.recv_ints(mapa, size);
	  }

    }catch(IOException e){

    }
    return mapa;
  }

  /**
   * Mtodo que captura o status das Bateris
   * @return Matriz com o valor da tenso nas 4 Baterias
   */
  public synchronized float[] getBatteryStatus(){

    int size = 4;
    float bat[] = new float[size];

    try
    {
      cmd[0] = ConectorWEB.CMD_GetBattery;
      client.send_bytes(cmd, 1);

      client.recv_floats(bat, size);

    } catch(IOException e){
      e.printStackTrace();
    }

    return bat;
  }


  /**
   * Mtodo que captura a posicao do robo no mapa
   * @return Matriz com o valor da coordenada X e Y do robo
   */

   public synchronized float[] getPosition(){
     int size = 3;
     float[] posicao = new float[size];

     try{
        cmd[0] = ConectorWEB.CMD_GetPosition;

        client.send_bytes(cmd, 1);

        client.recv_floats(posicao, size);

     }catch(IOException e)
     {
        e.printStackTrace();
     }

     return posicao;

   }



  /**
   * Mtodo que inicializa a conexo e o mapeamento com o Nomad
   * @param posx - coordenada X da posiao inicial do robo
   * @param posy - coordenada Y da posiao inicial do robo
   * @param nome_arquivo - Nome do Arquivo de Mapa que ser carregado
   * @return - status da inicializaao
   */
  public synchronized boolean startNomad(int posx, int posy, String nome_arquivo){

    int [] ret = new int[1];
    try
    {
      int []  pos = new int[2];
      pos[0] = posx;
      pos[1] = posy;
      cmd[0] = ConectorWEB.CMD_StartNomad;

      client.send_bytes(cmd, 1);
      client.send_ints(pos, 2);
      client.send_string(nome_arquivo);

      client.recv_ints(ret, 1);


      // Conecxo OK
      if (ret[0] == 0) {
        return true;
      }
      else {
        return false;
      }

    } catch(IOException e)
    {
      e.printStackTrace();
    }

    return false;
  }


  /**
   * Mtodo Nativo que finaliza a conexo com o Nomad
   * @return status da finalizaao
   */
  public boolean finishNomad(){

    int [] ret = new int[1];

    try {
      cmd[0] = ConectorWEB.CMD_FinishNomad;
      client.send_bytes(cmd, 1);
      client.recv_ints(ret, 1);

      if (ret[0] == 0){
        return true;
      }
      else {
        return false;
      }

    } catch(IOException e) {
        e.printStackTrace();
    }

    return false;
  }

  /**
   * Mtodo que envia o comando de movimento ao Nomad.
   * @param celx Coordenada X da posico de destino
   * @param cely Coordenada Y da posico de destino
   * @return Status do Comando
   */
  public boolean moveNomad(int celx, int cely){

    int [] ret = new int[1];

    try{
      int[] cels = new int[2];
      cels[0] = celx;
      cels[1] = cely;
      cmd[0] = ConectorWEB.CMD_MoveNomad;

      client.send_bytes(cmd, 1);
      client.send_ints(cels, 2);

      client.recv_ints(ret, 1);

      if (ret[0] == 0){
        return true;
      }
      else {
        return false;
      }
    }catch (IOException e){
      e.printStackTrace();
    }

    return false;
  }

  /**
   * Mtodo que cancela o movimento do robo Nomad
   * @return Status do Comandos
   */
  public boolean stopNomad(){
    int [] ret = new int[1];

    try{
      cmd[0] = ConectorWEB.CMD_StopNomad;
      client.send_bytes(cmd, 1);
      client.recv_ints(ret, 1);

      if (ret[0] == 0){
        return true;
      }
      else {
        return false;
      }
    }catch (IOException e ){
      e.printStackTrace();
    }

  return false;
  }

  /**
   * Mtodo que salva um arquivo de mapa
   * @param nome_arquivo Nome do Arquivo com extenso .map
   * @return status do comando
   */
  public boolean saveMap(String nome_arquivo){
    int [] ret = new int [1];

    try {
      cmd[0] = ConectorWEB.CMD_SaveMap;

      client.send_bytes(cmd, 1);
      client.send_string(nome_arquivo);

      client.recv_ints(ret, 1);

      if (ret[0] == 0){
        return true;
      }
      else {
        return false;
      }
    }catch (IOException e){
      e.printStackTrace();
    }

  return false;
  }



    /**
   * Envia texto para o sintetizador de Voz do robo
   * @param text Texto a ser enviado
   */
  public void textToSpeech(String text){
    try {
      cmd[0] = ConectorWEB.CMD_Speak;
      client.send_bytes(cmd, 1);
      client.send_string(text);
    } catch (IOException e){
      e.printStackTrace();
    }
  }

}