package navmap;

/**
 * Interface para os conectores, JNI e WEB
 * Title:        NavMap
 * Description:  Navegao por Mapeamento
 * Copyright:    Copyright (c) 2002
 * Company:      Graco - UnB
 * @author Marcio Moreira de Sousa e Paulo Roberto Corra Dutra
 * @version 1.0
 */


public interface IConector {


  /**
   * Mtodo que captura o Mapa feito em C++
   * @return Matriz com os dados do Mapa
   */
  public int[] getMap();

  /**
   * Mtodo que captura o status das Bateris
   * @return Matriz com o valor da tenso nas 4 Baterias
   */
  public float[] getBatteryStatus();

  /**
   * Mtodo para capturar a posicao do robo no mapa
   * @return Matriz com o valor da coordenada X e Y do robo
   */

   public float[] getPosition();

  /**
   * Mtodo para iniciar a conexo e Mapeamento do Nomad
   * @param posx - coordenada X da posiao inicial do robo
   * @param posy - coordenada Y da posiao inicial do robo
   * @param nome_arquivo - Nome do Arquivo de Mapa
   * @return status
   */
  public boolean startNomad(int posx, int posy, String nome_arquivo);

  /**
   * Mtodo para finalizar a conexo com o Nomad
   * @return status
   */
  public boolean finishNomad();

  /**
   * Mtodo que envia o comando de movimento ao Nomad.
   * @param celx Coordenada X da posico de destino
   * @param cely Coordenada Y da posico de destino
   * @return Status do Comando
   */
  public boolean moveNomad(int celx, int cely);

  /**
   * Mtodo que cancela o movimento do robo Nomad
   * @return Status do Comandos
   */
  public boolean stopNomad();

  /**
   * Mtodo que salva um arquivo de mapa
   * @param nome_arquivo Nome do Arquivo com extenso .map
   * @return status do comando
   */
  public boolean saveMap(String nome_arquivo);


  /**
   * Envia texto para o sintetizador de Voz do robo
   * @param text Texto a ser enviado
   */
  public void textToSpeech(String text);

  /**
   * Mtodo usado pelo ConectorWEB para fechar a conexao com o servidor;
   */
  public void close();
}