package navmap;

import javax.swing.JPanel;
import java.awt.Image;
import java.awt.image.*;


/**
 * Classe que captura o mapa do conector e gera uma imagem a partir da
 * matriz do mapa
 * <p>Title: NavMap</p>
 * <p>Description: Navegao por Mapeamento</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Graco - UnB</p>
 * @author Paulo Roberto e Marcio Moreira
 * @version 1.0
 */

public class ImageMapa extends JPanel {

  /**
   * Instancia da Factory de Conectores
   */
  private ConectorFactory conectorFactory;

  /**
   * Instancia para o Conector
   */
  private IConector conector;


  /**
   * Construtor
   */
  public ImageMapa() {

      conectorFactory = conectorFactory.getInstance();
      conector = conectorFactory.getConector();

  }


  /**
   * Pega a posiao atual do Robo;
   * @return matriz de inteiro com a posiao [x,y,rotacao] do robo
   */
  public int[] getPosicao()
  {
    float posicoes[] = conector.getPosition();

    int[] ret = new int[2];

    ret[0] = (int)(posicoes[0]/100);
    ret[1] = (int)(posicoes[1]/100);


    return ret;
  }

  /**
   * Classe que gera a imagem a partir da matriz do mapa
   * @return Imagem do mapa
   */
  public Image getImage() {

       // Pegando o mapa da biblioteca c++
       int pix[] = conector.getMap();
/*
        int w = 300;
        int h = 300;
        int pix[] = new int[w * h];
        int index = 0;
        for (int y = 0; y < h; y++) {
            int red = (y * 255) / (h - 1);
            for (int x = 0; x < w; x++) {
                        int blue = (x * 255) / (w - 1);
                    pix[index++] = (255 << 24) | (red << 16) | blue;
            }
        }
*/
       // Criando uma imagem a partir da matriz de pontos do mapa
       return createImage(new MemoryImageSource(300, 300, pix, 0, 300));

  }
}