package navmap;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Painel que contem os paineis de informaoes do Nomad
 * <p>Title: NavMap</p>
 * <p>Description: Navegao por Mapeamento</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Graco - UnB</p>
 * @author Paulo Roberto Correa Dutra & Marcio Moreira de Sousa
 * @version 1.0
 *
 */

public class PainelInfo extends JPanel {
  private JButton btConectar    = new JButton("Conectar");
  private JButton btDesconectar = new JButton("Desconectar");
  private JButton btSpeak       = new JButton("Falar");
  private JLabel lbStatus   = new JLabel("Status do NOMAD:");

  /* Paineis */
  private PainelBaterias painelBaterias;
  private PainelPosicao painelPosicao;
  private PainelStatus painelStatus;
  private ActionListener listener;



  /**
   *  Inicializa Threads
   * */
  public void Inicializar(int tipo){

    if (tipo == 0)
    {
      painelPosicao.SetSleep(2000);
      painelBaterias.SetSleep(10000);
    }

    painelBaterias.Inicializar();
    painelPosicao.Inicializar();
    painelStatus.setStatus(true);
  }


  /**
   *   Finaliza Threads
   * */
  public void Finalizar(){
    painelBaterias.Finalizar();
    painelPosicao.Finalizar();
    painelStatus.setStatus(false);
  }

  /**
   * Construtor
   * @param list listener para os botoes do painel
   */
  public PainelInfo(ActionListener list) {
    listener = list;
    try{
      jbInit();
    }catch(Exception ex){
      ex.printStackTrace();
    }
  }


  private void jbInit() throws Exception{
    /* Configuraes do PainelStatus */
    painelStatus = new PainelStatus();
    painelStatus.setBounds(new Rectangle(2, 156, 194, 132));

    /* Configuraes do PainelPosicoes */
    painelPosicao = new PainelPosicao();
    painelPosicao.setBounds(2,300,195,100);

    /* Configuraes do PainelBaterias */
    painelBaterias = new PainelBaterias();
    painelBaterias.setBounds(2,400,195,220);

    btConectar.setBounds(new Rectangle(42, 15, 111, 27));
    btConectar.addActionListener(listener);
    btConectar.setActionCommand("conectar");

    btDesconectar.setBounds(new Rectangle(42, 45, 111, 27));
    btDesconectar.addActionListener(listener);
    btDesconectar.setActionCommand("desconectar");
    btDesconectar.setEnabled(false);

    btSpeak.setBounds(new Rectangle(42, 90, 111, 27));
    btSpeak.addActionListener(listener);
    btSpeak.setActionCommand("falar");
    btSpeak.setEnabled(false);
    btSpeak.setForeground(Color.blue);


    lbStatus.setFont(new java.awt.Font("Serif", 1, 12));
    lbStatus.setForeground(Color.blue);
    lbStatus.setBounds(new Rectangle(46, 125, 150, 25));
    this.setLayout(null);
    this.setBorder(BorderFactory.createLineBorder(Color.black));
    this.add(painelPosicao, null);
    this.add(painelBaterias, null);
    this.add(painelStatus, null);
    this.add(lbStatus, null);
    this.add(btConectar, null);
    this.add(btDesconectar, null);
    this.add(btSpeak, null);

  }

  /**
   * Mtodo para inverter botoes de acordo com o status do Nomad
   */
  public void invertEnableButtons()
  {
    if (btDesconectar.isEnabled())
    {
      btDesconectar.setEnabled(false);
      btSpeak.setEnabled(false);
      btConectar.setEnabled(true);
    }
    else
    {
      btDesconectar.setEnabled(true);
      btSpeak.setEnabled(true);
      btConectar.setEnabled(false);

    }
  }



}