package navmap;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;




/**
 * Classe que mostra o Mapa
 * <p>Title: NavMap</p>
 * <p>Description: Navegao por Mapeamento</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Graco - UnB</p>
 * @author Paulo Roberto Correa Dutra & Marcio Moreira de Sousa
 * @version 1.0
 *
 */

public class PainelMapa extends JPanel{


  private Image mapa;
  private ImageMapa imageMapa = new ImageMapa();
  private Thread atualiza;
  private int grid = 0;
  boolean gridON = false;
  private int posx, posy;
  private boolean connected;
  private PainelPosicao pp;
  private int timetowait = 500;


  public void SetSleep(int valor){
    timetowait = valor;
  }


  /**
   * Mtodo que configura o grid que dever ser mostrado no Mapa
   * @param v_grid valor em milimetros da celula do grid da TELA
   * obs.: o grid do mapa  fixo, o que  alterado  o grid de visualizaao
   */
  public void Configurar(int v_grid)
  {
    if (v_grid > 0){
      gridON = true;
      grid = (int)v_grid/50;
    }
    else
      gridON = false;
  }

  /**
   * Inicializa a Thread
   */
  public void Inicializar()
  {
    connected = true;
    atualiza = new Thread(new AtualizaMapa());
    atualiza.start();
  }


  /**
   * Finaliza a thread
   */
  public void Finalizar()
  {
    connected = false;
    try
    {
       atualiza.join();
    } catch(InterruptedException e){
          e.printStackTrace();
    }

  }

  /**
   * Construtor
   * @param ml listener para enventos de click do mouse
   * @param mm listener para evento de movimentacao do mouse
   */
  public PainelMapa(MouseListener ml, MouseMotionListener mm) {

    // Inicializando o Mapa
    mapa = createImage(new MemoryImageSource(300, 300, null, 0, 300));
    this.addMouseListener(ml);
    this.addMouseMotionListener(mm);
    connected = false;
    posx = posy = 0;
  }

  /**
   * Mtodo que pinta os compononentes do painel
   * @param g -
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g); //paint background
    int x = (int)this.getSize().getWidth()/2;
    int y = (int)this.getSize().getHeight()/2;

    x = (x > 300)?x-300:0;
    y = (y > 300)?y-300:0;

    g.drawImage(mapa,x, y ,600,600,this);

    if (connected)
    {
        g.setColor(Color.blue);
        g.fillArc(posx*2+x-6, (600-posy*2)+y-6, 12,12,0,360);

        g.setColor(Color.red);
        g.fillArc(posx*2+x-6, (600-posy*2)+y-6,12,12,45,90);

    }

    if (gridON){
      g.setColor(Color.lightGray);
      for (int i=grid; i < 600; i+= grid){
        g.drawLine(i+x, y, i+x, y+600);
        g.drawLine(x, i+y, x+600, i+y);
      }
    }

  }


  /**
   * Inner class que atualiza valores do mapa a cada 500 milisegundos
   */
  public class AtualizaMapa implements Runnable{
    public void run(){
      int[] pos = new int[2];
      while (connected){
        try{
          mapa = imageMapa.getImage();
          pos = imageMapa.getPosicao();
          posx = pos[0] ;
          posy = pos[1];
          repaint();
          Thread.sleep(timetowait);

        }catch(InterruptedException e){
          e.printStackTrace();
        }
      }
    }
  }


} // Fim da Classe PainelMapa


