package navmap;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.File;

/**
 * Classe que contem todos os paineis da aplicacao
 * <p>Title: NavMap</p>
 * <p>Description: Navegao por Mapeamento</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Graco - UnB</p>
 * @author Paulo Roberto Correa Dutra & Marcio Moreira de Sousa
 * @version 1.0
 *
 */

public class TelaPrincipal extends JFrame {

  /**
   * Painel Info
   */
  private PainelInfo painelInfo;

  /**
   * Painel Mapa
   */
  private PainelMapa painelMapa;

  /**
   * Menu Principal
   */
  private MenuPrincipal menuPrincipal;

  /**
   * Tela de Conexao
   */
  private TelaConexao tc;

  /**
   * Tela Sobre
   */
  private TelaSobre telaSobre;

  /**
   * Barra de Status
   */
  private BarraStatus barraStatus;


  private EventoAcao listener = new EventoAcao();
  private EventoMouse mouseListener = new EventoMouse();
  private MouseMotion mouseMotion = new MouseMotion();
  private JFileChooser fc = new JFileChooser();
  private JScrollPane scrMapa;

  private IConector conector;
  private ConectorFactory conectorFactory;

  private boolean connected;

  private int tipo;

  /**
   * Construtor
   * @param tipo Tipo da conexao com a biblioteca libNavigator
   * (0 - WEB | 1 - JNI)
   */
  public TelaPrincipal(int valor){
    connected = false;

    tipo = valor;

    if (tipo == 0)
    {
        /* Instancia do ConectorWEB */
        conectorFactory = conectorFactory.getInstance(0);
        conector = conectorFactory.getConector();
    }
    else
    {
        /* Instancia do ConectorJNI */
        conectorFactory = conectorFactory.getInstance(1);
        conector = conectorFactory.getConector();

    }


    try{
      jbInit();
    }catch(Exception ex){
      ex.printStackTrace();
    }


  }

  private void jbInit() throws Exception{

    /* Menu Principal */
    menuPrincipal = new MenuPrincipal(listener);


    /* FileChooser */
    MapaFilter mapaFilter = new MapaFilter();
    fc.setFileFilter(mapaFilter);
    fc.setCurrentDirectory(new File("."));

    /* Configuraes do PainelInfo */
    painelInfo = new PainelInfo(listener);
    Dimension piDim = new Dimension(210, this.getHeight()-25);
    painelInfo.setPreferredSize(piDim);

    /* Configuraoes do PainelMapa*/
    painelMapa = new PainelMapa(mouseListener, mouseMotion);
    painelMapa.setPreferredSize(new Dimension(600,600));
    scrMapa = new JScrollPane();
    scrMapa.getViewport().add(painelMapa, null);

    /* Configuraoes da Barra de Status */
    barraStatus = new BarraStatus();
    barraStatus.setPreferredSize(new Dimension(this.getWidth(), 25));

    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(painelInfo, BorderLayout.WEST);
    this.getContentPane().add(scrMapa, BorderLayout.CENTER);
    this.getContentPane().add(barraStatus, BorderLayout.SOUTH);

    this.setJMenuBar(menuPrincipal);
    this.setTitle("NavMap - Navegao por Mapeamento");
    this.addWindowListener(listener);


    if (tipo == 0)
    {
        menuPrincipal.setMenuMapa_OFF();
        painelMapa.SetSleep(10000);
    }


   }

   /**
    * Verifica se est conectado com o Nomad
    * @return status
    */
  public boolean isConnected()
  {
    return connected;


  }

  /**
   * Abre a tela para o usuario entrar com as informaes antes de conectar com o Nomad
   */
   private void Conectar(){

     /* Tela de Conexao */

     tc = new TelaConexao(listener);
     int x = (Toolkit.getDefaultToolkit().getScreenSize().width/2 - 252);
     int y = (Toolkit.getDefaultToolkit().getScreenSize().height/2 - 95);
     tc.setBounds(x,y,505,190);
     tc.show();

   }


   /**
    * Faz a validaao dos dados inseridos e conecta com o Nomad
    */
   private void Connect_Nomad()
   {

      int x = tc.getStartCellX();
      int y = tc.getStartCellY();

      if (x <0 | x> 30000 | y< 0 | y> 30000)
      {
       JOptionPane.showMessageDialog(this, "Coordenadas invalidas ! Por favor digite novamente", "Erro", JOptionPane.ERROR_MESSAGE);
       tc.toFront();
      }

      else
      {
        connected = conector.startNomad(tc.getStartCellX(), tc.getStartCellY(), tc.getMapFileName());

        //connected = true;
        tc.dispose();
        if (connected)
        {
          painelInfo.invertEnableButtons();
          painelInfo.Inicializar(tipo);
          painelMapa.Inicializar();
          menuPrincipal.invertMenuEnable();
        }
        else
        {
          JOptionPane.showMessageDialog(this, "Nao foi possivel conectar-se com o Nomad", "Erro", JOptionPane.ERROR_MESSAGE);
        }
      }
   }

   /**
    * Desconecta com o Nomad
    */
   private void Desconectar(){
     if (connected)
     {
        painelMapa.Finalizar();
        painelInfo.Finalizar();
        conector.finishNomad();
        connected = false;
     }

   }


   private void Falar(){

        String text = JOptionPane.showInputDialog(this, "Texto:", "Texto para o Nomad Falar", JOptionPane.PLAIN_MESSAGE  );

        if (text != null)
        {
          conector.textToSpeech(text);
        }
   }


   /**
    * Fecha a aplicaao
    */
   private void Sair(){
     Desconectar();
     conector.close();
     System.exit(0);
   }

   /**
    * Salva o mapa para um arquivo
    */
   private void Salvar(){
     if (fc.showSaveDialog(this) == 0)
     {
        if (connected )
        {
          conector.saveMap(fc.getSelectedFile().getAbsolutePath());
        }
        else
        {
          JOptionPane.showMessageDialog(this, "Para salvar o mapa, voce precisa estar conectado com o robo", "Erro", JOptionPane.ERROR_MESSAGE);
        }
     }
   }

   /**
    * Mostra a tela Sobre
    */
   private void Sobre(){
    telaSobre = new TelaSobre();
    telaSobre.setSize(300,300);
    telaSobre.setLocation(250,200);
    telaSobre.show();
  }

  /**
   * Mostra a Tela Conteuto
   * Nao implementada nessa versao
   */
   private void Conteudo(){
     System.out.println("Conteudo");
   }



  /**
   *
   * <p>Title: NavMap</p>
   * <p>Description: Navegao por Mapeamento</p>
   * <p>Copyright: Copyright (c) 2002</p>
   * <p>Company: Graco - UnB</p>
   * @author Paulo Roberto Correa Dutra & Marcio Moreira de Sousa
   * @version 1.0
   * Classe de Tratamento de Eventos (Botoes e Menus);
   */

  class EventoAcao extends WindowAdapter implements ActionListener {
      public void actionPerformed(ActionEvent event){

        if (event.getActionCommand().equalsIgnoreCase("conectar"))
        {
          Conectar();
        }
        if (event.getActionCommand().equalsIgnoreCase("desconectar"))
        {
          painelInfo.invertEnableButtons();
          menuPrincipal.invertMenuEnable();
          Desconectar();
        }

        if (event.getActionCommand().equalsIgnoreCase("falar")) Falar();
        if (event.getActionCommand().equalsIgnoreCase("sair")) Sair();
        if (event.getActionCommand().equalsIgnoreCase("salvar")) Salvar();
        if (event.getActionCommand().equalsIgnoreCase("sobre")) Sobre();
        if (event.getActionCommand().equalsIgnoreCase("conteudo")) Conteudo();
        if (event.getActionCommand().equalsIgnoreCase("connect_nomad")) Connect_Nomad();

        /* Menu Config */
        if (event.getActionCommand().equalsIgnoreCase("grid off")){
          painelMapa.Configurar(0);
          painelMapa.repaint();
        }
        if (event.getActionCommand().equalsIgnoreCase("grid 100 mm")){
          painelMapa.Configurar(100);
          painelMapa.repaint();
        }
        if (event.getActionCommand().equalsIgnoreCase("grid 500 mm")){
          painelMapa.Configurar(500);
          painelMapa.repaint();

        }
        if (event.getActionCommand().equalsIgnoreCase("grid 1000 mm")){
          painelMapa.Configurar(1000);
          painelMapa.repaint();
        }

      }

      public void windowClosing(WindowEvent we)
      {
        Sair();
      }

  }

  /**
   *
   * <p>Title: NavMap</p>
   * <p>Description: Navegao por Mapeamento</p>
   * <p>Copyright: Copyright (c) 2002</p>
   * <p>Company: Graco - UnB</p>
   * @author Marcio Moreira de Sousa e Paulo Roberto Corra Dutra
   * @version 1.0
   * Classe para tratamento dos clicks de Mouse
   */
  class EventoMouse implements MouseListener{

         public void mouseExited(MouseEvent e){
         }
         public void mouseEntered(MouseEvent e){

         }

         public void mouseReleased(MouseEvent e){
           if (!connected) return;

           if (SwingUtilities.isLeftMouseButton(e))
           {
             int x = (int)painelMapa.getSize().getWidth()/2;
             int y = (int)painelMapa.getSize().getHeight()/2;



             x = (x > 300)?x-300:0;
             y = (y > 300)?y-300:0;

             x = e.getX() - x;
             y = e.getY() - y;

             // Mandar o robo andar at a posiao clicada

             //Adequando o sistema de referencia do mapa java e o mapa real
             x = (int)(x/2);
             y = 300-(int)(y/2);

             //System.out.println("Coordenadas Robo: " + x + "," + y);
             conector.moveNomad(x,y);


           }
           else
           {
              System.out.println("mouseRight in position (X,Y)" + e.getX() + "," + e.getY());
              System.out.println("Parando o robo");
              conector.stopNomad();
           }
         }

         public void mousePressed(MouseEvent e){
         }

         public void mouseClicked(MouseEvent e){
         }

  }



  /**
   *
   * <p>Title: NavMap</p>
   * <p>Description: Navegao por Mapeamento</p>
   * <p>Copyright: Copyright (c) 2002</p>
   * <p>Company: Graco - UnB</p>
   * @author Marcio Moreira de Sousa e Paulo Roberto Corra Dutra
   * @version 1.0
   *
   * Classe que trata os eventos de movimentao do mouse e calcula a coordenada
   * que dever ser mostrada na barra de status
   *
   */
  class MouseMotion implements MouseMotionListener{

         public void mouseMoved(MouseEvent e) {

             int x = (int)painelMapa.getSize().getWidth()/2;
             int y = (int)painelMapa.getSize().getHeight()/2;

             x = (x > 300)?x-300:0;
             y = (y > 300)?y-300:0;

             x = e.getX() - x;
             y = e.getY() - y;

             // Mandar o robo andar at a posiao clicada

             //Adequando o sistema de referencia do mapa java e o mapa real

             x = (int)(x/2);
             y = 300-(int)(y/2);

             if  ((x < 0) || (x > 300))
             {
                x = 0;
             }

             if  ((y < 0) || (y > 300))
             {
                y = 0;
             }

             barraStatus.SetStatusText("Mover nomad para a posiao: (" + x*100 + "," + y*100 + ") [mm]");
         }

         public void mouseDragged(MouseEvent e) {

         }

  }



}