// <APPLET CODE=RobotControl.class width=400 height=200></APPLET>

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.net.*;
import java.io.*;

//
// Main class for control of the XR4000 robot
//

public class RobotControl extends Applet {

static String[]
  stringButtons =
{"CCW","Front","CW","Left","","Right","","Back","","Change"},
  actionButtons =
{"counterclockwise","front","clockwise","left","","right","","back","",""},
  initSetup     = {"100","314","500","524"};

Panel
  controlPanel, sensorPanel, sonarPanel, batteryPanel;

Button[]
  controlButtons = new Button[11];

Choice
  sensorChoice = new Choice();

Label
  controlLabel;

URL
  server;

SocketIO
  connection;

GetSensors
  sensors;

SensorCanvas
  sensorCanvas;

BatteryCanvas
  batteryCanvas;

TextField[]
  setupText = new TextField[4];

int[] sonar = new int[24],
      ir = new int[24];

int battery = 13000;

// Init method for the applet. Initializes the applet and creates the panel and the
// buttons for its use.

public void init() {

  controlLabel = new Label ("");

  try {
  	connection = new SocketIO (getCodeBase().getHost(),8086); // opens the connection with the camera's remote server
	connection.send ("sensors=on, distance=500, speed=100");
  }
  catch (UnknownHostException e) {
	System.err.println ("Error in URL of robot server.");
  }
  catch (IOException e) {
	System.err.println ("Error connecting robot server.");
	controlLabel.setText ("Error connecting server.");
  }

  controlPanel = new Panel (new GridLayout (7,3));

  int i;

  for (i=0;i<4;i++) {			// creates all textfields
	setupText[i] = new TextField (initSetup[i]);
	setupText[i].addTextListener (new EventText (i));
  }

  for (i=0;i<9;i++) {                   // creates all buttons
    controlButtons[i]=new Button (stringButtons[i]);
    controlButtons[i].addActionListener (new EventButton (i));
    controlPanel.add (controlButtons[i]);
  }

  controlButtons[9]= new Button (stringButtons[9]);
  controlButtons[9].addActionListener (new EventButton (9));

  controlPanel.add (new Label (""));
  controlPanel.add (new Label ("Distance"));
  controlPanel.add (new Label ("Speed"));
  controlPanel.add (new Label ("Linear"));
  controlPanel.add (setupText[2]);
  controlPanel.add (setupText[0]);
  controlPanel.add (new Label ("Angular"));
  controlPanel.add (setupText[3]);
  controlPanel.add (setupText[1]);
  controlPanel.add (new Label (""));
  controlPanel.add (new Label (""));
  controlPanel.add (controlButtons[9]);

  setLayout (new BorderLayout (2,2));
  add (controlPanel,BorderLayout.CENTER);
  add (controlLabel,BorderLayout.SOUTH);

  sensors = new GetSensors();

  sensorCanvas = new SensorCanvas (sonar);
  sensorCanvas.setScale ((float) 1.5);
  sensorCanvas.repaint();
  

  batteryCanvas = new BatteryCanvas (battery);
  batteryCanvas.setScale ((float) 1.0);
  batteryCanvas.repaint();

  sensorPanel = new Panel (new GridLayout (2,1));
  sonarPanel = new Panel (new BorderLayout ());
  batteryPanel = new Panel (new BorderLayout ());

  sonarPanel.add (sensorCanvas,BorderLayout.SOUTH);
  sonarPanel.add (sensorChoice,BorderLayout.NORTH);
  batteryPanel.add (batteryCanvas,BorderLayout.SOUTH);

  sensorPanel.add (sonarPanel);
  sensorPanel.add (batteryPanel);

  sensorChoice.addItem ("Sonar");
  sensorChoice.addItem ("Infrared");

  add (sensorPanel,BorderLayout.EAST);


}

// Destroy method for the applet. Stops the socket connection with the servers.

public void destroy () {

  sensors.cancel ();

  try {
	connection.send("sensors=off, goodbye");	// closing the remote server
  	connection.close();
  } catch (IOException e) {
	System.err.println ("Error closing connection with robot server.");
  }
}


// Class used to catch the events created by the buttons.

class EventButton implements ActionListener {

	int index;

  EventButton (int index) {
		this.index = index;
  }

  public void actionPerformed (ActionEvent e) {
	String command;

	if (index < 9)
		command = "direction=" + actionButtons[index] + ", start";
	else
		command = "distance="+setupText[2].getText()+",teta="+setupText[3].getText()+",speed="+setupText[0].getText()+",omega="+setupText[1].getText()+",start";
	
	try {
       		connection.send (command);
	}
	catch(Exception exception) {
       		System.err.println ("Error connecting server. Command: " + command);
		exception.printStackTrace();
		controlLabel.setText ("Error sending command.");
	}
  }

}

class EventText implements TextListener {

	int index;

	EventText (int index) {
		this.index = index;
	}

	public void textValueChanged (TextEvent e) {

		int textvalue = Integer.parseInt (setupText[index].getText());

	}

}

// Threaded class that gets sensor data from server

class GetSensors extends Thread {

	String result;
	boolean running = true;
	StringToIntArray converter;
	int[] batterydata = new int[24];
	int i, lessbattery;

	GetSensors () {
		start();
	}

	void cancel () {
		running = false;
	}

	public void run () {

		while (running) {
			try {
				
				if (sensorChoice.getSelectedIndex() == 0) // sonar
					connection.send ("get=sonar");
				else
					connection.send ("get=IR");
				result = connection.get ();		
				sleep (500);		/* one second update */
			}
			catch (InterruptedException e) { }
			catch (IOException e) {
				controlLabel.setText ("Server out.");
				running = false;
			}
			converter = new StringToIntArray (result);

			if (sensorChoice.getSelectedIndex() == 0) // sonar
				sensorCanvas.setLimits (500,2000,5000);
			else
				sensorCanvas.setLimits (200,50,10);

			sensorCanvas.update (converter.getArray());
			
			try {
				connection.send ("get=battery");
				result = connection.get ();		
				sleep (500);		/* one second update */
			}
			catch (InterruptedException e) { }
			catch (IOException e) {
				controlLabel.setText ("Server out.");
				running = false;
			}
			converter = new StringToIntArray (result);

			batterydata = converter.getArray();			
			lessbattery = 15000;

			for (i=0;i<4;i++)
				lessbattery = batterydata[i] < lessbattery
					? batterydata[i] : lessbattery;

			batteryCanvas.update (lessbattery);

			System.err.println ("Battery: "+lessbattery);
		}
	}

}

// Class that transforms string from server to array of integers

public class StringToIntArray {

	String s;
	int p,q,i;
	int[] array;

	public StringToIntArray (String S) {
		s = S;
		array = new int[24];
	}

	public void putString (String S) {
		s = S;
	}

	public int[] getArray () {

		p = 0;
		q = 0;
		i = 0;

		while ((q != -1) && (i < 24)) {
			p = s.indexOf (' ',p);
			q = s.indexOf (' ',p+1);
			if (q != -1)
				array[i++] = Integer.parseInt (s.substring (p++,q).trim());
		}		

		return array;
	}

}


// Sensor canvas class

public class SensorCanvas extends Canvas {

	int close, medium, far, r;
	float scale;
	Dimension size;
	int[] sensordata;

	public SensorCanvas (int[] sensordata) {
		this.sensordata = new int[24];
		
		this.sensordata = sensordata;

		close = 500;
		medium = 2000;
		far = 5000;

		r = 25;
		scale = (float) 1.0;
		size = new Dimension (100,100);

		repaint();
	}

	public void update (int[] sensordata) {
		this.sensordata = sensordata;
		repaint ();
	}

	public void setLimits (int close, int medium, int far) {
		this.close = close;
		this.medium = medium;
		this.far = far;
	}

	public void setScale (float scale) {
		this.scale = scale;
	}

	public void setSize (Dimension size) {
		this.size = size;
	}

	public Dimension getPreferredSize () {
		return getMinimumSize();
	}

	public Dimension getMinimumSize () {
		return size;
	}

	public void paint (Graphics g) {

		Dimension s;
		Color color;
		int angle,i;
		
		s = getSize();

		g.clearRect (0,0,s.width,s.height);
		g.drawRect (0,0,s.width-1,s.height-1);
		g.fillOval ((int)(s.width/2 - r*scale-2),(int)(s.height/2 - r*scale-2), 
			(int)(2*r*scale+4),(int)(2*r*scale+4));		// robot

		for (angle = 0,i = 0;angle < 360; angle += 15,i++) {
			
			if (sensordata[i] < close) {
				color = Color.red;
				if (sensordata[i] < (int)(close/3))
					color = color.brighter();
				if (sensordata[i] > (int)(2*close/3))
					color = color.darker();
			}

			else
				if (sensordata[i] < medium) {
					color = Color.yellow;
					if (sensordata[i] < (int)(close + (medium-close)/3))
						color = color.brighter();
					if (sensordata[i] > (int)(close + (medium-close)*2/3))
						color = color.darker();
				}
				else {
					color = Color.green;
					if (sensordata[i] < (int)(medium+(far-medium)/3))
						color = color.brighter();
					if (sensordata[i] > (int)(medium+(far-medium)*2/3))
						color = color.darker();
				}
			
			g.setColor (color);
			g.fillArc ((int)(s.width/2 - r*scale),(int)(s.height/2 - r*scale), 
			(int)(2*r*scale),(int)(2*r*scale),angle+2,11);
		}

		g.setColor (Color.black);
		g.fillOval ((int)(s.width/2 - 2*r/3*scale-2),(int)(s.height/2 - 2*r/3*scale-2), 
			(int)(4*r/3*scale+4),(int)(4*r/3*scale+4));		// robot
	}

} // end of SensorCanvas


// Battery canvas class

public class BatteryCanvas extends Canvas {

	int min, red, yellow, green, max, voltage;
	float scale;
	Dimension size;

	public BatteryCanvas (int voltage) {
		this.voltage = voltage;
		min = 10000;
		red = 11000;
		yellow = 12000;
		green = 13000;
		max = 15000;
		scale = (float) 1.0;
		size = new Dimension (100,50);
		repaint();
	}

	public void setLimits (int min,int red, int yellow, int green, int max) {
		this.min = min;
		this.red = red;
		this.green = green;
		this.yellow = yellow;
		this.max = max;
		repaint();
	}

	public void update (int voltage) {
		this.voltage = voltage;
		repaint();
	}

	public void setScale (float scale) {
		this.scale = scale;
	}

	public void setSize (Dimension size) {
		this.size = size;
	}

	public Dimension getPreferredSize () {
		return getMinimumSize();
	}

	public Dimension getMinimumSize () {
		return size;
	}

	public void paint (Graphics g) {

		Dimension s;
		Color color;
		double angle;
		
		s = getSize();
		g.clearRect (0,0,s.width,s.height);
		g.setColor (Color.green);
		g.fillArc (0,0,s.width,s.height*2,180,-180);
		g.setColor (Color.yellow);
		g.fillArc (0,0,s.width,s.height*2,180,(int)
			(-((double)(yellow-min)/(max-min))*180));
		g.setColor (Color.red);
		g.fillArc (0,0,s.width,s.height*2,180,(int) 
			(-((double)(red-min)/(max-min))*180));
		g.setColor (Color.white);
		g.fillArc ((int)(10*scale),(int)(10*scale),(int) ((s.width-20)*scale),
			(int) ((s.height*2-20)*scale),0,180);
		g.setColor (Color.black);
		angle = Math.PI*(1-(double)(voltage-min)/(max-min));
		g.drawLine ((int) (s.width/2),s.height-1,
			(int) (s.width/2*(1+Math.cos(angle))-1),
			(int) (s.height-s.width/2*Math.sin(angle)-1));
	}	

} // end of BatteryCanvas


} // end of applet
