/* -----------------2/24/00 10:52AM------------------
 * SocketIO class
 *
 * Programmer: Srgio Roberto Gonsalves Tourino
 * Contact: tourino@graco.unb.br
 *
 * This class is used to define input and output streams
 * through a socket connection, and use them to transfer
 * string-like data. The conversion made between byte arrays
 * and string is for C language compatibility (ASCII characters).
 * --------------------------------------------------*/

import java.io.*;
import java.net.*;

public class SocketIO {

  BufferedInputStream in;               // input stream
  BufferedOutputStream out;             // output stream
  Socket socket;                        // socket connection
  String str;
  boolean connected = true;             // connection flag
  boolean flushIO;                      // flush flag
  byte[] inbyte = new byte[100];
  byte[] outbyte = new byte[100];

  /* -----------------2/24/00 10:43AM------------------
   * Default constructor defines flushed IO
   * --------------------------------------------------*/

  public SocketIO (String host, int port) throws UnknownHostException, IOException {

    this (host,port,true);              // Default is flushed IO
  }

  /* -----------------2/24/00 10:43AM------------------
   * Constructor to define the flushing behavior
   * --------------------------------------------------*/

  public SocketIO (String host, int port, boolean flushIO) throws 
	UnknownHostException, IOException {

    this.flushIO = flushIO;             // The user defines flushing
    socket = new Socket(host,port);
    out = new BufferedOutputStream(socket.getOutputStream());
    in = new BufferedInputStream(socket.getInputStream());
  }

   /* -----------------2/24/00 10:44AM------------------
    * Method for output strings as byte arrays
    * --------------------------------------------------*/

  public synchronized void send (String output) throws IOException {

    output = output + "\n";             // for C compatibility
    outbyte = output.getBytes();        // string->byte array
    out.write (outbyte,0,output.length());
    if (flushIO)
      out.flush();                    // flushes the stream
	}

  /* -----------------2/24/00 10:45AM------------------
   * Method for input byte arrays as string
   * --------------------------------------------------*/

  public synchronized String get() throws IOException {

    int size = in.read(inbyte);
    str = new String (inbyte);          // byte array->string
    str = str.substring (0,size);
		return str;
	}

  /* -----------------2/24/00 10:45AM------------------
   * Gets the handler for the input stream
   * --------------------------------------------------*/

	public InputStream getInputStream () {
		return in;
	}

  /* -----------------2/24/00 10:46AM------------------
   * Gets the handler for the output stream
   * --------------------------------------------------*/

	public OutputStream getOutputStream () {
		return out;
	}

  /* -----------------2/24/00 10:46AM------------------
   * Gets the handler for the socket connection
   * --------------------------------------------------*/

	public Socket getSocket () {
		return socket;
	}

  /* -----------------2/24/00 10:46AM------------------
   * Closes all the streams and sockets used
   * --------------------------------------------------*/

  public synchronized void close () throws IOException {

			in.close();
			out.close();
			socket.close();
	}

  /* -----------------2/24/00 10:47AM------------------
   * Default toString method for debugging
   * --------------------------------------------------*/

	public String toString () {
		return "Socket: "+socket.toString()+"\n"+"Input: "+in.toString()+"\n"+"Output: "+out.toString();
	}

}
