// <APPLET CODE=TestCanvas.class width=200 height=200></APPLET>

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.net.*;
import java.lang.*;

//
// Main class for control of the camera of the XR4000 robot
//

public class TestCanvas extends Applet {

	BatteryCanvas sensor;

// Init method for the applet. Initializes the applet and creates the panel and the
// buttons for its use.

public void init() {

	sensor = new BatteryCanvas (12000);

	add (sensor);
}

// Start method for the applet. Initiates the display of images from server

public void start () {

}

// Stop method for the applet. Stops the display of images.

public void stop () {

}

// Battery canvas class

public class BatteryCanvas extends Canvas {

	int min, red, yellow, green, max, voltage;
	float scale;
	Dimension size;

	public BatteryCanvas (int voltage) {
		this.voltage = voltage;
		min = 10000;
		red = 11000;
		yellow = 12000;
		green = 13000;
		max = 15000;
		scale = (float) 1.0;
		size = new Dimension (100,50);
		repaint();
	}

	public void setLimits (int min,int red, int yellow, int green, int max) {
		this.min = min;
		this.red = red;
		this.green = green;
		this.yellow = yellow;
		this.max = max;
		repaint();
	}

	public void update (int voltage) {
		this.voltage = voltage;
		repaint();
	}

	public void setScale (float scale) {
		this.scale = scale;
	}

	public void setSize (Dimension size) {
		this.size = size;
	}

	public Dimension getPreferredSize () {
		return getMinimumSize();
	}

	public Dimension getMinimumSize () {
		return size;
	}

	public void paint (Graphics g) {

		Dimension s;
		Color color;
		double angle;
		
		s = getSize();
		g.clearRect (0,0,s.width,s.height);
		g.setColor (Color.green);
		g.fillArc (0,0,s.width,s.height*2,180,-180);
		g.setColor (Color.yellow);
		g.fillArc (0,0,s.width,s.height*2,180,(int)
			(-((double)(yellow-min)/(max-min))*180));
		g.setColor (Color.red);
		g.fillArc (0,0,s.width,s.height*2,180,(int) 
			(-((double)(red-min)/(max-min))*180));
		g.setColor (Color.white);
		g.fillArc ((int)(10*scale),(int)(10*scale),(int) ((s.width-20)*scale),
			(int) ((s.height*2-20)*scale),0,180);
		g.setColor (Color.black);
		angle = Math.PI*(1-(double)(voltage-min)/(max-min));
		g.drawLine ((int) (s.width/2),s.height-1,
			(int) (s.width/2*(1+Math.cos(angle))-1),
			(int) (s.height-s.width/2*Math.sin(angle)-1));
	}	

} // end of BatteryCanvas


// Sensor canvas class

public class SensorCanvas extends Canvas {

	int close, medium, far, r;
	float scale;
	Dimension size;
	int[] sensordata;

	public SensorCanvas (int[] sensordata) {
		this.sensordata = new int[24];
		
		this.sensordata = sensordata;

		close = 500;
		medium = 2000;
		far = 5000;

		r = 25;
		scale = (float) 1.0;
		size = new Dimension (100,100);

		repaint();
	}

	public void update (int[] sensordata) {
		this.sensordata = sensordata;
		repaint ();
	}

	public void setLimits (int close, int medium, int far) {
		this.close = close;
		this.medium = medium;
		this.far = far;
	}

	public void setScale (float scale) {
		this.scale = scale;
	}

	public void setSize (Dimension size) {
		this.size = size;
	}

	public Dimension getPreferredSize () {
		return getMinimumSize();
	}

	public Dimension getMinimumSize () {
		return size;
	}

	public void paint (Graphics g) {

		Dimension s;
		Color color;
		int angle,i;
		
		s = getSize();

		g.clearRect (0,0,s.width,s.height);
		g.fillOval ((int)(s.width/2 - r*scale-2),(int)(s.height/2 - r*scale-2), 
			(int)(2*r*scale+4),(int)(2*r*scale+4));		// robot

		for (angle = 0,i = 0;angle < 360; angle += 15,i++) {
			
			if (sensordata[i] < close) {
				color = Color.red;
				if (sensordata[i] < (int)(close/3))
					color = color.brighter();
				if (sensordata[i] > (int)(2*close/3))
					color = color.darker();
			}

			else
				if (sensordata[i] < medium) {
					color = Color.yellow;
					if (sensordata[i] < (int)(close + (medium-close)/3))
						color = color.brighter();
					if (sensordata[i] > (int)(close + (medium-close)*2/3))
						color = color.darker();
				}
				else {
					color = Color.green;
					if (sensordata[i] < (int)(medium+(far-medium)/3))
						color = color.brighter();
					if (sensordata[i] > (int)(medium+(far-medium)*2/3))
						color = color.darker();
				}
			
			g.setColor (color);
			g.fillArc ((int)(s.width/2 - r*scale),(int)(s.height/2 - r*scale), 
			(int)(2*r*scale),(int)(2*r*scale),angle+2,11);
		}

		g.setColor (Color.black);
		g.fillOval ((int)(s.width/2 - 2*r/3*scale-2),(int)(s.height/2 - 2*r/3*scale-2), 
			(int)(4*r/3*scale+4),(int)(4*r/3*scale+4));		// robot
	}

} // end of SensorCanvas





// Canvas class

public class PanCanvas extends Canvas {

	int teta, dx, dy, bc, hc, bs, hs, r;	// robot's dimensions
	float scale, phi;
	Dimension size;

	public PanCanvas (int teta, int dx, int dy, int bc, int hc, int bs, int hs, int r, int 		phi) {

		this.teta = teta;
		this.dx = dx;
		this.dy = dy;
		this.bc = bc;
		this.hc = hc;
		this.bs = bs;
		this.hs = hs;
		this.r = r;
		this.phi = (float) Math.toRadians(phi);
		scale = (float) 1.0;
		size = new Dimension (200,200);
		repaint ();
	}

	public void setAngle (int teta) {

		this.teta = teta;
		repaint();
	}

	public void setScale (float s) {
		this.scale = s;
		repaint();
	}

	public void setSize (Dimension d) {
		size = d;
	}		

	Point fromCamera (Point P) {

		Point n = new Point ();
		double trad;

		trad = Math.toRadians ((double) teta);

		n.x = (int) (Math.cos(trad)*P.x - Math.sin(trad)*P.y - Math.cos(trad)*dx);
		n.y = (int) (Math.sin(trad)*P.x + Math.cos(trad)*P.y - Math.sin(trad)*dx + dy);

		n.x = (int) (n.x * scale);
		n.y = (int) (n.y * scale);

		return n;
	}

	Point fromPanTilt (Point P) {

		Point n = new Point ();
		double trad;

		trad = Math.toRadians ((double) teta);

		n.x = (int) (Math.cos(trad)*P.x - Math.sin(trad)*P.y);
		n.y = (int) (Math.sin(trad)*P.x + Math.cos(trad)*P.y + dy);

		n.x = (int) (n.x * scale);
		n.y = (int) (n.y * scale);

		return n;
	}


	public Dimension getPreferredSize() {

	        return getMinimumSize();
    	}

	public Dimension getMinimumSize() {

        	return size;
    	}


	public void paint (Graphics g) {

		Dimension s;
		Point A,B,C,D,E,F,G,H,M,N,O,P;
		Polygon camera, pantilt;

		camera = new Polygon();
		pantilt = new Polygon();

		s = getSize();

		g.clearRect (0,0,s.width,s.height);
		g.setColor (Color.black);
		g.drawOval ((int)(s.width/2 - r*scale),(int)(s.height/2 - r*scale), 
			(int)(2*r*scale),(int)(2*r*scale));		// robot
		
		A = new Point (fromCamera (new Point (-bc/2, hc/2)));
		B = new Point (fromCamera (new Point (bc/2, hc/2)));
		C = new Point (fromCamera (new Point (bc/2, -hc/2)));
		D = new Point (fromCamera (new Point (-bc/2, -hc/2)));

		E = new Point (fromPanTilt (new Point (-bs/2, hs/2)));
		F = new Point (fromPanTilt (new Point (bs/2, hs/2)));
		G = new Point (fromPanTilt (new Point (bs/2, -hs/2)));
		H = new Point (fromPanTilt (new Point (-bs/2, -hs/2)));

		M = new Point (fromCamera (new Point (0, hc/2)));
		N = new Point (fromCamera (new Point ((int)(-10*hc*Math.sin(phi)), 						(int)(hc/2+10*hc*Math.cos(phi)) )));
		O = new Point (fromCamera (new Point (0,21*hc/2)));
		P = new Point (fromCamera (new Point ((int)(10*hc*Math.sin(phi)),						(int)(hc/2+10*hc*Math.cos(phi)) )));

		camera.addPoint (s.width/2 + A.x, -A.y + s.height/2);
		camera.addPoint (s.width/2 + B.x, -B.y + s.height/2);
		camera.addPoint (s.width/2 + C.x, -C.y + s.height/2);
		camera.addPoint (s.width/2 + D.x, -D.y + s.height/2);

		pantilt.addPoint (s.width/2 + E.x, -E.y + s.height/2);
		pantilt.addPoint (s.width/2 + F.x, -F.y + s.height/2);
		pantilt.addPoint (s.width/2 + G.x, -G.y + s.height/2);
		pantilt.addPoint (s.width/2 + H.x, -H.y + s.height/2);

		g.setColor (Color.blue);
		g.fillPolygon (pantilt);
		g.setColor (Color.green);
		g.fillPolygon (camera);

		g.setColor (Color.red);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + N.x,s.height/2 - N.y);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + P.x,s.height/2 - P.y);
		g.setColor (Color.black);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + O.x,s.height/2 - O.y);

		g.drawRect (0,0,s.width-1,s.height-1);
	}
} // end of PanCanvas



public class TiltCanvas extends Canvas {

	int teta, dx, dy, bc, hc, bs, hs, r;	// robot's dimensions
	float scale, phi;
	Dimension size;

	public TiltCanvas (int teta, int dy, int bc, int hc, int bs, int hs, int phi) {

		this.teta = teta;
		this.dy = dy;
		this.bc = bc;
		this.hc = hc;
		this.bs = bs;
		this.hs = hs;
		this.phi = (float) Math.toRadians(phi);
		scale = (float) 1.0;
		size = new Dimension (200,200);
		repaint ();
	}

	public void setAngle (int teta) {

		this.teta = teta;
		repaint();
	}

	public void setScale (float s) {
		this.scale = s;
		repaint();
	}

	public void setSize (Dimension d) {
		size = d;
	}		

	Point fromPanTilt (Point P) {

		Point n = new Point ();
		double trad;

		trad = Math.toRadians ((double) teta);

		n.x = (int) (Math.cos(trad)*P.x - Math.sin(trad)*P.y);
		n.y = (int) (Math.sin(trad)*P.x + Math.cos(trad)*P.y + dy);

		n.x = (int) (n.x * scale);
		n.y = (int) (n.y * scale);

		return n;
	}


	public Dimension getPreferredSize() {

	        return getMinimumSize();
    	}

	public Dimension getMinimumSize() {

        	return size;
    	}


	public void paint (Graphics g) {

		Dimension s;
		Point A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P;
		Polygon camera, pantilt, base;

		base = new Polygon();
		camera = new Polygon();
		pantilt = new Polygon();

		s = getSize();

		g.clearRect (0,0,s.width,s.height);
		g.setColor (Color.black);
		
		A = new Point ((int)(-bs/2*scale), 0);
		B = new Point ((int)(bs/2*scale), 0);
		C = new Point ((int)(bs/2*scale),(int) (dy*scale));
		D = new Point ((int)(-bs/2*scale),(int) (dy*scale));

		E = new Point (fromPanTilt (new Point (-bs/2, 0)));
		F = new Point (fromPanTilt (new Point (bs/2, 0)));
		G = new Point (fromPanTilt (new Point (bs/2, hs)));
		H = new Point (fromPanTilt (new Point (-bs/2, hs)));

		I = new Point (fromPanTilt (new Point (-bc/2, hs)));
		J = new Point (fromPanTilt (new Point (bc/2, hs)));
		K = new Point (fromPanTilt (new Point (bc/2, hs+hc)));
		L = new Point (fromPanTilt (new Point (-bc/2, hs+hc)));


		M = new Point (fromPanTilt (new Point (bc/2, hs+hc/2)));
		N = new Point (fromPanTilt (new Point ((int)(10*bc*Math.cos(phi)), 						(int)(hs+hc/2+10*bc*Math.sin(phi)) )));
		O = new Point (fromPanTilt (new Point (21*bc/2,hs+hc/2)));
		P = new Point (fromPanTilt (new Point ((int)(10*bc*Math.cos(phi)),						(int)(hs+hc/2-10*bc*Math.sin(phi)) )));

		base.addPoint (s.width/2 + A.x, -A.y + s.height/2);
		base.addPoint (s.width/2 + B.x, -B.y + s.height/2);
		base.addPoint (s.width/2 + C.x, -C.y + s.height/2);
		base.addPoint (s.width/2 + D.x, -D.y + s.height/2);

		pantilt.addPoint (s.width/2 + E.x, -E.y + s.height/2);
		pantilt.addPoint (s.width/2 + F.x, -F.y + s.height/2);
		pantilt.addPoint (s.width/2 + G.x, -G.y + s.height/2);
		pantilt.addPoint (s.width/2 + H.x, -H.y + s.height/2);

		camera.addPoint (s.width/2 + I.x, -I.y + s.height/2);
		camera.addPoint (s.width/2 + J.x, -J.y + s.height/2);		
		camera.addPoint (s.width/2 + K.x, -K.y + s.height/2);
		camera.addPoint (s.width/2 + L.x, -L.y + s.height/2);

		g.setColor (Color.black);
		g.fillPolygon (base);
		g.setColor (Color.blue);
		g.fillPolygon (pantilt);
		g.setColor (Color.green);
		g.fillPolygon (camera);

		g.setColor (Color.red);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + N.x,s.height/2 - N.y);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + P.x,s.height/2 - P.y);
		g.setColor (Color.black);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + O.x,s.height/2 - O.y);

		g.drawRect (0,0,s.width-1,s.height-1);
	}
} // end of TiltCanvas



} // end of TestCanvas


