/*--------------------------------------------------------*/
/* Programa    : ADLP_10.C                                */
/* Funcao      : Computer Link ADLP 10 - PC versus IRB S3 */
/* Versao      : 1.0                                      */
/* Data        : Novembro/96                              */
/* Programador : Marcio Heidi Suguieda - GRACO/UnB        */
/* Compilador  : Borland C++ - Versao 3.1 - 1990-1992 (c) */
/* Linguagem   : C                                        */
/* Observacao  : ADLP = ABB DATA LINK PROTOCOL            */
/* -------------------------------------------------------*/

#include <bios.h>
#include <stdio.h>
#include <conio.h>
#include <time.h>
#include "adlp_10.h"

#define COM1       0
#define COM2       1
#define DATA_READY 0x100
#define SETTINGS   (_COM_CHR8 | _COM_STOP1 | _COM_EVENPARITY | _COM_9600)

#define TRUE  1
#define FALSE 0

#define ENQ     0x05   /* 'ENQUIRY' */
#define	ACK     0x06   /* 'POSITIVE ACKNOWLEDGEMENT' */
#define WACK    0x0E   /* 'POSITIVE ACKNOWLEDGEMENT and WAIT' */
#define RVI     0x0F   /* 'REVERSE INTERRUPT' */
#define NAK     0x15   /* 'NEGATIVE ACKNOWLEDGEMENT' */
#define DLE     0x10   /* 'DATA LINK ESCAPE' */
#define STXeven 0x02   /* 'START OF TEXT EVEN' = 0000 0010 */
#define STXodd  0x82   /* 'START OF TEXT ODD'  = 1000 0010 */
#define ETX     0x03   /* 'END OF TEXT' */
#define EOT     0x04   /* 'END OF TRANSMISSION' */

#define TOUTmaster 1   /* 'TIME OUT FOR MASTER' = 1000 ms = 1 segundo */
#define TOUTslave  3   /* 'TIME OUT FOR SLAVE' = 3000 ms = 3 segundos */
#define ATTEMPTS   4   /* Numero de tentativas de escrita/leitura */


int SerialCommPort;
char STXevenTX, STXevenRX, FirstSend, FirstReceive;

/* -- Funcoes que lidam com 'status' da porta serial e de inicializacao -- */

void InitADLP_10(void)
{
 STXevenTX = TRUE;
 STXevenRX = TRUE;
 FirstSend = TRUE;
 FirstReceive = TRUE;
}

char InitCommPort(int CommPort)
{
 InitADLP_10();
 SerialCommPort = CommPort;  /* 0 = 'COM1' e 1 = 'COM2' */
 switch (CommPort)
  {
   case 0: bioscom (_COM_INIT, SETTINGS, COM1);
	   return TRUE;
   case 1: bioscom (_COM_INIT, SETTINGS, COM2);
	   return TRUE;
   default: return FALSE;
  }
}

char DataReady(void)
{
 int Status;

 Status = bioscom(_COM_STATUS, 0, SerialCommPort);
 if (Status & DATA_READY)
  return TRUE;
  else return FALSE;
}

/* -- Funcoes que lidam com leitura e escrita de dados na porta serial -- */

char ReadByte(time_t TimeOut, char* TheByte)
{
 int SerialData;
 time_t TimeSec;

 TimeSec = time(NULL);
 do
  {
   if (DataReady() == TRUE)
    {
     SerialData = bioscom(_COM_RECEIVE, 0, SerialCommPort);
     if (!(SerialData & 0xFF00))
      {
       *TheByte = SerialData & 0xFF;
       return TRUE;
      }
      else return FALSE;
    }
  } while (time(NULL) - TimeSec <= TimeOut);
 return FALSE;
}

void WriteByte(char TheByte)
{
 bioscom(_COM_SEND, TheByte, SerialCommPort);
}

/* -- Funcoes que lidam com a transmissao de dados do protocolo ADLP 10 -- */

int TX_Phase1(void)          /* Se retorna -9: resposta desconhecida ! */
			     /* Se retorna -1: 'time out' ! */
			     /* Se retorna  0: fase 1 completada ! */
{
 char Byte_RX;

 WriteByte(ENQ);             /* Solicita comunicacao como mestre */
 STXevenTX = TRUE;           /* Se fase 1, inicia com 'STXeven' apos 'ENQ' */
 if (ReadByte(TOUTmaster, &Byte_RX) == TRUE)
  switch (Byte_RX)
   {
    case ACK: return 0;
    case RVI: return 0;      /* RVI sempre rejeitado neste modulo ! */
    case ENQ: if (ReadByte(TOUTmaster, &Byte_RX) == TRUE)
	       if (Byte_RX == ACK)
		return 0;
		else return -9;
	       else return -1;
			     /* ENQ simultaneo: busca ACK do robo ! */
    default: return -9;
   }
  else return -1;
}

int TX_Phase2(char* Telegram_TX)  /* Se retorna -9: resposta desconhecida ! */
				  /* Se retorna -2: 'NAK' recebido ! */
				  /* Se retorna -1: 'time out' ! */
				  /* Se retorna  0: fase 2 completada ! */
{
 int i, NOB;                 /* 'NOB = NUMBER OF BYTES' */
 char Byte_RX, STX, BCS;     /* 'BCS = BLOCK CHECK SUM' */

 /* O telegrama do protocolo ARAP vem com um maximo de  128 bytes, */
 /* logo poderia ser substituido apenas por NOB := Telegram_TX[1]; */
 NOB = Telegram_TX[0] * 0x100 + Telegram_TX[1];

 /* Verifica STX: se repeticao, mantem; caso contrario, troca ! */
 if (STXevenTX == TRUE)
  STX = STXeven;
  else STX = STXodd;

/* Calculo do BCS: telegrama ARAP ainda sem os 'DLEs' de controle */
 BCS = 0;
 for (i = 0; i < NOB; i++)
  BCS = BCS ^ Telegram_TX[i];
 BCS = BCS ^ ETX;

 /* Envia caracteres de controle de inicio do telegrama: 'DLE/STX' */
 WriteByte(DLE);
 WriteByte(STX);

 /* Envia telegrama com caracteres de controle: 'DLEs' */
 for (i = 0; i < NOB; i++)
  {
   if (Telegram_TX[i] == DLE)
    WriteByte(DLE);
   WriteByte(Telegram_TX[i]);
  }

 /* Envia caracteres de controle de fim do telegrama: 'DLE/ETX/BCS' */
 WriteByte(DLE);
 WriteByte(ETX);
 WriteByte(BCS);

 if (ReadByte(TOUTmaster, &Byte_RX) == TRUE)
  switch (Byte_RX)
   {
    case ACK: STXevenTX = !(STXevenTX);
	      return 0;	     /* Habilita envio de continuacao de telegrama */
    case RVI: STXevenTX = !(STXevenTX);
	      return 0;      /* RVI sempre rejeitado neste modulo ! */
    case NAK: return -2;
    default: return -9;
   }
  else return -1;
}

void TX_Phase3(void)
{
 WriteByte(EOT);             /* Termina a comunicacao como mestre */
}

/* -- Funcoes que lidam com a recepcao de dados do protocolo ADLP 10 -- */

int RX_Phase1(void)          /* Se retorna -9: resposta desconhecida ! */
			     /* Se retorna -1: 'time out' ! */
			     /* Se retorna  0: fase 1 completada ! */
{
 char Byte_RX;

 if (ReadByte(TOUTslave, &Byte_RX) == TRUE)
  if (Byte_RX == ENQ)
   {
    WriteByte(ACK);
    STXevenRX = TRUE;        /* Se fase 1, inicia com 'STXeven' */
    return 0;
   }
   else return -9;
  else return -1;
}

int RX_Phase2(char* Telegram_RX)  /* Se retorna -9: resposta desconhecida ! */
				  /* Se retorna -2: 'NAK' enviado ! */
				  /* Se retorna -1: 'time out' ! */
				  /* Se retorna  0: fase 2 completada ! */
				  /* Se retorna  2: repeticao de telegrama ! */
{
 int i;
 char Repetition, Byte_RX, BCS;   /* 'BCS = BLOCK CHECK SUM' */

 if (ReadByte(TOUTslave, &Byte_RX) == TRUE)
  if (Byte_RX == DLE)
   if (ReadByte(TOUTslave, &Byte_RX) == TRUE)
    switch (Byte_RX)
     {
      case STXeven: if (STXevenRX == TRUE)
		     Repetition = FALSE;
		     else Repetition = TRUE;
		    break;
      case STXodd: if (STXevenRX == FALSE)
		    Repetition = FALSE;
		    else Repetition = TRUE;
		   break;
      default: return -9;
     }
    else return -1;
   else return -9;
  else return -1;

 BCS = 0;
 i = 0;
 while (i < 129)
  {
   if (ReadByte(TOUTslave, &Byte_RX) == TRUE)
    if (Byte_RX != DLE)
     {
      Telegram_RX[i] = Byte_RX;
      BCS = BCS ^ Byte_RX;
     }
     else if (ReadByte(TOUTslave, &Byte_RX) == TRUE)
	   switch (Byte_RX)
	    {
	     case DLE: Telegram_RX[i] = Byte_RX;
		       BCS = BCS ^ Byte_RX;
		       break;
	     case ETX: BCS = BCS ^ ETX;
		       if (ReadByte(TOUTslave, &Byte_RX) == TRUE)
			if (Byte_RX == BCS)
			 if (Repetition == FALSE)
			  {
			   WriteByte(ACK);
			   STXevenRX = !(STXevenRX);
			   return 0;
			  }
			  else {
				WriteByte(ACK);
				return 2;
			       }
			 else {
			       WriteByte(NAK);
			       return -2;
			      }
			else return -1;
	     default: return -9;
	    }
	   else return -1;
    else return -1;
    i++;
  }
 return -9;
}

int RX_Phase3(void)          /* Se retorna -9: resposta desconhecida ! */
			     /* Se retorna -1: 'time out' ! */
			     /* Se retorna  0: fase 3 completada ! */
{
 char Byte_RX;

 if (ReadByte(TOUTslave, &Byte_RX) == TRUE)
  if (Byte_RX == EOT)
   return 0;
   else return -9;
  else return -1;
}

/* -- Funcoes principais de envio e recebimento de telegramas -- */

int SendTelegram(char* Telegram)
  /* Se retorna -2: erro na fase 2 ! */
  /* Se retorna -1: erro na fase 1 ! */
  /* Se retorna  0: telegrama enviado com a comunicacao concluida ! */
  /* Se retorna  1: telegrama enviado sem a comunicacao concluida ! */
{
 int Retry;
 char Send_OK, TX_EOT;

 /* Bit 3 do byte 5 = 'MLI - Message Length Indicator' */
 /* Se 0: tem continuacao;  e, se 1: ultimo telegrama  */
 if ((Telegram[5] & 8) == 0)
  TX_EOT = TRUE;
  else TX_EOT = FALSE;

 /* A fase 1 de transmissao soh eh realizada quando do primeiro telegrama */
 if (FirstSend == TRUE)
  if (TX_Phase1() == 0)
   Send_OK = TRUE;
   else {
	 Send_OK = FALSE;
	 Retry = 1;
	 do
	  {
	   Retry++;
	   //FlushCommPort;
	   if (TX_Phase1() == 0)
	    Send_OK = TRUE;
	  } while (Send_OK == FALSE && Retry < ATTEMPTS);
	 if (Send_OK == FALSE)
	  {
	   TX_Phase3();
	   return -1;
	  }
	}

 /* A fase 2 de transmissao eh realizada se nao for o primeiro telegrama */
 /* ou se for o primeiro telegrama com fase 1 bem sucedida               */
 if (FirstSend == FALSE || Send_OK == TRUE)
  if (TX_Phase2(Telegram) == 0)
   Send_OK = TRUE;
   else {
	 Send_OK = FALSE;
	 Retry = 1;
	 do
	  {
	   Retry++;
	   //FlushCommPort;
	   if (TX_Phase2(Telegram) == 0)
	    Send_OK = TRUE;
	  } while (Send_OK == FALSE && Retry < ATTEMPTS);
	 if (Send_OK == FALSE)
	  {
	   TX_Phase3();
	   FirstSend = TRUE;
	   return -2;
	  }
	}

 /* A fase 3 de transmissao soh eh realizada quando do ultimo telegrama */
 if (Send_OK == TRUE)
  {
   if (TX_EOT == TRUE)
    {
     TX_Phase3();
     FirstSend = TRUE;
     return 0;
    }
    else {
	  FirstSend = FALSE;
	  return 1;
	 }
  }

 return -9;                  /* Situacao impossivel (p/ evitar warning) */
}

int ReceiveTelegram(char* Telegram)
  /* Se retorna -3: erro na fase 3 ! */
  /* Se retorna -2: erro na fase 2 ! */
  /* Se retorna -1: erro na fase 1 ! */
  /* Se retorna  0: telegrama recebido com a comunicacao concluida ! */
  /* Se retorna  1: telegrama recebido sem a comunicacao concluida ! */
  /* Se retorna  2: repeticao de telegrama com a comunicacao concluida ! */
  /* Se retorna  3: repeticao de telegrama sem a comunicacao concluida ! */
  /* Se retorna  9: nenhum dado disponivel durante o tempo de 'time out' ! */
{
 int Retry;
 char Receive_OK = FALSE, Repetition = FALSE, RX_EOT;

 /* A fase 1 de recepcao soh eh realizada quando do primeiro telegrama */
 if (FirstReceive == TRUE)
  switch (RX_Phase1())
   {
    case 0: Receive_OK = TRUE;
	    break;
    case -1: return 9;
    default: //FlushCommPort;
	     return -1;
   }

 /* A fase 2 de recepcao eh realizada se nao for o primeiro telegrama */
 /* ou se for o primeiro telegrama com fase 1 bem sucedida            */
 if (FirstReceive == FALSE || Receive_OK == TRUE)
  switch (RX_Phase2(Telegram))
   {
    case 2: Repetition = TRUE;
	    if ((Telegram[5] & 8) == 0)
	     RX_EOT = TRUE;
	     else RX_EOT = FALSE;
	    Receive_OK = TRUE;
	    break;
    case 0: if ((Telegram[5] & 8) == 0)
	     RX_EOT = TRUE;
	     else RX_EOT = FALSE;
	    Receive_OK = TRUE;
	    break;
    case -1: FirstReceive = TRUE;
	     Receive_OK = FALSE;
	     return -2;
    case -2: Receive_OK = FALSE;
	     Retry = 1;
	     do
	      {
	       Retry++;
	       switch (RX_Phase2(Telegram))
		{
		 case 0: Receive_OK = TRUE;
			 break;
		 case -2: Receive_OK = FALSE;
			  break;
		 default: Retry = ATTEMPTS; /* Forca a saida do laco */
			  break;
		}
	      } while (Receive_OK == FALSE && Retry < ATTEMPTS);
	     if (Receive_OK == FALSE)
	      {
	       //FlushCommPort;
	       FirstReceive = TRUE;
	       return -2;
	      }
	     break;
    default: //FlushCommPort;
	     FirstReceive = TRUE;
	     Receive_OK = FALSE;
	     return -2;
   }

 /* A fase 3 de recepcao soh eh realizada quando do ultimo telegrama */
 if (Receive_OK == TRUE)
  {
   if (RX_EOT == TRUE)
    {
     FirstReceive = TRUE;
     if (RX_Phase3() == -9)
      {
       //FlushCommPort;
       return -3;
      }
      else if (Repetition == FALSE)
	    return 0;
	    else return 2;
    }
    else {
	  FirstReceive = FALSE;
	  if (Repetition == FALSE)
	   return 1;
	   else return 3;
	 }
  }

 return -9;                  /* Situacao impossivel (p/ evitar warning) */
}
