#include "cinematic.h"
#include <stdio.h>
#include <iostream>


/**
*
*	@title   Library C++ XR4000 - Class Cinematic
*	@author  Mrcio Moreira, Paulo Roberto Corra Dutra e Gabriel
*	Last update: 24/11/2002
*
**/

/* --------------------------------------------------------------
 * Function: 	Cinematic::Cinematic and Cinematic::~Cinematic
 * Purpose: 	The constructor and destructor of Cinematic class
 * Arguments:	none
 * Return:	an instance of Cinematic class
 * --------------------------------------------------------------*/
Cinematic::Cinematic()
{
	global_ref = false;
}
Cinematic::~Cinematic(){}


/* --------------------------------------------------------------
 * Function: 	Cinematic::ZeroRobot
 * Purpose: 	Make the atual position the global zero of Robot
 * Arguments:	none
 * Return:	none
 * --------------------------------------------------------------*/

void Cinematic::ZeroRobot(N_RobotState *state)
{
	Coord zeroing;
	global_ref = true;

	zeroing.x = 0;
	zeroing.y = 0;
	zeroing.rot = 0;

	SetPosition(state, zeroing, N_AXIS_POSITION_ABSOLUTE);
	SetOdometry(state, zeroing);


	global_ref = false;

}

/* --------------------------------------------------------------
 * Function: 	Cinematic::SetOdometry
 * Purpose: 	Correct the value of odometer
 * Arguments:	*state: the Robot State
 		coord: the correct coordinates
 * Return:	Error code : N_NO_ERROR to OK
 * --------------------------------------------------------------*/
int Cinematic::SetOdometry(N_RobotState *state, Coord coord)
{

	state->Integrator.DataActive = TRUE;
	state->Integrator.TimeStampActive = TRUE;
	state->Integrator.x = coord.x;
	state->Integrator.y = coord.y;
	state->Integrator.Rotation = coord.rot;

	return N_SetIntegratedConfiguration(1);

}

/* --------------------------------------------------------------
 * Function: 	Cinematic::GetOdometry
 * Purpose: 	Get the actual values of Robot's odometer
 * Arguments:	*state: the Robot State
 * Return:	Coord: the coordinates from odometer
 * --------------------------------------------------------------*/

Coord Cinematic::GetOdometry(N_RobotState *state)
{
	Coord coord;

	N_GetIntegratedConfiguration(1);

	coord.x = state->Integrator.x;
	coord.y = state->Integrator.y;
	coord.rot = state->Integrator.Rotation;

	return coord;

}


/* --------------------------------------------------------------
 * Function: 	Cinematic::Moving
 * Purpose: 	Verify if robot is moving now
 * Arguments:	*state: the Robot State
 * Return:	bool: true if robot is moving and false if not
 * --------------------------------------------------------------*/

bool Cinematic::Moving(N_RobotState *state)
{

   	N_GetAxes(1);

	if (state->AxisSet.Axis[N_XTRANSLATION].InProgress )
 	{
         		return true;
	}

	if (state->AxisSet.Axis[N_YTRANSLATION].InProgress )
	{
        		return true;
	}

	if (state->AxisSet.Axis[N_ROTATION].InProgress )
	{
        		return true;
	}

	return false;

}




/* --------------------------------------------------------------
 * Function: 	Cinematic::SetPosition
 * Purpose: 	Move robot to the desired position
 * Arguments:	*state: the Robot State
		pos: the cordinates of the desired position
		mode: the mode of motion [N_AXIS_POSITION_ABSOLUTE]
					 [N_AXIS_POSITION_RELATIVE]
 * Return:	none
 * --------------------------------------------------------------*/

void Cinematic::SetPosition(N_RobotState *state,Coord pos, char mode)
{

	state->AxisSet.Global = global_ref;   /* global reference frame   */

	state->AxisSet.Axis[N_XTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_ROTATION].DataActive = TRUE;

	//estudar a opcao de referencia - global ou relativa ao corpo do robo
	state->AxisSet.Axis[N_XTRANSLATION].Mode = mode; //N_AXIS_POSITION_ABSOLUTE;  /* move axis with respect to the absoluty zero*/
	state->AxisSet.Axis[N_YTRANSLATION].Mode = mode; //N_AXIS_POSITION_ABSOLUTE;  /* move axis with respect to the absoluty zero*/
	state->AxisSet.Axis[N_ROTATION].Mode = mode; //N_AXIS_POSITION_ABSOLUTE;  /* move with respect to the absoluty zero*/

	state->AxisSet.Axis[N_XTRANSLATION].DesiredPosition = pos.x; /*  unit: mm */
	state->AxisSet.Axis[N_YTRANSLATION].DesiredPosition = pos.y; /*  unit: mm */
	state->AxisSet.Axis[N_ROTATION].DesiredPosition = pos.rot;   /*  unit: mrad */


	state->AxisSet.Axis[N_XTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_ROTATION].Update = TRUE;

	N_SetAxes(1);

	while (Moving(state)) ;

}



/*****************************
Impementacao 2 do SetPosition
******************************/

void Cinematic::SetPosition(N_RobotState *state,Coord pos, Coord vel, Coord acc, char mode)
{

	state->AxisSet.Global = global_ref;   /* global reference frame   */

	state->AxisSet.Axis[N_XTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_ROTATION].DataActive = TRUE;

	state->AxisSet.Axis[N_XTRANSLATION].Mode = mode;
	state->AxisSet.Axis[N_YTRANSLATION].Mode = mode;
	state->AxisSet.Axis[N_ROTATION].Mode = mode;

	if (mode != N_AXIS_VELOCITY)
	{
		state->AxisSet.Axis[N_XTRANSLATION].DesiredPosition = pos.x; /*  unit: mm */
		state->AxisSet.Axis[N_YTRANSLATION].DesiredPosition = pos.y; /*  unit: mm */
		state->AxisSet.Axis[N_ROTATION].DesiredPosition = pos.rot;   /*  unit: mrad */
	}

	// Configurando a velocidade
	state->AxisSet.Axis[N_XTRANSLATION].DesiredSpeed = vel.x; /*  unit: mm/s */
	state->AxisSet.Axis[N_YTRANSLATION].DesiredSpeed = vel.y; /*  unit: mm/s */
	state->AxisSet.Axis[N_ROTATION].DesiredSpeed = vel.rot;   /*  unit: mrad/s */

	// Configurando a aceleracao
	state->AxisSet.Axis[N_XTRANSLATION].Acceleration = acc.x; /*  unit: mm/s/s */
    	state->AxisSet.Axis[N_YTRANSLATION].Acceleration = acc.y; /*  unit: mm/s/s */
	state->AxisSet.Axis[N_ROTATION].Acceleration = acc.rot;   /*  unit: mrad/s/s */

	state->AxisSet.Axis[N_XTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_ROTATION].Update = TRUE;

	// Movendo
	int error;
	if ( (error = N_SetAxes(1)) != N_NO_ERROR &&
	  error != N_AXES_NOT_READY)
	  {
		printf("Erro movendo o robo\n");
		exit(-1);
	  }


	// Fazendo alcancar o destino
	if (mode != N_AXIS_VELOCITY)
	{
		while (Moving(state));
	}

}





/* --------------------------------------------------------------
 * Function: 	Cinematic::SetVelocity
 * Purpose: 	Set the velocity to move Robot
 * Arguments:	*state: the Robot State
		vel:  the desired cordinates velocities
 * Return:	none
 * --------------------------------------------------------------*/
void Cinematic::SetVelocity(N_RobotState *state, Coord vel)
{

	state->AxisSet.Global=TRUE;   /* global axis mode   */
	state->AxisSet.Axis[N_XTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_ROTATION].DataActive = TRUE;

	state->AxisSet.Axis[N_XTRANSLATION].Mode = N_AXIS_VELOCITY;  /* constant velocity  */
	state->AxisSet.Axis[N_YTRANSLATION].Mode = N_AXIS_VELOCITY;  /* constant velocity  */
    	state->AxisSet.Axis[N_ROTATION].Mode = N_AXIS_VELOCITY;  /* constant velocity  */

	state->AxisSet.Axis[N_XTRANSLATION].DesiredSpeed = vel.x; /*  unit: mm/s */
    	state->AxisSet.Axis[N_YTRANSLATION].DesiredSpeed = vel.y; /*  unit: mm/s */
	state->AxisSet.Axis[N_ROTATION].DesiredSpeed = vel.rot;   /*  unit: mrad/s */

	state->AxisSet.Axis[N_XTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_ROTATION].Update = TRUE;


	int error;
	if ((error = N_SetAxes(1)) != N_NO_ERROR && error != N_AXES_NOT_READY)
    	{
		printf("erro\n");
	}
	printf("Movimentou \n");



}

/* --------------------------------------------------------------
 * Function: 	Cinematic::SetAccel
 * Purpose: 	Set the acceleration to move Robot
 * Arguments:	*state: the Robot State
		acc:  the desired cordinates acceleration
 * Return:	none
 * --------------------------------------------------------------*/
void Cinematic::SetAccel(N_RobotState *state, Coord acc)
{

	state->AxisSet.Global=TRUE;   /* global axis mode   */

	state->AxisSet.Axis[N_XTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].DataActive = TRUE;
	state->AxisSet.Axis[N_ROTATION].DataActive = TRUE;

	state->AxisSet.Axis[N_XTRANSLATION].Mode = N_AXIS_ACCELERATION;  /* constant acceleration  */
	state->AxisSet.Axis[N_YTRANSLATION].Mode = N_AXIS_ACCELERATION;  /* constant acceleration  */
    	state->AxisSet.Axis[N_ROTATION].Mode = N_AXIS_ACCELERATION;  /* constant acceleration  */

	state->AxisSet.Axis[N_XTRANSLATION].Acceleration = acc.x; /*  unit: mm/s/s */
    	state->AxisSet.Axis[N_YTRANSLATION].Acceleration = acc.y; /*  unit: mm/s/s */
	state->AxisSet.Axis[N_ROTATION].Acceleration = acc.rot;   /*  unit: mrad/s/s */

	state->AxisSet.Axis[N_XTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_YTRANSLATION].Update = TRUE;
	state->AxisSet.Axis[N_ROTATION].Update = TRUE;

	N_SetAxes(1);   /* moves the robot*/


}


/* --------------------------------------------------------------
 * Function: 	Cinematic::GetPosition
 * Purpose: 	Get the actual position of Robot
 * Arguments:	*state: the Robot State
 * Return:	Coord: the cordinates of actual position
 * --------------------------------------------------------------*/

Coord Cinematic::GetPosition(N_RobotState *state)
{

	Coord pos;

	N_GetAxes(1);

	pos.x = state->AxisSet.Axis[N_XTRANSLATION].ActualPosition;
	pos.y = state->AxisSet.Axis[N_YTRANSLATION].ActualPosition;
	pos.rot = state->AxisSet.Axis[N_ROTATION].ActualPosition;

	return pos;


}

/* --------------------------------------------------------------
 * Function: 	Cinematic::GetVelocity
 * Purpose: 	Get the actual velocity of Robot
 * Arguments:	*state: the Robot State
 * Return:	Coord: the actual velocity in all Axes
 * --------------------------------------------------------------*/
Coord Cinematic::GetVelocity(N_RobotState *state)
{
	Coord vel;

	N_GetAxes(1);

	vel.x = state->AxisSet.Axis[N_XTRANSLATION].ActualVelocity;
	vel.y = state->AxisSet.Axis[N_YTRANSLATION].ActualVelocity;
	vel.rot = state->AxisSet.Axis[N_ROTATION].ActualVelocity;

	return vel;

}

/* --------------------------------------------------------------
 * Function: 	Cinematic::GetAccel
 * Purpose: 	Get the actual acceleration of Robot
 * Arguments:	*state: the Robot State
 * Return:	Coord: the actual acceleration in all Axes
 * --------------------------------------------------------------*/
Coord Cinematic::GetAccel(N_RobotState *state)
{
	Coord acc;

	N_GetAxes(1);

	acc.x = state->AxisSet.Axis[N_XTRANSLATION].Acceleration;
	acc.y = state->AxisSet.Axis[N_YTRANSLATION].Acceleration;
	acc.rot = state->AxisSet.Axis[N_ROTATION].Acceleration;

	return acc;

}
