#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <sys/time.h>


#include "nomad.h"
#include "util.h"
#include "sensonar.h"
#include "senir.h"
#include "mapa.h"
#include "cinematic.h"

#include "jni.h"
#include "navmap_ConectorJava.h"

#define CELULA_VAZIA    0
#define CELULA_OCUPADA  1
#define CELULA_SEMINFO -1
#define RAIO_ROBO 300 // mm


#define m 8500 // translacao inicial em X
#define n 8500 // translacao inicial em Y

// Variaveis Globais;


Sensor sensorValido[24];
SenIr ir;
SenSonar sonar;
Coord pos;
Cinematic cinematic;
Nomad nomad;
Mapa mapaEstatico;
struct N_RobotState* state;

#define ANG 0.261799387799

void paint(long dist, int sensor_index)
{

	int col, lin;
	float i,dx,dy;

	pos = cinematic.GetOdometry(state);

	for (i=sensor_index*ANG; i<sensor_index*ANG+ANG; i+=0.017453)
	{

		//obstaculo em relacao ao robo
		dx = (dist+RAIO_ROBO)*cos(i);
		dy = (dist+RAIO_ROBO)*sin(i);

		//Transformacao de cordenadas em relacao ao referencial global
                //m e n translacao em x e y respec. do referencial do robo em relacao ao referencial global

		col = (int)((cos(pos.rot/1000)*dx - sin(pos.rot/1000)*dy + m + pos.x)/100);
		lin = (int)((sin(pos.rot/1000)*dx + cos(pos.rot/1000)*dy + n + pos.y)/100);

		col= 0 >? col;
		lin= 0 >? lin;

		col = 299 <? col;
		lin = 299 <? lin;

		mapaEstatico.insEnvMap(col, lin, 1);

	}

}

void AtualizaSensores()
{
	long result;
	for (int i=0; i< 24; i++)
	{
		result = sonar.Read(state,i); // <? ir.Read(state,i);
		paint(result, i);
	}
}


//---------------------------------------------------------------------------
/*
 * Class:     navmap_ConectorJava
 * Method:    finishNomad
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_navmap_ConectorJava_finishNomad
  (JNIEnv *env, jobject)
{
	state = nomad.GetRobotState();
	sonar.StopAll(state);
	if (nomad.Disconnect() == N_NO_ERROR)
		cout << "Desconexo OK" << endl;
	else
		cout << "Erro ao Desconectar-se do Robo"  << endl;

}

//---------------------------------------------------------------------------

/*
 * Class:     navmap_ConectorJava
 * Method:    startNomad
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_navmap_ConectorJava_startNomad
  (JNIEnv *env, jobject)
{
	cout << "Conectando com o Robo" << endl;
	if (nomad.Connect() == N_NO_ERROR)
		cout << "Conexo OK" << endl;
	else
		cout << "Erro ao Conectar com o Robo"  << endl;

	nomad.Speak("Conectado!");

	state = nomad.GetRobotState();

	// Zerando Robo
	cinematic.ZeroRobot(state);

   	if (ir.Start(state) != N_NO_ERROR )
    		cout << "Erro ao iniciar o Sensor Infra Vermelho" << endl;


    	int fo[8] = {0,1,2,3,4,5,6,7};


    	for (int k = 0; k < 6; k++)
		if ( sonar.Start(state,k, fo, 3 ) != N_NO_ERROR )
			cout << "Erro ao iniciar o Sonar " << k << endl;

}


//---------------------------------------------------------------------------
/*
 * Class:     navmap_ConectorJava
 * Method:    getMap
 * Signature: ()[I
 */
JNIEXPORT jintArray JNICALL Java_navmap_ConectorJava_getMap
  (JNIEnv *env, jobject)
{
        jint* pix;
	int index = 0;
	jintArray result;

        state = nomad.GetRobotState();
        AtualizaSensores();

	pix = (jint*)malloc(300*300*4);
	if (pix == 0) printf("Nao ha memoria suficiente \n");


	for (int i=0; i<300; i--)
		for (int j=0; j<300; j++)
		{
			pix[index++] = mapaEstatico.envMap[i][j];
		}

	result = env->NewIntArray(300*300);
	env->SetIntArrayRegion(result, 0, 300*300, pix);

        return result;

}


/*
 * Class:     navmap_ConectorJava
 * Method:    getBatteryStatus
 * Signature: ()[F
 */
JNIEXPORT jfloatArray JNICALL Java_navmap_ConectorJava_getBatteryStatus
  (JNIEnv *env, jobject)
{
	jfloatArray result;
	long baterias[4];
	
	state = nomad.GetRobotState();
	for (int i=0; i < 4; i++)
		baterias[i] = nomad.GetBattery(state, i);

	env->SetFloatArrayRegion(result, 0, 4, (jfloat*)baterias);

	return result;

}





