package navmap;

import java.io.File;
import javax.swing.filechooser.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * <p>Title: NavMap</p>
 * <p>Description: Navegao por Mapeamento</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Graco - UnB</p>
 * @author Paulo Roberto Correa Dutra & Marcio Moreira de Sousa
 * @version 1.0
 */

public class MenuPrincipal extends JMenuBar {

  private EventoAcao listener;

  private JMenu MenuSistema = new JMenu();
  private JMenuItem MenuSistema_Conectar = new JMenuItem();
  private JMenuItem MenuSistema_Desconectar = new JMenuItem();
  private JMenuItem MenuSistema_Sair = new JMenuItem();
  private JMenu MenuMapa = new JMenu();
  private JMenuItem MenuMapa_Abrir = new JMenuItem();
  private JMenuItem MenuMapa_Novo = new JMenuItem();
  private JMenuItem MenuMapa_Salvar = new JMenuItem();
  private JMenu MenuAjuda = new JMenu();
  private JMenuItem MenuAjuda_Sobre = new JMenuItem();
  private JMenuItem MenuAjuda_Conteudo = new JMenuItem();
  private JFileChooser fc = new JFileChooser();

  public MenuPrincipal() {

   listener = new EventoAcao();

   /* Escolher Arquivo para carregar Mapa */
   MapaFilter mapaFilter = new MapaFilter();
   fc.setFileFilter(mapaFilter);
   fc.setCurrentDirectory(new File("."));

   /* Menu Sistema */
   MenuSistema.setMnemonic('S');
   MenuSistema.setText("Sistema");

   MenuSistema_Conectar.setMnemonic('C');
   MenuSistema_Conectar.setText("Conectar");
   MenuSistema_Conectar.addActionListener(listener);

   MenuSistema_Desconectar.setMnemonic('D');
   MenuSistema_Desconectar.setText("Desconectar");
   MenuSistema_Desconectar.addActionListener(listener);

   MenuSistema_Sair.setMnemonic('S');
   MenuSistema_Sair.setText("Sair");
   MenuSistema_Sair.addActionListener(listener);

   MenuSistema.add(MenuSistema_Conectar);
   MenuSistema.add(MenuSistema_Desconectar);
   MenuSistema.add(MenuSistema_Sair);


   /* Menu Mapa */
   MenuMapa.setMnemonic('M');
   MenuMapa.setText("Mapas");

   MenuMapa_Abrir.setMnemonic('A');
   MenuMapa_Abrir.setText("Abrir");
   MenuMapa_Abrir.addActionListener(listener);

   MenuMapa_Novo.setMnemonic('N');
   MenuMapa_Novo.setText("Novo");
   MenuMapa_Novo.addActionListener(listener);

   MenuMapa_Salvar.setMnemonic('S');
   MenuMapa_Salvar.setText("Salvar");
   MenuMapa_Salvar.addActionListener(listener);

   MenuMapa.add(MenuMapa_Abrir);
   MenuMapa.add(MenuMapa_Novo);
   MenuMapa.add(MenuMapa_Salvar);


   /* Menu Ajuda */
   MenuAjuda.setMnemonic('A');
   MenuAjuda.setText("Ajuda");

   MenuAjuda_Sobre.setMnemonic('S');
   MenuAjuda_Sobre.setText("Sobre");
   MenuAjuda_Sobre.addActionListener(listener);

   MenuAjuda_Conteudo.setMnemonic('C');
   MenuAjuda_Conteudo.setText("Contedo");
   MenuAjuda_Conteudo.addActionListener(listener);

   MenuAjuda.add(MenuAjuda_Sobre);
   MenuAjuda.addSeparator();
   MenuAjuda.add(MenuAjuda_Conteudo);

   this.add(MenuSistema);
   this.add(MenuMapa);
   this.add(MenuAjuda);

  }


  /**
   *
   * <p>Title: NavMap</p>
   * <p>Description: Navegao por Mapeamento</p>
   * <p>Copyright: Copyright (c) 2002</p>
   * <p>Company: Graco - UnB</p>
   * @author Paulo Roberto Correa Dutra & Marcio Moreira de Sousa
   * @version 1.0
   * Classe de Tratamento de Eventos
   */
  class EventoAcao implements ActionListener {

    public void actionPerformed(ActionEvent event) {
      if (event.getSource().equals(MenuSistema_Sair)) Sair();
      if (event.getSource().equals(MenuSistema_Conectar)) Conectar();
      if (event.getSource().equals(MenuSistema_Desconectar)) Desconectar();
      if (event.getSource().equals(MenuMapa_Abrir)) Abrir();
      if (event.getSource().equals(MenuMapa_Novo)) Novo();
      if (event.getSource().equals(MenuMapa_Salvar)) Salvar();
      if (event.getSource().equals(MenuAjuda_Sobre)) Sobre();
      if (event.getSource().equals(MenuAjuda_Conteudo)) Conteudo();
    }
  }


  /**
   * Trata Evento Conectar do Menu Sistema
   */
   private void Conectar(){
     System.out.println("Menu Conectar");
   }

  /**
   * Trata Evento Desconectar do Menu Sistema
   */
   private void Desconectar(){
     System.out.println("Menu Desconectar");
   }

  /**
   * Trata Evento Sair do Menu Sistema
   */
   private void Sair(){
     System.out.println("Sair");
     System.exit(0);
   }

  /**
   * Trata Evento Abrir do Menu Mapa
   */
   private void Abrir() {
     System.out.println("Abrir");
     fc.showOpenDialog(this);
   }

  /***
   * Trata Evento Novo do Menu Mapa
   */
   private void Novo(){
     System.out.println("Novo");
   }

  /***
   * Trata Evento Salvar do Menu Mapa
   */
   private void Salvar(){
     System.out.println("Salvar");
     fc.showSaveDialog(this);
   }

  /***
   * Trata Evento Sobre do Menu Ajuda
   */
   private void Sobre(){
     System.out.println("Sobre");
   }

  /***
   * Trata Evento Contedo do Menu Ajuda
   */
   private void Conteudo(){
     System.out.println("Conteudo");
   }


   /**
    * Classe que cria um filtro para os arquivos de Mapa (.map)
    * <p>Title: NavMap</p>
    * <p>Description: Navegao por Mapeamento</p>
    * <p>Copyright: Copyright (c) 2002</p>
    * <p>Company: Graco - UnB</p>
    * @author unascribed
    * @version 1.0
    */

   class MapaFilter extends FileFilter {

    // Aceita todos os diretrios e os arquivos de Mapas
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }

        String extension = Utils.getExtension(f);
        if (extension != null) {
            if (extension.equals(Utils.mapa) ){
                return true;
            } else {
                return false;
            }
            }

        return false;
    }

    // The description of this filter
    public String getDescription() {
        return "Arquivos de Mapas do NavMap v1.0";
    }
   }

} // End class MenuPrincipal
