#include "nomad.h"

/* --------------------------------------------------------------
 * Function: 	Nomad::Nomad and Nomad::~Nomad
 * Purpose: 	The constructor and destructor of Nomad class
 * Arguments:	none
 * Return:	an instance of Nomad class
 * -------------------------------------------------------------- */
Nomad::Nomad(){}
Nomad::~Nomad(){}


/* --------------------------------------------------------------
 * Function: 	Nomad::Connect
 * Purpose: 	Connect to Robot
 * Arguments:	none
 * Return:	Error code of connect action
 		N_NO_ERROR to OK
 * --------------------------------------------------------------*/
int Nomad::Connect()
{
	N_InitializeClient(HOST,PORT);
	return N_ConnectRobot(1);
}


/* --------------------------------------------------------------
 * Function: 	Nomad::Disconnect
 * Purpose: 	Disconnect from Robot
 * Arguments:	none
 * Return:	Error code of disconnect action
 		N_NO_ERROR to OK
 * --------------------------------------------------------------*/
int Nomad::Disconnect()
{
	return N_DisconnectRobot(1);
}


/* --------------------------------------------------------------
 * Function: 	Nomad::Speak
 * Purpose: 	Make the Nomad speak a text
 * Arguments:	*text : the text that Nomad will be speak
 * Return:	Error code of speak action
 		N_NO_ERROR to OK
 * --------------------------------------------------------------*/

int Nomad::Speak(const char *text)
{
	return N_Speak(1, text);
}

/* --------------------------------------------------------------
 * Function: 	Nomad::GetBettary
 * Purpose: 	Get the status of Robot's batteries
 * Arguments:	*state: Robot State
 		idBattery: index of battery to read [0-3]
 * Return:	long: the actual Voltage of a given Battery index
 * --------------------------------------------------------------*/

long Nomad::GetBattery(struct N_RobotState *state, int idBattery)
{
	N_GetBattery(1);
	return state->BatterySet.Battery[idBattery].Voltage/100;
}


/* --------------------------------------------------------------
 * Function: 	Nomad::GetState
 * Purpose: 	Get the actual state of Robot
 * Arguments:	none
 * Return:	struct N_RobotState*: the struct of the actual 
 		state of Robot

 * --------------------------------------------------------------*/
struct N_RobotState* Nomad::GetRobotState()
{
	return N_GetRobotState(1);
}
