#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <asm/io.h>
#include <time.h>

#define DATA 	0x378		/* LPT 1 */
#define STATUS	DATA+1
#define CONTROL	DATA+2

		void 	initport (void);
		void 	endport (void);
		void 	send (int n);
		void 	sendc (int n);
		int	get  (void);
		void 	waiting (int t);


void initport (void)

{
int porta = DATA;

	if (ioperm (porta,3,1))
		{
		printf ("ioperm\n");
		exit (1);
		}
}

void endport (void)

{
int porta = DATA;

	if (ioperm (porta,3,0))
		{
		printf ("ioperm\n");
		exit (1);
		}
}

void send (int n)

{
int porta = DATA;

	outb (n,porta);
}

void sendc (int n)

{
int porta = DATA;

	outb (n,porta+2);
}

int get2 (void)

{
int porta = DATA;

	outb (0x01^0x0B,porta+2);
	return inb (porta+2);
}

int get (void)

{
int porta = DATA;

	return inb (porta+1);
}

void waiting (int t)

{
	usleep (t*1000);
}


int motor1_s [4] = {0x01,0x02,0x04,0x08},
    motor2_s [4] = {0x10,0x20,0x40,0x80},
    motor1_d [4] = {0x09,0x03,0x06,0x0C},
    motor2_d [4] = {0x90,0x30,0x60,0xC0};

int	numero,intervalo,
        tipo_excitacao,
	excitacao [4],
	ultima_excitacao;
float	precisao;

void	set (int n);		/* define o motor usado */
void 	precision (float p);	/* define a precisao do motor */
void 	sdelay (int tempo);	/* define o intervalo entre passos */
int     getsensor (int id);	/* le o valor do sensor especificado */
int 	turn (int angle,int nsensor);	/* gira angulo ate sensor */
void	wait (int time);	/* espera tempo excitado */
void	sstep (void);		/* define excitacao simples */
void	dstep (void);		/* define excitacao dupla */
void    off (void);		/* desliga a excitacao do motor */

void set (int n)

{	numero = n;
/*	if (numero == 1)
		ultima_excitacao = 0xF0 & ultima_excitacao;
	if (numero == 2)
		ultima_excitacao = 0x0F & ultima_excitacao;
*/
	if (tipo_excitacao)
	   dstep ();
	else
	   sstep ();
}

void sstep (void)

{
	if (numero == -1) {
		excitacao [0] = motor1_s [0] | motor2_s [3];
		excitacao [1] = motor1_s [1] | motor2_s [2];
		excitacao [2] = motor1_s [2] | motor2_s [1];
		excitacao [3] = motor1_s [3] | motor2_s [0]; }
	if (numero == 0) {
		excitacao [0] = motor1_s [0] | motor2_s [0];
		excitacao [1] = motor1_s [1] | motor2_s [1];
		excitacao [2] = motor1_s [2] | motor2_s [2];
		excitacao [3] = motor1_s [3] | motor2_s [3]; }
	if (numero == 1) {
		excitacao [0] = motor1_s [0];
		excitacao [1] = motor1_s [1];
		excitacao [2] = motor1_s [2];
		excitacao [3] = motor1_s [3]; }
	if (numero == 2) {
		excitacao [0] = motor2_s [0];
		excitacao [1] = motor2_s [1];
		excitacao [2] = motor2_s [2];
		excitacao [3] = motor2_s [3]; }
	tipo_excitacao = 0;
}

void dstep (void)

{
	if (numero == -1) {
		excitacao [0] = motor1_d [0] | motor2_d [3];
		excitacao [1] = motor1_d [1] | motor2_d [2];
		excitacao [2] = motor1_d [2] | motor2_d [1];
		excitacao [3] = motor1_d [3] | motor2_d [0];  }
	if (numero == 0) {
		excitacao [0] = motor1_d [0] | motor2_d [0];
		excitacao [1] = motor1_d [1] | motor2_d [1];
		excitacao [2] = motor1_d [2] | motor2_d [2];
		excitacao [3] = motor1_d [3] | motor2_d [3];  }
	if (numero == 1) {
		excitacao [0] = motor1_d [0];
		excitacao [1] = motor1_d [1];
		excitacao [2] = motor1_d [2];
		excitacao [3] = motor1_d [3]; }
	if (numero == 2) {
		excitacao [0] = motor2_d [0];
		excitacao [1] = motor2_d [1];
		excitacao [2] = motor2_d [2];
		excitacao [3] = motor2_d [3]; }
	tipo_excitacao = 1;
}


void off (void)

{
	ultima_excitacao = 0x00;
	send (0x00);
}

void precision (float p)

{
	precisao = p;
}

void sdelay (int tempo)

{
	intervalo = tempo;
}

/*

Esta rotina possui passo de 7.2 graus, pois efetua 4 passos de uma vez...

void turn (int angle)

{
	int ciclos,girando;
	class Interface porta;

	ciclos = (int) (angle / precisao);
	if (ciclos > 0) {
		for (girando = 0;girando < ciclos;girando++) {
			porta.send (excitacao [0]);
			porta.waiting (intervalo);
			porta.send (excitacao [1]);
			porta.waiting (intervalo);
			porta.send (excitacao [2]);
			porta.waiting (intervalo);
			porta.send (excitacao [3]);
			porta.waiting (intervalo);
			}
		ultima_excitacao = excitacao [3];  // grava a ultima excitacao
		}
	else {
		for (girando = ciclos;girando < 0;girando++) {
			porta.send (excitacao [3]);
			porta.waiting (intervalo);
			porta.send (excitacao [2]);
			porta.waiting (intervalo);
			porta.send (excitacao [1]);
			porta.waiting (intervalo);
			porta.send (excitacao [0]);
			porta.waiting (intervalo);
			}
		ultima_excitacao = excitacao [0];
		};
}
*/

/*  Le o valor da porta de entrada e compara com o valor dado em id */

int getsensor (int id)

{
int valor;

	valor = get ();
	printf ("%d\n",valor);
	return (valor & id);
}


int turn (int angle,int nsensor)

{
	int ciclos,sensor,temp,altera;
	int sensibilidade = 1;
	int girando = 0;

	altera = 0;
	sensor = getsensor (nsensor);
	ciclos = (int) (angle / precisao) + girando;

		/* + girando pois girando pode ser != 0... */

	if (ciclos > 0) {
		for (;girando < ciclos;girando++) {
			send (excitacao [girando % 4]);

		/* girando % 4 e' um numero entre 0 e 3... */

			waiting (intervalo);
			temp = getsensor (nsensor);
			if (temp != sensor)
				altera++;
			else
				altera = 0;
			if (altera == sensibilidade) {
				return temp;
				}
			sensor = temp;
			}
		ultima_excitacao = excitacao [girando % 4];  /* grava a ultima excitacao */
		girando = girando % 4;		/* atualiza o valor de girando */
		}
	else {
		for (;girando < (-ciclos);girando++) {
			send (excitacao [3 - girando % 4]);
			waiting (intervalo);
			temp = getsensor (nsensor);
			if (temp != sensor)
				altera++;
			else
				altera = 0;
			if (altera == sensibilidade) {
				return temp;
				}
			sensor = temp;
			}
		ultima_excitacao = excitacao [3 - girando % 4];
		girando = girando % 4;
		};
	return 0;
}

void wait (int time)

{
	send (ultima_excitacao);		/* envia a ultima excitacao */
						/* usada no motor */
	waiting (time);
}

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

void main (void)

{
int sensor,d;
long t1,t2;

d = 10;

initport ();
set (-1);
sdelay (100);
precision (1.8);
sstep ();

sensor = turn (1000,0);

off ();
endport ();
}

