// <APPLET CODE=ImageControl.class width=400 height=200></APPLET>

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.net.*;
import java.io.*;
import java.lang.*;

//
// Main class for control of the camera of the XR4000 robot
//

public class ImageControl extends Applet {

static String[]
  stringButtons = {"UL","Up","UR","Left","Center","Right","DL","Down","DR","Move","Change"},
  initSetup     = {"1000","400","0","0"};

static int[]
  maxLimits = {11000, 3000, 11000, 2400},
  minLimits = {100, 100, -11000, -3600};

Panel
  controlPanel,
  canvasPanel,
  centerPanel;

Button[]
  controlButtons = new Button[11];

Label
  controlLabel;

URL
  server;

SocketIO
  connection;

TextField[]
  setupText = new TextField[4];

PanCanvas
  panCanvas;

TiltCanvas
  tiltCanvas;

int
  pan = 0, 
  tilt = 0,		// absolute location of the camera
  panchange = 1000,
  tiltchange = 400;	// change values for the pantilt

boolean
  inUse = false;

// Init method for the applet. Initializes the applet and creates the panel and the
// buttons for its use.

public void init() {

  controlLabel = new Label ("Connecting to server...");

  try {
  	connection = new SocketIO (getCodeBase().getHost(),8086); // opens the connection with the camera's remote server
	controlLabel.setText ("Server connected.");
  }
  catch (UnknownHostException e) {
	System.err.println ("Error in URL of camera server.");
	controlLabel.setText ("Error in URL of camera server.");
	inUse = true;
  }
  catch (IOException e) {
	System.err.println ("Error connecting camera server.");
	controlLabel.setText ("Error connecting camera server.");
	inUse = true;
  }

  controlPanel = new Panel (new GridLayout (7,3));

  panCanvas = new PanCanvas (pan,10,15,5,10,25,5,40,30);
  tiltCanvas = new TiltCanvas (tilt,15,10,5,5,15,30);

  Dimension sizeCanvas = new Dimension (100,100);
  panCanvas.setSize (sizeCanvas);
  tiltCanvas.setSize (sizeCanvas);
  panCanvas.setScale ((float) 0.5);
  tiltCanvas.setScale ((float) 0.5);

  int i;

  for (i=0;i<4;i++) {			// creates all textfields
	setupText[i] = new TextField (initSetup[i]);
	setupText[i].addTextListener (new EventText (i));
  }

  for (i=0;i<9;i++) {                   // creates all buttons
    controlButtons[i]=new Button (stringButtons[i]);
    controlButtons[i].addActionListener (new EventButton (i));
    controlPanel.add (controlButtons[i]);
  }

  controlButtons[9]= new Button (stringButtons[9]);
  controlButtons[9].addActionListener (new EventButton (9));
  controlButtons[10]= new Button (stringButtons[10]);
  controlButtons[10].addActionListener (new EventButton (10));

  controlPanel.add (new Label (""));
  controlPanel.add (new Label ("Absolute"));
  controlPanel.add (new Label ("Change"));
  controlPanel.add (new Label ("Pan"));
  controlPanel.add (setupText[2]);
  controlPanel.add (setupText[0]);
  controlPanel.add (new Label ("Tilt"));
  controlPanel.add (setupText[3]);
  controlPanel.add (setupText[1]);
  controlPanel.add (new Label (""));
  controlPanel.add (controlButtons[9]);
  controlPanel.add (controlButtons[10]);

  canvasPanel = new Panel (new GridLayout (2,1));
  canvasPanel.add (panCanvas);
  canvasPanel.add (tiltCanvas);

  centerPanel = new Panel (new GridLayout (1,2));
  centerPanel.add (controlPanel);
  centerPanel.add (canvasPanel);

  setLayout (new BorderLayout ());
  add (centerPanel,BorderLayout.CENTER);
  add (controlLabel,BorderLayout.SOUTH);

}

// Destroy method for the applet. Stops the socket connection with the servers.

public void destroy () {
  try {
	connection.send("direction=center, start, goodbye");	// closing the remote server
  	connection.close();
  } catch (IOException e) {
	System.err.println ("Error closing connection with camera server.");
	controlLabel.setText ("Error closing connection.");
  }
}


// Class used to catch the events created by the buttons.

class EventButton implements ActionListener {

	int index;

  EventButton (int index) {
		this.index = index;
  }

  public void actionPerformed (ActionEvent e) {
	String command;
	int i;

	switch (index) {
		case 0:
		case 3:
		case 6:
			pan += panchange;
			break;
		case 2:
		case 5:
		case 8:
			pan -= panchange;
			break;
	}
	switch (index) {
		case 0:
		case 1:
		case 2:
			tilt += tiltchange;
			break;
		case 6:
		case 7:
		case 8:
			tilt -= tiltchange;
			break;
		case 4:
			pan = 0;
			tilt= 0;
			break;
		case 9:
			pan = Integer.parseInt (setupText[2].getText());
			tilt = Integer.parseInt (setupText[3].getText());
			break;
		case 10:
			panchange = Integer.parseInt (setupText[0].getText());
			tiltchange = Integer.parseInt (setupText[1].getText());
			break;
	}

	if (index < 9) {
		setupText[2].setText(pan+"");
		setupText[3].setText(tilt+"");
	}

	if (pan < minLimits[2])
		pan = minLimits[2];
	if (pan > maxLimits[2])
		pan = maxLimits[2];
	if (tilt < minLimits[3])
		tilt = minLimits[3];
	if (tilt > maxLimits[3])
		tilt = maxLimits[3];

	panCanvas.setAngle ((int) (pan*0.012857));
	tiltCanvas.setAngle ((int) (tilt*0.012857));

	command = "xabsolute="+setupText[2].getText()+",yabsolute="+setupText[3].getText()+", start";

      	try {
        	connection.send (command);
		System.err.println ("Comando: "+command);
      	}
      	catch(Exception exception) {
        	System.err.println ("Error connecting server. Command: " + command);
		controlLabel.setText ("Error: Server in use.");
		inUse = true;
		exception.printStackTrace();
      }

	if (inUse) {
		for (i=0;i<11;i++)
			controlButtons[i].setEnabled (false);
	}
  }

}

class EventText implements TextListener {

	int index;

	EventText (int index) {
		this.index = index;
	}

	public void textValueChanged (TextEvent e) {

		int textvalue = Integer.parseInt (setupText[index].getText());

		if (textvalue > maxLimits[index])
			setupText[index].setText(maxLimits[index]+"");
		if (textvalue < minLimits[index])
			setupText[index].setText(minLimits[index]+"");
	}

}


// Canvas class

public class PanCanvas extends Canvas {

	int teta, dx, dy, bc, hc, bs, hs, r;	// robot's dimensions
	float scale,phi;
	Dimension size;

	public PanCanvas (int teta, int dx, int dy, int bc, int hc, int bs, int hs, int r, int 		phi) {

		this.teta = teta;
		this.dx = dx;
		this.dy = dy;
		this.bc = bc;
		this.hc = hc;
		this.bs = bs;
		this.hs = hs;
		this.r = r;
		this.phi = (float) MyMath.toRadians((double) phi);
		scale = (float) 1.0;
		size = new Dimension (200,200);
		repaint ();
	}

	public void setAngle (int teta) {

		this.teta = teta;
		repaint();
	}

	public void setScale (float s) {
		this.scale = s;
		repaint();
	}

	public void setSize (Dimension d) {
		size = d;
	}		

	Point fromCamera (Point P) {

		Point n = new Point ();
		double trad;

		trad = MyMath.toRadians ((double) teta);

		n.x = (int) (Math.cos(trad)*P.x - Math.sin(trad)*P.y - Math.cos(trad)*dx);
		n.y = (int) (Math.sin(trad)*P.x + Math.cos(trad)*P.y - Math.sin(trad)*dx + dy);

		n.x = (int) (n.x * scale);
		n.y = (int) (n.y * scale);

		return n;
	}

	Point fromPanTilt (Point P) {

		Point n = new Point ();
		double trad;

		trad = MyMath.toRadians ((double) teta);

		n.x = (int) (Math.cos(trad)*P.x - Math.sin(trad)*P.y);
		n.y = (int) (Math.sin(trad)*P.x + Math.cos(trad)*P.y + dy);

		n.x = (int) (n.x * scale);
		n.y = (int) (n.y * scale);

		return n;
	}


	public Dimension getPreferredSize() {

	        return getMinimumSize();
    	}

	public Dimension getMinimumSize() {

        	return size;
    	}


	public void paint (Graphics g) {

		Dimension s;
		Point A,B,C,D,E,F,G,H,M,N,O,P;
		Polygon camera, pantilt;

		camera = new Polygon();
		pantilt = new Polygon();

		s = getSize();

		g.clearRect (0,0,s.width,s.height);
		g.setColor (Color.black);
		g.drawOval ((int)(s.width/2 - r*scale),(int)(s.height/2 - r*scale), 
			(int)(2*r*scale),(int)(2*r*scale));		// robot
		
		A = new Point (fromCamera (new Point (-bc/2, hc/2)));
		B = new Point (fromCamera (new Point (bc/2, hc/2)));
		C = new Point (fromCamera (new Point (bc/2, -hc/2)));
		D = new Point (fromCamera (new Point (-bc/2, -hc/2)));

		E = new Point (fromPanTilt (new Point (-bs/2, hs/2)));
		F = new Point (fromPanTilt (new Point (bs/2, hs/2)));
		G = new Point (fromPanTilt (new Point (bs/2, -hs/2)));
		H = new Point (fromPanTilt (new Point (-bs/2, -hs/2)));

		M = new Point (fromCamera (new Point (0, hc/2)));
		N = new Point (fromCamera (new Point ((int)(-10*hc*Math.sin(phi)), 						(int)(hc/2+10*hc*Math.cos(phi)) )));
		O = new Point (fromCamera (new Point (0,21*hc/2)));
		P = new Point (fromCamera (new Point ((int)(10*hc*Math.sin(phi)),						(int)(hc/2+10*hc*Math.cos(phi)) )));

		camera.addPoint (s.width/2 + A.x, -A.y + s.height/2);
		camera.addPoint (s.width/2 + B.x, -B.y + s.height/2);
		camera.addPoint (s.width/2 + C.x, -C.y + s.height/2);
		camera.addPoint (s.width/2 + D.x, -D.y + s.height/2);

		pantilt.addPoint (s.width/2 + E.x, -E.y + s.height/2);
		pantilt.addPoint (s.width/2 + F.x, -F.y + s.height/2);
		pantilt.addPoint (s.width/2 + G.x, -G.y + s.height/2);
		pantilt.addPoint (s.width/2 + H.x, -H.y + s.height/2);

		g.setColor (Color.blue);
		g.fillPolygon (pantilt);
		g.setColor (Color.green);
		g.fillPolygon (camera);

		g.setColor (Color.red);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + N.x,s.height/2 - N.y);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + P.x,s.height/2 - P.y);
		g.setColor (Color.black);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + O.x,s.height/2 - O.y);

		g.drawRect (0,0,s.width-1,s.height-1);
	}
} // end of PanCanvas



public class TiltCanvas extends Canvas {

	int teta, dx, dy, bc, hc, bs, hs, r;	// robot's dimensions
	float scale, phi;
	Dimension size;

	public TiltCanvas (int teta, int dy, int bc, int hc, int bs, int hs, int phi) {

		this.teta = teta;
		this.dy = dy;
		this.bc = bc;
		this.hc = hc;
		this.bs = bs;
		this.hs = hs;
		this.phi = (float) MyMath.toRadians((double) phi);
		scale = (float) 1.0;
		size = new Dimension (200,200);
		repaint ();
	}

	public void setAngle (int teta) {

		this.teta = teta;
		repaint();
	}

	public void setScale (float s) {
		this.scale = s;
		repaint();
	}

	public void setSize (Dimension d) {
		size = d;
	}		

	Point fromPanTilt (Point P) {

		Point n = new Point ();
		double trad;

		trad = MyMath.toRadians ((double) teta);

		n.x = (int) (Math.cos(trad)*P.x - Math.sin(trad)*P.y);
		n.y = (int) (Math.sin(trad)*P.x + Math.cos(trad)*P.y + dy);

		n.x = (int) (n.x * scale);
		n.y = (int) (n.y * scale);

		return n;
	}


	public Dimension getPreferredSize() {

	        return getMinimumSize();
    	}

	public Dimension getMinimumSize() {

        	return size;
    	}


	public void paint (Graphics g) {

		Dimension s;
		Point A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P;
		Polygon camera, pantilt, base;

		base = new Polygon();
		camera = new Polygon();
		pantilt = new Polygon();

		s = getSize();

		g.clearRect (0,0,s.width,s.height);
		g.setColor (Color.black);
		
		A = new Point ((int)(-bs/2*scale), 0);
		B = new Point ((int)(bs/2*scale), 0);
		C = new Point ((int)(bs/2*scale),(int) (dy*scale));
		D = new Point ((int)(-bs/2*scale),(int) (dy*scale));

		E = new Point (fromPanTilt (new Point (-bs/2, 0)));
		F = new Point (fromPanTilt (new Point (bs/2, 0)));
		G = new Point (fromPanTilt (new Point (bs/2, hs)));
		H = new Point (fromPanTilt (new Point (-bs/2, hs)));

		I = new Point (fromPanTilt (new Point (-bc/2, hs)));
		J = new Point (fromPanTilt (new Point (bc/2, hs)));
		K = new Point (fromPanTilt (new Point (bc/2, hs+hc)));
		L = new Point (fromPanTilt (new Point (-bc/2, hs+hc)));


		M = new Point (fromPanTilt (new Point (bc/2, hs+hc/2)));
		N = new Point (fromPanTilt (new Point ((int)(10*bc*Math.cos(phi)), 						(int)(hs+hc/2+10*bc*Math.sin(phi)) )));
		O = new Point (fromPanTilt (new Point (21*bc/2,hs+hc/2)));
		P = new Point (fromPanTilt (new Point ((int)(10*bc*Math.cos(phi)),						(int)(hs+hc/2-10*bc*Math.sin(phi)) )));

		base.addPoint (s.width/2 + A.x, -A.y + s.height/2);
		base.addPoint (s.width/2 + B.x, -B.y + s.height/2);
		base.addPoint (s.width/2 + C.x, -C.y + s.height/2);
		base.addPoint (s.width/2 + D.x, -D.y + s.height/2);

		pantilt.addPoint (s.width/2 + E.x, -E.y + s.height/2);
		pantilt.addPoint (s.width/2 + F.x, -F.y + s.height/2);
		pantilt.addPoint (s.width/2 + G.x, -G.y + s.height/2);
		pantilt.addPoint (s.width/2 + H.x, -H.y + s.height/2);

		camera.addPoint (s.width/2 + I.x, -I.y + s.height/2);
		camera.addPoint (s.width/2 + J.x, -J.y + s.height/2);		
		camera.addPoint (s.width/2 + K.x, -K.y + s.height/2);
		camera.addPoint (s.width/2 + L.x, -L.y + s.height/2);

		g.setColor (Color.black);
		g.fillPolygon (base);
		g.setColor (Color.blue);
		g.fillPolygon (pantilt);
		g.setColor (Color.green);
		g.fillPolygon (camera);

		g.setColor (Color.red);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + N.x,s.height/2 - N.y);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + P.x,s.height/2 - P.y);
		g.setColor (Color.black);
		g.drawLine (s.width/2 + M.x,s.height/2 - M.y,s.width/2 + O.x,s.height/2 - O.y);

		g.drawRect (0,0,s.width-1,s.height-1);
	}
} // end of TiltCanvas

} // end of applet
