/* Ram - Mobile Autonomous Robot Program Controller

	Programmer: Srgio Roberto Gonsalves Tourino
	Last update: 26/03/1999

	Contacts by email: tourino@graco.unb.br
*/

/************************************************************************/
/*				"Include" files															*/
/************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <asm/io.h>

/************************************************************************/
/*			Definitions and Variables													*/
/************************************************************************/

/* Port definitions (default) */

#define DATA 		0x3BC		/* LPT 1 */
#define STATUS		DATA+1
#define CONTROL	DATA+2

/* Excitation type definitions */

#define E_SIMPLES		0		/* Single step excitation */
#define E_DUPLA		1	  	/* Double step excitation */

/* Motors definitions */

#define MOTOR_A_SO  -1				/* Runs both motors in opposite directions */
#define MOTOR_A_MS	0				/* Runs both motors in same direction */
#define MOTOR_VER	   1				/* Runs only first motor */
#define MOTOR_HOR    2				/* Runs only second motor */
#define MOTOR_PRECISION    1.8   /* Motor's resolution */

/* Sensors definitions */

/* Speeds definitions */

#define SLOW          30
#define MEDIUM        20
#define FAST          10

/* Routine's return values */

#define OK		 0
#define ERRO	-1

/* Global Variables */

char    confname[80] = "ram.conf";      /* configuration file */
int     data_port;                      /* parallel port address */

/**************************************************************************/
/*			Interface Motor-Paralell Port               						     */
/**************************************************************************/

/* You must have the root access on Unix to use the following implementation */
/* Alternatively you can set the file's permition with:
	chmod 4755 file
	...logged as root	to use this program without the root access. */

void InitPort (int dataport);      /* Opens access to the paralell port */
void EndPort (void);               /* Closes access to the paralell port */
void send (int n);                 /* Sends the value to the DATA port */
void sendc (int n);                /* Sends the value to the CONTROL port */
int get (void);                    /* Gets data from STATUS port */
int getcontrol (int value);        /* Gets data from CONTROL port */
void waiting (int t);              /* Sleeps the machine */

int porta;			/* global variable for the parallel port */

void InitPort (int dataport)

/* opens access for the parallel port */

{
	porta = dataport;

	if (ioperm (porta,3,1))
		{
		perror ("ioperm");
		exit (1);
		}
}

void EndPort (void)

/* closes access for parallel port */

{
	if (ioperm (porta,3,0))
		{
		perror ("ioperm");
		exit (1);
		}
}


void send (int n)

/* sends data for the data byte in parallel port */

{
	outb (n,porta);
}

void sendc (int n)

/* sends data for the control byte in parallel port */

{
	outb (n,porta+2);
}

int get (void)

/* gets data from status byte in parallel port */

{
/*	return inb (porta+1);*/
			return 0;
}

int getcontrol (int value)

/* gets data from control byte in parallel port */

{
/* outb (porta+2,value);
	return inb (porta+2); */
 	return 0;
}


void waiting (int t)

/* makes a t milisseconds interruption */

{
/*	usleep (t*1000);*/
}

/************************************************************************/
/*				Motor controls																*/
/************************************************************************/

int motor1_s [4] = {0x01,0x02,0x04,0x08},
	 motor2_s [4] = {0x10,0x20,0x40,0x80},
	 motor1_d [4] = {0x09,0x03,0x06,0x0C},
	 motor2_d [4] = {0x90,0x30,0x60,0xC0};

int	numero,intervalo,
		  tipo_excitacao,
	excitacao [4],
	ultima_excitacao;
float	precisao;

void	set (int n);			/* defines the motor to be used */
void 	precision (float p);	/* defines motor's precision in degrees */
void 	sdelay (int tempo);	/* defines excitation delay */
int   getsensor (int id);	/* reads data from status port */
int 	turn (int angle,int nsensor);	/* turns stepper motor */
void	wait (int time);		/* stops motor at a specified time */
void	sstep (void);			/* defines single step */
void	dstep (void);			/* defines double step */
void  off (void);				/* turn off the motors */

void set (int n)

{	numero = n;


	if (numero == 1)
		ultima_excitacao = 0xF0 & ultima_excitacao;
	if (numero == 2)
		ultima_excitacao = 0x0F & ultima_excitacao;


	if (tipo_excitacao)
		dstep ();
	else
		sstep ();
}

void sstep (void)

{
	if (numero == -1) {
		excitacao [0] = motor1_s [0] | motor2_s [3];
		excitacao [1] = motor1_s [1] | motor2_s [2];
		excitacao [2] = motor1_s [2] | motor2_s [1];
		excitacao [3] = motor1_s [3] | motor2_s [0]; }
	if (numero == 0) {
		excitacao [0] = motor1_s [0] | motor2_s [0];
		excitacao [1] = motor1_s [1] | motor2_s [1];
		excitacao [2] = motor1_s [2] | motor2_s [2];
		excitacao [3] = motor1_s [3] | motor2_s [3]; }
	if (numero == 1) {
		excitacao [0] = motor1_s [0] | ultima_excitacao;
		excitacao [1] = motor1_s [1] | ultima_excitacao;
		excitacao [2] = motor1_s [2] | ultima_excitacao;
		excitacao [3] = motor1_s [3] | ultima_excitacao; }
	if (numero == 2) {
		excitacao [0] = motor2_s [0] | ultima_excitacao;
		excitacao [1] = motor2_s [1] | ultima_excitacao;
		excitacao [2] = motor2_s [2] | ultima_excitacao;
		excitacao [3] = motor2_s [3] | ultima_excitacao; }
	tipo_excitacao = E_SIMPLES;
}

void dstep (void)

{
	if (numero == -1) {
		excitacao [0] = motor1_d [0] | motor2_d [3];
		excitacao [1] = motor1_d [1] | motor2_d [2];
		excitacao [2] = motor1_d [2] | motor2_d [1];
		excitacao [3] = motor1_d [3] | motor2_d [0];  }
	if (numero == 0) {
		excitacao [0] = motor1_d [0] | motor2_d [0];
		excitacao [1] = motor1_d [1] | motor2_d [1];
		excitacao [2] = motor1_d [2] | motor2_d [2];
		excitacao [3] = motor1_d [3] | motor2_d [3];  }
	if (numero == 1) {
		excitacao [0] = motor1_d [0] | ultima_excitacao;
		excitacao [1] = motor1_d [1] | ultima_excitacao;
		excitacao [2] = motor1_d [2] | ultima_excitacao;
		excitacao [3] = motor1_d [3] | ultima_excitacao; }
	if (numero == 2) {
		excitacao [0] = motor2_d [0] | ultima_excitacao;
		excitacao [1] = motor2_d [1] | ultima_excitacao;
		excitacao [2] = motor2_d [2] | ultima_excitacao;
		excitacao [3] = motor2_d [3] | ultima_excitacao;}
	tipo_excitacao = E_DUPLA;
}


void off (void)

{
	ultima_excitacao = 0x00;
	send (0x00);
}

void precision (float p)

{
	precisao = p;
}

void sdelay (int tempo)

{
	intervalo = tempo;
}


/*  Reads the status port */

int getsensor (int id)

{
int valor;

	valor = get ();
	printf ("%d\n",valor);
	return (valor & id);
}


int turn (int angle,int nsensor)

{
	int ciclos,sensor,temp,altera;
	int sensibilidade = 1;
	int girando = 0;

	altera = 0;
	sensor = getsensor (nsensor);
	ciclos = (int) (angle / precisao) + girando;

		/* + girando pois girando pode ser != 0... */

	if (ciclos > 0) {
		for (;girando < ciclos;girando++) {
			send (excitacao [girando % 4]);

		/* girando % 4 e' um numero entre 0 e 3... */

			waiting (intervalo);
			temp = getsensor (nsensor);
			if (temp != sensor)
				altera++;
			else
				altera = 0;
			if (altera == sensibilidade) {
				return girando * precisao;
				}
			sensor = temp;
			}
		ultima_excitacao = excitacao [girando % 4];  /* grava a ultima excitacao */
		girando = girando % 4;		/* atualiza o valor de girando */
		}
	else {
		for (;girando < (-ciclos);girando++) {
			send (excitacao [3 - girando % 4]);
			waiting (intervalo);
			temp = getsensor (nsensor);
			if (temp != sensor)
				altera++;
			else
				altera = 0;
			if (altera == sensibilidade) {
				return girando * precisao;
				}
			sensor = temp;
			}
		ultima_excitacao = excitacao [3 - girando % 4];
		girando = girando % 4;
		};
	return angle;
}

void wait (int time)

{
	send (ultima_excitacao);		/* envia a ultima excitacao */
						/* usada no motor */
	waiting (time);
}

/************************************************************************/
/*				Generic Routines															*/
/************************************************************************/

void InitMotors (void);			/* Initializes motors */
void EndMotors (void);			/* Stops motors */

void InitMotors (void)

{
	InitPort (DATA);
	off ();
	precision (MOTOR_PRECISION);
	sdelay (MEDIUM);
	sstep ();
}

void EndMotors (void)

{
	off ();
	EndPort ();
}

int main (int argc,char *argv [])

{
	InitMotors ();
	set (MOTOR_A_MS);
	sdelay (FAST);
	dstep ();
	turn (360,0);
	turn (-360,0);
	set (MOTOR_A_SO);
	turn (360,0);
	turn (-360,0);
	EndMotors ();
	return 0;
}


