#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define FALSE   (0==1)
#define TRUE    (0==0)
#define VERT_CHANGE	400
#define HORI_CHANGE	1000
#define MAX_X		11000
#define MIN_X		-11000
#define MAX_Y		2400
#define MIN_Y		-3600

/******************************************************************************

CAMSERVER.C - Robot's camera server through the INETD main server
 Sergio Roberto Gonsalves Tourino - tourino@graco.unb.br - 11/02/2000

Before using this program, edit the 'inetd.conf' and the 'services' files:

INETD.CONF:
 camserver  stream  tcp     nowait  root /home/tourino/projeto/socket/servnet servnet

SERVICES:
 camserver  8086/tcp

After these modifications, restart the inetd server:
 kill -1 <PID do inetd - ver com ps>

*******************************************************************************/


/*-----------------2/29/00 5:17PM-------------------
 * This main routine parses commands from stdin, which
 * corresponds to a socket connection from a remote client
 * connected to this server. Its outputs are sent to the
 * stdout stream, back to the client machine. The local
 * tests are made with the stderr stream. This program is
 * to be used with the INETD main server.
 * --------------------------------------------------*/

int main (void) {

    const char delimiters[] = "= ,";    /* delimiters for the commands */
    char command[] = "direction=front, distance=500, nothing important, direction=left, goodbye";
    char *token, parameters[10], commandline[100];
    char direction[10];                 /* direction for the movement */
    char xabsolute[5], yabsolute[5];	/* absolute values for movement */
    char xchange[4], ychange[4];	/* change values for movement */
    int open_connection = TRUE;
    int x = 0,y = 0;
    int result, msg_id;
    int vert_change = VERT_CHANGE, hori_change = HORI_CHANGE;

    FILE *connection, *lock;

    /* verify the lock file */

    lock = fopen ("/home/tourino/camserver.lock","r");

    if (lock == NULL) { 		/* the lock does not exist */
	lock = fopen ("/home/tourino/camserver.lock","w+");
	fclose (lock);
    }
    else {				/* the lock exists */
	fclose (lock);
	fprintf (stderr,"Lock file found. Program aborted.\n");
	exit (0);			/* closes the program */
    }

    connection = fopen ("/home/tourino/camserver.out","w+"); /* opening file */
    msg_id = init_message();				/* starting IPC */

    while (open_connection) {           /* loop until 'open_connection' is false */

        fscanf (stdin,"%s",command);    /* reads commands from stdin */
	fprintf (connection,"%s\n",command);	/* writes to file */
	fflush (connection);			/* it's better... */

        token = strtok(command,delimiters);  /* strtok parses the commands from 'command' variable */

        while ((token != NULL) && open_connection) {

/*	    fprintf (stdout,"%s\n",token);	/* echoing back */
/*	    fflush (stdout);			/* it's better... */

	    if (strcmp(token,"framerate") == 0) {
		send_message (msg_id,1,strtok(NULL,delimiters)); 
	    }

            if (strcmp(token,"direction") == 0) {
                strcpy (direction,strtok(NULL,delimiters));
/*              fprintf (stderr,"I will walk to %s.\n",direction); */
            }

	    if (strcmp(token,"xabsolute") == 0) {
		strcpy (xabsolute,strtok(NULL,delimiters));
/*		fprintf (stderr,"Going to xabsolute=%s\n",xabsolute); */
	    }

	    if (strcmp(token,"yabsolute") == 0) {
		strcpy (yabsolute,strtok(NULL,delimiters));
/*		fprintf (stderr,"Going to yabsolute=%s\n",yabsolute); */
	    }

	    if (strcmp(token,"xchange") == 0) {
		strcpy (xchange,strtok(NULL,delimiters));
/*		fprintf (stderr,"Setup: xchange=%s\n",xchange); */
		if (strlen(xchange) > 0)
			hori_change = atoi (xchange);			
	    }

	    if (strcmp(token,"ychange") == 0) {
		strcpy (ychange,strtok(NULL,delimiters));
/*		fprintf (stderr,"Setup ychange=%s\n",ychange); */
		if (strlen(ychange) > 0)
			vert_change = atoi (ychange);
	    }

            if (strcmp(token,"start") == 0) {
                /* Here comes the commands for the control of the robot.
                 * This command MUST only work with all the variables (speed, acceleration, ...)
                 * adjusted accordingly. */

		if (strcmp (direction,"zero") == 0) {
			x = 0;
			y = 0;
		}

		if (strcmp (direction,"center") == 0) {
			x = 0;
			y = 0;
		}

		if (strcmp (direction,"up") == 0)
			if (y+vert_change <= MAX_Y)
				y += vert_change;
		if (strcmp (direction,"down") == 0)
			if (y-vert_change >= MIN_Y)
				y -= vert_change;

		if (strcmp (direction,"left") == 0)
			if (x+hori_change >= MIN_X)
				x += hori_change;
		if (strcmp (direction,"right") == 0)
			if (x-hori_change <= MAX_X)
				x -= HORI_CHANGE;

		if (strcmp (direction,"upleft") == 0) {
			if (x+hori_change >= MIN_X)
				x += hori_change;
			if (y+vert_change <= MAX_Y)
				y += vert_change;
		}

		if (strcmp (direction,"downleft") == 0) {
			if (x+hori_change >= MIN_X)
				x += hori_change;
			if (y-vert_change >= MIN_Y)
				y -= vert_change;
		}

		if (strcmp (direction,"upright") == 0) {
			if (x-hori_change <= MAX_X)
				x -= hori_change;
			if (y+vert_change <= MAX_Y)
				y += vert_change;
		}
	
		if (strcmp (direction,"downright") == 0) {
			if (x-hori_change <= MAX_X)
				x -= hori_change;
			if (y-vert_change >= MIN_Y)
				y -= vert_change;
		}
		
		if ((strlen (xabsolute) > 0) && (strlen (yabsolute) > 0)) {
			if ((atoi(xabsolute) > MIN_X) && (atoi(xabsolute)
			< MAX_X) && (atoi(yabsolute) > MIN_Y) && 
			(atoi(yabsolute) < MAX_Y)) {
				sprintf (parameters,"%s %s",xabsolute,yabsolute);
				x = atoi (xabsolute);
				y = atoi (yabsolute);
			}
			else {
				sprintf (xabsolute,"%d",x);
				sprintf (yabsolute,"%d",y);
				strcpy (parameters,xabsolute);
				strcat (parameters," ");
				strcat (parameters,yabsolute);
			}
		}
		else
			sprintf (parameters,"%d %d",x,y);
		strcpy (commandline,"/usr/local/robot-devices/dp_pantilt/pt ");
		strcat (commandline,parameters);
/*		fprintf (stderr,"%s\n",commandline); */
		result = system (commandline);
		strcpy (xabsolute,"");		/* erasing the old values... */
		strcpy (yabsolute,"");
		strcpy (direction,"");
            }

            if (strcmp(token,"goodbye") == 0) {
                open_connection = FALSE;  /* terminates all of the loops */
            }

            token = strtok (NULL,delimiters);
        }
    }

/*    fprintf (stderr,"Goodbye. Connection closed.\n"); */

    fclose (connection);
    remove_queue (msg_id);
    result = system ("rm /home/tourino/camserver.lock"); /* erases lock file */

    return 0;
}
