#include <stdio.h>		/* printf */
#include <errno.h>		/* errno */
#include <stdlib.h>		/* malloc */
#include <unistd.h>		/* sleep */
#include <sys/mman.h>		/* MAP_FILE */
#include <sys/fcntl.h>		/* open */
#include <jpeglib.h>            /* IJG routines */
#include <sys/resource.h>	/* setpriority */
#include <signal.h>		/* getpid */

#include <sys/signal.h>
#include <sys/time.h>

#include "ioctl_meteor.h"

#define VIDEO_DEV       "/dev/meteor0" 

#define COLS 240		/* # of pixels in a row in output image */
#define ROWS 180		/* # of lines in output image */
#define SIZE (ROWS * COLS * 4)	/* 4 bytes/pixel for METEOR_GEO_RGB24 */

#define PROG_PRIO -10		/* priority for the server */

static JSAMPLE * image_buffer;  /* Points to large array of R,G,B-order data */
static int image_height;        /* Number of rows in image */
static int image_width;         /* Number of columns in image */

char *mmbuf;   /* memory map buffer */
char *data;    /* image data buffer */
int fc=0;      /* frame counter */
int size = SIZE;      /* image size */
int im_ready;  /* image ready in data buffer */
int framerate=10;


/* function prototypes */
static void gotframe(int x);

/* signal structure */
static struct sigaction sigact = { gotframe,
                                   0,
                                   SA_RESTART,
                                   NULL };

static struct sigaction oldact;




void gotframe(int sig)
{
  fc++;

  if (fc%framerate==0) {
    memcpy(data,mmbuf,size);
    im_ready=1;
  }

  if ((oldact.sa_handler)!=NULL) {
    (oldact.sa_handler)(sig);
  }
}





void write_JPEG_file (int quality)
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  JSAMPROW row_pointer[1];      /* pointer to JSAMPLE row[s] */
  int row_stride;               /* physical row width in image buffer */

  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);

  jpeg_stdio_dest(&cinfo, stdout);  	/* writes compressed image to stdout */

  cinfo.image_width = image_width;      /* image width and height, in pixels */
  cinfo.image_height = image_height;
  cinfo.input_components = 3;           /* # of color components per pixel */
  cinfo.in_color_space = JCS_RGB;       /* colorspace of input image */

  jpeg_set_defaults(&cinfo);

  jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);

  jpeg_start_compress(&cinfo, TRUE);

  row_stride = image_width * 3; /* JSAMPLEs per row in image_buffer */

  while (cinfo.next_scanline < cinfo.image_height) {
    row_pointer[0] = &image_buffer[cinfo.next_scanline * row_stride];
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  jpeg_finish_compress(&cinfo);

  jpeg_destroy_compress(&cinfo);
}




void write_JPEG_Image (void) 
{
  struct meteor_counts cnt;
  struct meteor_geomet geo;
  char *imdata, *buf;
  int ifd, c, x, y, sigmode = 0;

  struct timeval t; 
  long t0,t1;
  int ic=0;  /* image file counter */
  char nf[20];

  FILE *time_cap;

  time_cap = fopen ("/home/tourino/timecap.out","w");

  if ((ifd = open(VIDEO_DEV, O_RDONLY)) < 0) {
    perror (VIDEO_DEV);
    exit (1);
  }

  geo.rows = ROWS;		/* # of lines in output image */
  geo.columns = COLS;		/* # of pixels in a row in output image */
  geo.frames = 1;		/* # of frames in a buffer */
  geo.oformat = METEOR_GEO_RGB24 ; /* RGB 24 in 4 bytes: B,G,R,NULL */

  sigmode = METEOR_SIG_FRAME;

  if (ioctl (ifd, METEORSETGEO, &geo) < 0) {
    perror ("ioctl SetGeometry failed");
    exit (1);
  }

  c = METEOR_FMT_NTSC;
  if (ioctl (ifd, METEORSFMT, &c) < 0) {
    perror ("ioctl SetFormat failed");
    exit (1);
  }

  c = METEOR_INPUT_DEV0;	/* composite video RCA jack */
  if (ioctl (ifd, METEORSINPUT, &c) < 0) {
    perror ("ioctl Setinput failed");
    exit (1);
  }

  sleep (1);

  if ( sigaction(SIGUSR2, &sigact, &oldact) ) {
    perror("sigaction failed");
    exit(1);
  }

  c = SIGUSR2 | sigmode;
  if (ioctl(ifd, METEORSSIGNAL, &c) < 0) {
    perror("ioctl SetSignal failed");
    exit(1);
  }

  data=(char *)malloc(size);

  mmbuf=(char *)mmap((caddr_t)0, size, PROT_READ,
                     MAP_FILE|MAP_PRIVATE, ifd, (off_t)0);
  if ( mmbuf == (char *)-1 ) {
    perror("mmap failed");
    exit(1);
  }

  /* capture continuously */

  c = METEOR_CAP_CONTINOUS;
  if (ioctl(ifd, METEORCAPTUR, &c)) {
    perror("ioctl SingleCapture failed");
    exit(1);
  }

  /*
   *    M A I N   L O O P
   */

  gettimeofday(&t, NULL);
  t0 = t.tv_sec*1000 + t.tv_usec/1000;
  t1 = t0;

  msg_id = init_message();	/* starting IPC */

  while (t1-t0<600000)    /* Timeout = 10 min. */
    {

      msg = read_message (msg_id,1);

      if (msg != NULL)
	framerate = atol (msg);      	/* frame rate setup */

      // usleep(100);
      gettimeofday(&t, NULL);
      t1 = t.tv_sec*1000 + t.tv_usec/1000;

      if (im_ready) {

	fprintf (time_cap,"%ld\n",t1);
	fflush (time_cap);

	printf ("\n--JPEGImage\nContent-type: image/jpeg\n\n");

	image_width = COLS;
	image_height = ROWS;
  	if (NULL == (image_buffer = (JSAMPLE *)malloc(image_width*image_height*3))) {
    		perror ("malloc");
    		exit (1);
  	}

  	buf = mmbuf;			/* start of frame grabber data (B,G,R,NULL) */
  	imdata = (char *) image_buffer; /* start of JPEG buffer (R,G,B) */
  	for (y=0; y<ROWS; y++) {
    		for (x=0; x<COLS; x++) {
      			*(imdata++) = *(buf+2);	/* R */      
      			*(imdata++) = *(buf+1);	/* G */
      			*(imdata++) = *(buf);	/* B */
      			buf += 4;			/* skip NULL */
    		}	
  	}

  	write_JPEG_file (30);
	fflush (stdout);

        im_ready=0;

      }

      gettimeofday(&t, NULL);
      t1 = t.tv_sec*1000 + t.tv_usec/1000;
  
  }

  c = METEOR_CAP_STOP_CONT;
  if (ioctl(ifd, METEORCAPTUR, &c)) {
    perror("ioctl SingleCapture2 failed");
    exit(1);
  }

  close(ifd);
  fclose (time_cap);

}


int main (int argc, char **argv)
{
	int pid;
	FILE *mypid;	

	if (setpriority (PRIO_PROCESS,0,PROG_PRIO) != 0)
		fprintf (stderr,"Error in setpriority!\n");

	pid = getpid ();	/* getting the pid of program */
	
	/* saving pid */
	mypid = fopen ("/home/tourino/push.pid","w+");
	fprintf (mypid,"%d",pid);
	fclose (mypid);

	printf ("HTTP/1.0 200 Okay\n");
	printf ("Content-type: multipart/x-mixed-replace;boundary=JPEGImage\n\n");
	write_JPEG_Image ();
}
