#include <stdio.h>		/* printf */
#include <errno.h>		/* errno */
#include <stdlib.h>		/* malloc */
#include <unistd.h>		/* sleep */
#include <sys/mman.h>		/* MAP_FILE */
#include <sys/fcntl.h>		/* open */
#include <jpeglib.h>            /* IJG routines */

#include "ioctl_meteor.h"

#define VIDEO_DEV       "/dev/meteor0" 

#define COLS 320		/* # of pixels in a row in output image */
#define ROWS 240		/* # of lines in output image */
#define SIZE (ROWS * COLS * 4)	/* 4 bytes/pixel for METEOR_GEO_RGB24 */

static JSAMPLE * image_buffer;  /* Points to large array of R,G,B-order data */
static int image_height;        /* Number of rows in image */
static int image_width;         /* Number of columns in image */


void write_JPEG_file (int quality)
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  JSAMPROW row_pointer[1];      /* pointer to JSAMPLE row[s] */
  int row_stride;               /* physical row width in image buffer */

  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);

  jpeg_stdio_dest(&cinfo, stdout);  	/* writes compressed image to stdout */

  cinfo.image_width = image_width;      /* image width and height, in pixels */
  cinfo.image_height = image_height;
  cinfo.input_components = 3;           /* # of color components per pixel */
  cinfo.in_color_space = JCS_RGB;       /* colorspace of input image */

  jpeg_set_defaults(&cinfo);

  jpeg_set_quality(&cinfo, quality, TRUE /* limit to baseline-JPEG values */);

  jpeg_start_compress(&cinfo, TRUE);

  row_stride = image_width * 3; /* JSAMPLEs per row in image_buffer */

  while (cinfo.next_scanline < cinfo.image_height) {
    row_pointer[0] = &image_buffer[cinfo.next_scanline * row_stride];
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }

  jpeg_finish_compress(&cinfo);

  jpeg_destroy_compress(&cinfo);
}


int main (int argc, char **argv)
{
  struct meteor_counts cnt;
  struct meteor_geomet geo;
  char *mmbuf;   /* memory map buffer */
  char *imdata, *buf;
  int ifd, c, x, y;

  if ((ifd = open(VIDEO_DEV, O_RDONLY)) < 0) {
    perror (VIDEO_DEV);
    exit (1);
  }

  geo.rows = ROWS;		/* # of lines in output image */
  geo.columns = COLS;		/* # of pixels in a row in output image */
  geo.frames = 1;		/* # of frames in a buffer */
  geo.oformat = METEOR_GEO_RGB24 ; /* RGB 24 in 4 bytes: B,G,R,NULL */
  if (ioctl (ifd, METEORSETGEO, &geo) < 0) {
    perror ("ioctl SetGeometry failed");
    exit (1);
  }

  c = METEOR_FMT_NTSC;
  if (ioctl (ifd, METEORSFMT, &c) < 0) {
    perror ("ioctl SetFormat failed");
    exit (1);
  }

  c = METEOR_INPUT_DEV0;	/* composite video RCA jack */
  if (ioctl (ifd, METEORSINPUT, &c) < 0) {
    perror ("ioctl Setinput failed");
    exit (1);
  }

  sleep (1);	

  if (0 > (mmbuf = (char *) mmap((caddr_t) 0, SIZE, PROT_READ,
				 MAP_FILE | MAP_PRIVATE, ifd, (off_t) 0))) {
    perror ("mmap failed");
    exit (1);
  }

  c = METEOR_CAP_SINGLE ;
  if (ioctl (ifd, METEORCAPTUR, &c)) {
    perror ("ioctl SingleCapture failed");
    exit (1);
  }

  if (ioctl (ifd, METEORGCOUNT, &cnt)) {
      perror ("ioctl GetCount failed");
      exit (1);
  }

  close (ifd);

  image_width = COLS;
  image_height = ROWS;
  if (NULL == (image_buffer = (JSAMPLE *)malloc(image_width*image_height*3))) {
    perror ("malloc");
    exit (1);
  }

  buf = mmbuf;			/* start of frame grabber data (B,G,R,NULL) */
  imdata = (char *) image_buffer; /* start of JPEG buffer (R,G,B) */
  for (y=0; y<ROWS; y++) {
    for (x=0; x<COLS; x++) {
      *(imdata++) = *(buf+2);	/* R */      
      *(imdata++) = *(buf+1);	/* G */
      *(imdata++) = *(buf);	/* B */
      buf += 4;			/* skip NULL */
    }
  }

  printf ("HTTP/1.0 200 Okay\n");
  printf ("Content-type: image/jpeg\n\n");
  write_JPEG_file (30);

  return (0);
}
