// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: db_debug.cpp 
// Compiler Used: MSVC, BCC32, GCC, HPUX aCC, SOLARIS CC
// Produced By: glNET Software
// File Creation Date: 02/04/1997  
// Date Last Modified: 11/17/2002
// Copyright (c) 2001-2002 glNET Software
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
 
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
USA

gxDatabase file recovery functions.
*/
// ----------------------------------------------------------- // 
#include "gxdlcode.h"

#if defined (__USE_ANSI_CPP__) // Use the ANSI Standard C++ library
#include <iostream>
#else // Use the old iostream library by default
#include <iostream.h>
#endif // __USE_ANSI_CPP__

#include <string.h>
#include "gxdbase.h"
#include "gxdstats.h"
#include "ustring.h"

#if defined (__64_BIT_DATABASE_ENGINE__)
const char *ProgramName = "db_debug64";
#else // Use the 32-bit version by default
const char *ProgramName = "db_debug";
#endif

class gxDatabaseDebugManager : public gxDatabase
{
public:
  gxDatabaseDebugManager() { }
  ~gxDatabaseDebugManager() { }

public:
  gxDatabaseError BlindOpen(const char *fname);
  FAU_t BlockSearch(FAU_t Offset); 
};

// Function prototypes for display menu
void ClearInputStream(GXSTD::istream &s);
int Quit();
void Menu(gxDatabaseDebugManager *f);
void PausePrg();
void Version();

// Function prototypes for database utilities 
void FindBlock(gxDatabaseDebugManager *f, int verbose = 0, int walk = 0);
void AnalyzeHeader(gxDatabaseDebugManager *f);
void DisplayStats(gxDatabaseDebugManager *f);
void Rebuild(gxDatabaseDebugManager *f, int update_ver = 0);

gxDatabaseError gxDatabaseDebugManager::BlindOpen(const char *fname)
// Open the file in read only mode without check the file type
{
  // Close any open files
  if(Close() != gxDBASE_NO_ERROR) return gxd_error;
  
  fp = gxdFPTROpen(fname, gxDBASE_READONLY);
    
  if(fp == 0) {
    gxd_error = gxDBASE_FILE_OPEN_ERROR;
#ifdef __CPP_EXCEPTIONS__
    throw gxCDatabaseException();
#else
    return gxd_error;
#endif
  }
  else {
    ready_for_reading = 1;
    ready_for_writing = 0;
    is_open = 1;
    is_ok = 1;
    strcpy(file_name, fname);
    last_operation = gxDBASE_WRITE;
    
    if(ReadFileHdr()!= gxDBASE_NO_ERROR) return gxd_error;

    // Check the version number and set the previous version compatibility mode
    // NOTE: Version 4 and higher compatibility is set by default 
    if((int)file_header.gxd_ver < 2000) { 
      // Version 1 and 2 compatibility mode
      gxDatabase::gxVersion = file_header.gxd_ver;
      gxDatabase::gxInternalCheckWord = 0x0000fefe;
      memmove(gxDatabase::gxSignature, file_header.gxd_sig, 
	      (gxSignatureSize-1)); 
      GXSTD::cout << "Detected version " << (int)file_header.gxd_ver << "\n";
      GXSTD::cout << "Signature "; 
      GXSTD::cout.write(file_header.gxd_sig, (gxSignatureSize-1));
      GXSTD::cout << "\n";
    }
    else if((int)file_header.gxd_ver < 4000) {
      // Version 3 compatibility mode
      gxDatabase::gxVersion = file_header.gxd_ver;
      gxDatabase::gxInternalCheckWord = 0xfefefefe;
      memmove(gxDatabase::gxSignature, file_header.gxd_sig, 
	      (gxSignatureSize-1)); 
      GXSTD::cout << "Detected version " << (int)file_header.gxd_ver << "\n";
      GXSTD::cout << "Signature "; 
      GXSTD::cout.write(file_header.gxd_sig, (gxSignatureSize-1));
      GXSTD::cout << "\n";
    }
    
    // Test file type, checking the revision letter
    if(memcmp(file_header.gxd_sig, gxDatabase::gxSignature,
	      (gxSignatureSize-1)) == 0) {

      // Set the revision letter according to the file header
      char revision[gxSignatureSize];
      memmove(revision, file_header.gxd_sig, gxSignatureSize);
      rev_letter = revision[gxSignatureSize-1];
    }
  }

  // Ensure that true end of file is stored in the file header. 
  FAU_t filesize;
  filesize = FileSize(fname);
  if(file_header.gxd_eof < filesize) {
    file_header.gxd_eof = filesize;
    if(Flush() != gxDBASE_NO_ERROR) return gxd_error;
  }
  
  return gxd_error = gxDBASE_NO_ERROR;
}

FAU_t gxDatabaseDebugManager::BlockSearch(FAU_t Offset)
// Search through the database file until a valid block is found.
// The search starts at the beginning  of the file or the
// offset value. Returns 0 if no valid block is found in the
// file.
{
  gxBlockHeader blk;
  FAU_t file_address = (FAU_t)0;

  FAU_t StaticEOF = FileSize(file_name);

  if(!Offset) // If no Offset, start after database header
    file_address = FileHeaderSize(); 
  else {
    file_address = file_address + Offset; // Offset the starting address 

    if(file_address >= StaticEOF) // Prevent offsetting past EOF
      return 0; // Invalid address
  }
  
  while(1) {
    if((FAU_t)(file_address + BlockHeaderSize()) >= StaticEOF || !IsOK()) 
      return (FAU_t)0;

    if(Read(&blk, sizeof(gxBlockHeader), file_address) != gxDBASE_NO_ERROR)
      return (FAU_t)0;
    if(blk.block_check_word != gxDatabase::gxInternalCheckWord)
      file_address++; // Loop through the file byte by byte
    else
      break; // Found valid block
  }
  return file_address;
}

void DisplayStats(gxDatabaseDebugManager *f)
{
  gxFileHeader fh;

  f->Read(&fh, sizeof(gxFileHeader), (FAU_t)0);
  if(CheckError((gxDatabase *)f) != 0) return;
  
  if(memcmp(fh.gxd_sig, gxDatabase::gxSignature, (gxSignatureSize-1))) {
    // Test file type
    GXSTD::cout << "\n";
    GXSTD::cout << "Database file header is damaged or does not exist" << "\n";
    GXSTD::cout << "\n";
    return;
  }

  DatabaseStats((gxDatabase *)f);
}

void AnalyzeHeader(gxDatabaseDebugManager *f)
{
  gxFileHeader fh;
  gxBlockHeader blk;
  int errors = 0;
  
  f->Read(&fh, sizeof(gxFileHeader), (FAU_t)0);
  if(CheckError((gxDatabase *)f) != 0) return;
  GXSTD::cout << "\n";
  GXSTD::cout << "Analyzing database file header..." << "\n";
  GXSTD::cout << "\n";

  GXSTD::cout << "Checking database signature..." << "\n";
  if (memcmp(fh.gxd_sig, gxDatabase::gxSignature, (gxSignatureSize-1))) {
    // Test file type
    GXSTD::cout << "\n";
    GXSTD::cout << "Database file header is damaged or does not exist" << "\n";
    GXSTD::cout << "\n";
    FindBlock(f, 0);
    return;
  }
  else if (memcmp(fh.gxd_sig, gxDatabase::gxSignature, gxSignatureSize)) {
    // Check revision letters
    char revision[gxSignatureSize];
    revision[gxSignatureSize] = 0; // Ensure null termination
    char rev_letter;
    GXSTD::cout << "\n";
    GXSTD::cout << "NOTE: The database file revision letters do not match." 
		<< "\n";
    memmove(revision, gxDatabase::gxSignature, gxSignatureSize);
    rev_letter = revision[gxSignatureSize-1];
    if(rev_letter == 0)
      GXSTD::cout << "Current database file revision = ZERO" << "\n";
    else
      GXSTD::cout << "Current database file revision = " << rev_letter << "\n";
    memmove(revision, fh.gxd_sig, gxSignatureSize);
    rev_letter = revision[gxSignatureSize-1];
    if(rev_letter == 0)
      GXSTD::cout << "Current database file revision = ZERO" << "\n";
    else
      GXSTD::cout << "The file reads: " << rev_letter << "\n";
    GXSTD::cout << "Backward compatibility rules will be enforced." << "\n";
    GXSTD::cout << "\n";
  }

  GXSTD::cout << "Checking for End of File errors..." << "\n";
  FAU_t StaticEOF = f->FileSize(f->DatabaseName());
  if(StaticEOF > fh.gxd_eof) {
    GXSTD::cout << "\n";
    GXSTD::cout << "End of file error in file: " << f->DatabaseName() << "\n";
    GXSTD::cout << "The actual length is longer then the allocated length!" 
		<< "\n";
    GXSTD::cout << "\n";
    errors++;
  }

  GXSTD::cout << "Checking the heap start value..." << "\n";
  f->Read(&blk, sizeof(gxBlockHeader), fh.gxd_hs_fptr);
  if(blk.block_check_word != gxDatabase::gxInternalCheckWord) {
    GXSTD::cout << "\n";
    GXSTD::cout << "Bad heap start value in file: " << f->DatabaseName() 
		<< "\n";
    GXSTD::cout << "No database block found at file address: "
	 << fh.gxd_hs_fptr << "\n";
    GXSTD::cout << "\n";
    errors++;
  }

  GXSTD::cout << "Checking the free space value..." << "\n";
  if(fh.gxd_fs_fptr != (FAU_t)0) {
    f->Read(&blk, sizeof(gxBlockHeader), fh.gxd_fs_fptr);
    if(blk.block_check_word != gxDatabase::gxInternalCheckWord) {
      GXSTD::cout << "\n";
      GXSTD::cout << "Bad free space value in file: " << f->DatabaseName() 
		  << "\n";
      GXSTD::cout << "No database block found at file address: "
	   << fh.gxd_fs_fptr << "\n";
      GXSTD::cout << "\n";
      errors++;
    }
  }

  GXSTD::cout << "Checking the highest block value..." << "\n";
  f->Read(&blk, sizeof(gxBlockHeader), fh.gxd_hb_fptr);
  if (blk.block_check_word != gxDatabase::gxInternalCheckWord) {
    GXSTD::cout << "\n";
    GXSTD::cout << "Bad heap start value in file: " << f->DatabaseName() 
		<< "\n";
    GXSTD::cout << "No database block found at file address: "
	 << fh.gxd_hb_fptr << "\n";
    GXSTD::cout << "\n";
    errors++;
  }
  
  GXSTD::cout << "Checking the database version number..." << "\n";
  if(fh.gxd_ver != gxDatabase::gxVersion) {
    GXSTD::cout << "\n";
    GXSTD::cout << "NOTE: The version numbers do not match." << "\n";
    GXSTD::cout << "Current database File Version = " << gxDatabase::gxVersion 
		<< "\n";
    GXSTD::cout << "The file reads: " << fh.gxd_ver << "\n";
    GXSTD::cout << "Backward compatibility rules will be enforced." << "\n";
    GXSTD::cout << "\n";
  }

  if(errors) {
    GXSTD::cout << "\n";
    GXSTD::cout << "Database file header has errors!" << "\n";
  }
  else {
    GXSTD::cout << "\n";
    GXSTD::cout << "Database file header checks good." << "\n";
  }
  GXSTD::cout << "\n";
}

void FindBlock(gxDatabaseDebugManager *f, int verbose, int walk)
// Serach the file for all database block.
{
  gxBlockHeader blk;
  int count = 0;
  int badgx = 0;
  UString intbuf;

  GXSTD::cout << "\n";
  GXSTD::cout << "Searching file for all database blocks..." << "\n";
  
  FAU_t StaticEOF = f->FileSize(f->DatabaseName());
  FAU_t addr = f->BlockSearch((FAU_t)0); // Search the entire file
  
  if(addr == (FAU_t)0) {
    GXSTD::cout << "\n";
    GXSTD::cout << "No database blocks found in file: "
	 << f->DatabaseName() << "\n";
    GXSTD::cout << "\n";
    return;
  }

  while(1) { 
    if(addr >= StaticEOF) break;
    f->Read(&blk, sizeof(gxBlockHeader), addr);
    if(CheckError((gxDatabase *)f) != 0) return;
    
    if (blk.block_check_word == gxDatabase::gxInternalCheckWord) {
      count++; // Increment the database block count
      if(verbose) {
	BlockStats((gxDatabase *)f, (addr+f->BlockHeaderSize()));
	if(walk) {
	  char c;
	  GXSTD::cout << "\n";
	  GXSTD::cout << "Press enter to continue or enter 'X' to exit >";
	  GXSTD::cin.clear();
	  GXSTD::cin.get(c);
	  switch(c) {
	    case 'x' : case 'X' :
	      GXSTD::cout << "\n";
	      return;
	    default:
	      break;
	  }
	}
      }
      addr = addr + blk.block_length; // Goto the next database block
    }
    else {
      badgx++;
      GXSTD::cout << "\n";
      intbuf << clear << addr;
      GXSTD::cout << "Found bad database block at address " << intbuf.c_str() 
		  << "\n";
      GXSTD::cout << "Searching for next good database block..." << "\n";
      addr = f->BlockSearch(addr); // Search for the next good block
      if(!addr) {
	GXSTD::cout << "None found!" << "\n";
	GXSTD::cout << "\n";
	return;
      }
    }
  }
  GXSTD::cout << "\n";
  GXSTD::cout << "Found " << count << " good database blocks." << "\n";
  if(badgx) GXSTD::cout << "Found " << badgx << " bad database blocks." 
			<< "\n";
  GXSTD::cout << "\n";
}

void Rebuild(gxDatabaseDebugManager *f, int update_ver)
{
  GXSTD::cout << "\n";
  if(update_ver) 
    GXSTD::cout << "Rebuilding database file and updating to \
latest version..." << "\n";
  else
    GXSTD::cout << "Rebuilding damaged database file..." << "\n";
  GXSTD::cout << "\n";

  char buf[255];
  int retry = 3;
  UString intbuf;

  while(1) { // Loop until a good file name is found
    GXSTD::cout << "Enter new name of file to build>";
    GXSTD::cin.getline(buf, sizeof(buf));
    GXSTD::cout << "\n";
    if(!*buf) { // Return if nothing is entered
      Menu(f);
      return;
    }
    if(gxDatabase::Exists(buf)) {
      GXSTD::cout << "File already exists!" << "\n" << "\n";
       retry--;
      if(!retry) {
	Menu(f);
	return;
      }
    }
    else
      break;
  }

  gxFileHeader fh;
  gxBlockHeader blk;
  int errors = 0;
  
  // Analyze the database file header to determine if the file has
  // a pre-allocated static area
  FAU_t static_area;
  f->Read(&fh, sizeof(gxFileHeader), (FAU_t)0);
  if(CheckError((gxDatabase *)f) != 0) return;
  char rev_letter = f->GetRevLetter();
  
  // Check the database file header's signature. 06/08/2000: Modified
  // to ignore the revision letter. If the header is damaged the
  // current database revision will be used.
  if (memcmp(fh.gxd_sig, gxDatabase::gxSignature, (gxSignatureSize-1))) {
    rev_letter = gxDatabaseRevisionLetter;
    errors++; // Header is damaged and cannot be read
  }
  
  if(!errors) { // Check the heap start value 
    f->Read(&blk, sizeof(gxBlockHeader), fh.gxd_hs_fptr);
    if(CheckError((gxDatabase *)f) != 0) return;
    if (blk.block_check_word != gxDatabase::gxInternalCheckWord)
      errors++;
  }

  // If no errors, calculate the the size of the static area.
  if(!errors) static_area = fh.gxd_hs_fptr - (FAU_t)f->FileHeaderSize();

  FAU_t StaticEOF = f->FileSize(f->DatabaseName());
  FAU_t addr = f->BlockSearch((FAU_t)0); // Search the entire file
  
  if(addr == (FAU_t)0) {
    GXSTD::cout << "\n";
    GXSTD::cout << "No database blocks found in file: "
	 << f->DatabaseName() << "\n";
    PausePrg();
    return;
  }

  // Create the new file
  gxDatabaseDebugManager *NewFile = new gxDatabaseDebugManager;
  if(!NewFile) {
    GXSTD::cout << "Memory allocation error!" << "\n";
  }
  
  // Get the previous state of all the version specific information
  FAU_t prev_ver = gxDatabase::gxVersion;
  char prev_sig[gxSignatureSize];
  memmove(prev_sig, gxDatabase::gxSignature, gxSignatureSize);
  __LWORD__ prev_check_word = gxDatabase::gxInternalCheckWord;

  if(update_ver) {
    gxDatabase::gxVersion = gxDatabaseVersionNumber;

#if defined (__64_BIT_DATABASE_ENGINE__)
    memmove(gxDatabase::gxSignature, "GXDBASE64", gxSignatureSize);
#else // Use the 32-bit version by default
    memmove(gxDatabase::gxSignature, "GXDBASE", gxSignatureSize);
#endif
    
    gxDatabase::gxInternalCheckWord = gxCheckWord;
    rev_letter = gxDatabaseRevisionLetter;
    NewFile->Create(buf, static_area, rev_letter);
    if(CheckError((gxDatabase *)NewFile) != 0) {
      delete NewFile;
      return;
    }
    gxDatabase::gxVersion = prev_ver;
    memmove(gxDatabase::gxSignature, prev_sig, gxSignatureSize);
    gxDatabase::gxInternalCheckWord = prev_check_word;
  }
  else {
    NewFile->Create(buf, static_area, rev_letter);
    if(CheckError((gxDatabase *)NewFile) != 0) {
      delete NewFile;
      return;
    }
  }
  
  int count = 0;
  int badgx = 0;

  char *v;

  if(static_area) { // Write the static area data
    v = new char[(__ULWORD__)static_area];
    if(!v) {
      GXSTD::cout << "Memory allocation error!" << "\n";
      delete NewFile;
      return;
    }
    f->Read(v, (__ULWORD__)static_area, (FAU_t)f->FileHeaderSize());
    if(CheckError((gxDatabase *)f) != 0) {
      delete NewFile;
      return;
    }

    NewFile->Write(v, (__ULWORD__)static_area, (FAU_t)f->FileHeaderSize());
    if(CheckError((gxDatabase *)NewFile) != 0) {
      delete NewFile;
      return;
    }
    delete v;
  }
  
  FAU_t ObjectLength;
  gxINT32 checksum;
    
  while(1) { 
    if(addr >= StaticEOF) break;

    f->Read(&blk, sizeof(gxBlockHeader), addr);
    if(CheckError((gxDatabase *)f) != 0) {
      delete NewFile;
      return;
    }
    if((blk.block_check_word == gxDatabase::gxInternalCheckWord)) {
      if(blk.block_status == gxNormalBlock) { // Only copy normal blocks
	ObjectLength = f->ObjectLength(addr + f->BlockHeaderSize());
	v = new char[(int)ObjectLength];
	if(!v) {
	  GXSTD::cout << "Memory allocation error!" << "\n";
	  delete NewFile;
	  return;
	}
	f->Read(v, (__ULWORD__)ObjectLength);
	if(CheckError((gxDatabase *)f) != 0) {
	  delete NewFile;
	  return;
	}
        if((__SBYTE__)blk.block_status == gxNormalBlock) {
	  count++; // Increment the database block count
	  if(update_ver) {
	    gxDatabase::gxInternalCheckWord = gxCheckWord;
	    NewFile->Write(v, (__ULWORD__)ObjectLength,
			   NewFile->Alloc((__ULWORD__)ObjectLength));
	    if(CheckError((gxDatabase *)NewFile) != 0) {
	      delete NewFile;
	      return;
	    }
	    gxDatabase::gxInternalCheckWord = prev_check_word;
	  }
	  else 
	    NewFile->Write(v, (__ULWORD__)ObjectLength,
			   NewFile->Alloc((__ULWORD__)ObjectLength));
	  if(CheckError((gxDatabase *)NewFile) != 0) {
	    delete NewFile;
	    return;
	  }
	  switch(rev_letter) {
	    case '\0': case ' ': case 'e': case 'E':
	      break;

	    default: // Default to rev A, B, C, and D
	      checksum = calcCRC32(v, (__ULWORD__)ObjectLength);
	      NewFile->Write(&checksum, sizeof(checksum));
	      if(CheckError((gxDatabase *)NewFile) != 0) {
		delete NewFile;
		return;
	      }
	      break;
	  }
	}
	delete v;
      }
      addr = addr + blk.block_length; // Goto the next database block
    }
    else {
      badgx++;
      addr = f->BlockSearch(addr); // Search for the next good block
      if(!addr) break; 
    }
    
  }

  GXSTD::cout << "\n";

  GXSTD::cout << "Wrote " << count << " good database blocks to file: "
       << NewFile->DatabaseName() << "\n";

  if(static_area) {
    intbuf << clear << static_area;
    GXSTD::cout << "Wrote " << intbuf.c_str() << " bytes of Static area data."
		<< "\n";
  }
  if(badgx) 
    GXSTD::cout << "Did not write " << badgx << " bad blocks found in file: "
		<< f->DatabaseName() << "\n";

  if(errors) {
    GXSTD::cout << f->DatabaseName() << " file header is damaged!" << "\n";
    GXSTD::cout << "No header information was copied to "
	 << NewFile->DatabaseName() << "\n";
  }

  NewFile->Close();
  delete NewFile;
  GXSTD::cout << "\n";
}

void Menu(gxDatabaseDebugManager *f)
{
  GXSTD::cout << "\n";
  GXSTD::cout << "Analyzing file: " << f->DatabaseName() << "\n";
  GXSTD::cout << "Enter the letter of your selection at the prompt." << "\n";
  GXSTD::cout << "\n";
  GXSTD::cout << "(A, a)    - Analyze the database file header" << "\n";
  GXSTD::cout << "(D, d)    - Dump every database block" << "\n";
  GXSTD::cout << "(F, f)    - Find every database block in the file" << "\n";
  GXSTD::cout << "(H, h, ?) - Displays this menu" << "\n";
  GXSTD::cout << "(Q, q)    - Quit this program" << "\n";
  GXSTD::cout << "(r)       - Rebuild a damaged database file" << "\n";
  GXSTD::cout << "(R)       - Rebuild and update to current version/revision" 
	      << "\n";
  GXSTD::cout << "(S, s)    - Display database file statistics" << "\n";
  GXSTD::cout << "(W, s)    - Walk through every database block" << "\n";
  GXSTD::cout << "\n";
}

void ClearInputStream(GXSTD::istream &s)
// Used to clear istream
{
  char c;
  s.clear();
  while(s.get(c) && c != '\n') { ; }
}

void PausePrg()
{
  GXSTD::cout << "\n";
  GXSTD::cout << "Press enter to continue..." << "\n";
  GXSTD::cin.get();
}

int Quit()
{
  // Cleanup and exit the porgram
  GXSTD::cout << "Exiting..." << "\n";
  return 0;
}

void Version()
{
  GXSTD::cout << "\n";
  GXSTD::cout << ProgramName << " gxDatabase file recovery program." << "\n";
  EchoDatabaseVersion();
  GXSTD::cout << "\n";
}

int main(int argc, char **argv)
{
  // Display the program version information and exit the program
  if(argc >= 2) {
    if(strcmp(argv[1], "version") == 0) {
      Version();
      return 0;
    }
  }

  if(argc < 2) {
    GXSTD::cout << "\n";
    Version();
    GXSTD::cout << "Usage: " << ProgramName << " infile" << "\n";
    GXSTD::cout << "Usage: " << ProgramName << " infile (command)" << "\n";
    GXSTD::cout << "\n";
    return 1;
   }

  gxDatabaseDebugManager *f = new gxDatabaseDebugManager;   
  if(!f) {
    GXSTD::cout << "Memory allocation error!" << "\n";
    return 1;
  }

  const char *fname = argv[1];
  if(!gxDatabase::Exists(fname)) {
    GXSTD::cout << "The specified file does not exist!" << "\n";
    GXSTD::cout << "Exiting..." << "\n";
    delete f;
    return 1;
  }
  else {
    f->BlindOpen(fname);
    if(CheckError((gxDatabase *)f) != 0) return 1;
  }

  char key;
  if(argc <= 2) Menu(f); // Not processing a command
  int rv = 1;

  while(rv) {
    if(argc > 2) { // Process a single command and exit the loop
      key = *(argv[2]);
      rv = 0;
    }
    else {
      if (!GXSTD::cin) { 
	ClearInputStream(GXSTD::cin);
	if (!GXSTD::cin) {
          GXSTD::cout << "Input stream error" << "\n";
          return 0;
	}
      }
      GXSTD::cout << '>';
      GXSTD::cin >> key;
      if (!GXSTD::cin) continue;
    }
    switch(key) {
      case 'a' : case 'A' :
	if(argc <= 2) ClearInputStream(GXSTD::cin);
	AnalyzeHeader(f);
	break;
      case 'f' : case 'F' :
	if(argc <= 2) ClearInputStream(GXSTD::cin);
	FindBlock(f);
	break;
      case 'd' : case 'D' :
	if(argc <= 2) ClearInputStream(GXSTD::cin);
	FindBlock(f, 1);
	break;
      case 'h' : case 'H' :
	Menu(f);
	break;
      case '?' :
	Menu(f);
	break; 
      case 'q' : case 'Q' :
	rv = Quit();
	break;
      case 'r' : 
	if(argc <= 2) ClearInputStream(GXSTD::cin);
	Rebuild(f);
	break;
      case 'R' :
	if(argc <= 2) ClearInputStream(GXSTD::cin);
	Rebuild(f, 1);
	break;
      case 's' : case 'S' :
	if(argc <= 2) ClearInputStream(GXSTD::cin);
	DisplayStats(f);
	break;
      case 'w' : case 'W' :
	if(argc <= 2) ClearInputStream(GXSTD::cin);
	FindBlock(f, 1, 1);
	break;
      default:
        GXSTD::cout << "Unrecognized command" << "\n";
    }
  }

  delete f;
  return 0;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //

