// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: ffind.cpp 
// Compiler Used: MSVC, BCC32, GCC, HPUX aCC, SOLARIS CC
// Produced By: glNET Software
// File Creation Date: 03/09/1999 
// Date Last Modified: 11/17/2002
// Copyright (c) 2001-2002 glNET Software
// ----------------------------------------------------------- // 
// ------------- Program description and details ------------- // 
// ----------------------------------------------------------- // 
/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
 
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
USA
  
This program is used to find and replace strings in a file.
*/
// ----------------------------------------------------------- // 
#include "gxdlcode.h"

#if defined (__USE_ANSI_CPP__) // Use the ANSI Standard C++ library
#include <iostream>
#else // Use the old iostream library by default
#include <iostream.h>
#endif // __USE_ANSI_CPP__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "strutil.h"
#include "ustring.h"
#include "gxlist.h"
#include "dfileb.h"
#include "futils.h"
#include "gxd_ver.h"

#ifdef __MSVC_DEBUG__
#include "leaktest.h"
#endif

// Version number and program name
const double FFindVersionNumber = gxDatabaseVersion;

#if defined (__64_BIT_DATABASE_ENGINE__)
const char *ProgramName = "ffind64";
#else // Use the 32-bit version by default
const char *ProgramName = "ffind";
#endif

// Program globals
const int MAX_LINE = 1024;   // Maximum characters per line
char *open_file = 0;         // Name of file currently opened
unsigned num_files = 0;      // Total number of files processed
char *string_to_find = 0;    // String to find in each line
int find_string = 0;         // Find specified string flag
int find_all = 0;            // Find all occurrences
int check_case = 1;          // Case sensitive/insensitive compare flag
unsigned num_matches = 0;    // Total number of matches found
int cat_file = 0;            // Concatenate file flag
char *string_to_replace = 0; // String to replace
char *string_to_insert = 0;  // String to insert
int replacing_string = 0;    // Replace string flag
int num_replaced = 0;        // Number of strings replaced
int replacing_line = 0;      // Replace line flag
int inserting_string = 0;    // Insert line flag
int append_lines = 1;        // Append lines connected with a backslash
int num_processed = 0;       // Number of string processed per file

void HelpMessage(const char *program_name, const double version_number)
{
  char gxuffer[255];
  sprintf(gxuffer, "%.3f", version_number);
  GXSTD::cout << "\n";
  GXSTD::cout << program_name << " program version "
       << gxuffer  << "\n";
  GXSTD::cout << "Usage: " << program_name << " [switches] iofile.txt " 
	      << "\n"; 
  GXSTD::cout << "Switches: " << "\n";
  GXSTD::cout << "          -a = Do not append lines connected by a backslash"
	      << "\n";
  GXSTD::cout << "          -c = Concatenate file to stdout." << "\n";
  GXSTD::cout << "          -f = Find first occurrence of specified string: "
	      << "-f\"string\"" << "\n";
  GXSTD::cout << "          -F = Find all occurrences of specified string: "
	      << "-F\"string\"" << "\n";
  GXSTD::cout << "          -i = Perform case insensitive compare." << "\n";
  GXSTD::cout << "\n";
  GXSTD::cout << "          -R -W = Replace all occurrences of a string \
in a file." << "\n";
  GXSTD::cout << "          Usage: -R\"string\" -W\"replacement\"" << "\n";
  GXSTD::cout << "\n";
  GXSTD::cout << "          -L -W = Replace entire line after the specifed \
string." << "\n";
  GXSTD::cout << "          Usage: -L\"string\" -W\"replacement\"" << "\n";
  GXSTD::cout << "\n";
  GXSTD::cout << "          -I -W = Insert a line after the specifed string."
	      << "\n";
  GXSTD::cout << "          Usage: -I\"string\" -W\"insert\"" << "\n";
  GXSTD::cout << "\n";
  exit(0);
}

int ProcessArgs(char *arg)
{
  switch(arg[1]) {
    case 'a':
      append_lines = 0;
      break;
      
    case 'c':
      cat_file = 1;
      break;

    case 'f':				
      if(arg[2] == '\0') {
	GXSTD::cerr << "\n";
	GXSTD::cerr << "You must enter a string following the -f option." 
		    << "\n";
	GXSTD::cerr << "Example usage: " << ProgramName << " -f\"string\"" 
		    << "\n"; 
	GXSTD::cerr << "\n";
	return 0;
      }
      find_string = 1;
      find_all = 0;
      replacing_line = 0;  
      replacing_string = 0;  
      inserting_string = 0;
      string_to_find = arg+2;
      break;

    case 'F':				
      if(arg[2] == '\0') {
	GXSTD::cerr << "\n";
	GXSTD::cerr << "You must enter a string following the -F option." 
		    << "\n";
	GXSTD::cerr << "Example usage: " << ProgramName << " -F\"string\"" 
		    << "\n"; 
	GXSTD::cerr << "\n";
	return 0;
      }
      find_string = 1;
      find_all =1;
      replacing_line = 0;  
      replacing_string = 0;  
      inserting_string = 0;
      string_to_find = arg+2;
      break;

    case 'i':
      check_case = 0;
      break;
      
    case 'R':
      if(arg[2] == '\0') {
	GXSTD::cerr << "\n";
	GXSTD::cerr << "You must enter a string following the -R option." 
		    << "\n";
	GXSTD::cerr << "Example usage: " << ProgramName << " -R\"string\"" 
		    << "\n"; 
	GXSTD::cerr << "\n";
	return 0;
      }
      string_to_replace = arg+2;
      replacing_string = 1;  
      replacing_line = 0;
      inserting_string = 0;
      find_string = 0;
      find_all = 0;
      break;

    case 'L':
      if(arg[2] == '\0') {
	GXSTD::cerr << "\n";
	GXSTD::cerr << "You must enter a string following the -L option." 
		    << "\n";
	GXSTD::cerr << "Example usage: " << ProgramName << " -L\"string\"" 
		    << "\n"; 
	GXSTD::cerr << "\n";
	return 0;
      }
      string_to_replace = arg+2;
      replacing_line = 1;  
      replacing_string = 0;
      inserting_string = 0;
      find_string = 0;
      find_all = 0;
      break;

    case 'I':
      if(arg[2] == '\0') {
	GXSTD::cerr << "\n";
	GXSTD::cerr << "You must enter a string following the -I option." 
		    << "\n";
	GXSTD::cerr << "Example usage: " << ProgramName << " -I\"string\"" 
		    << "\n"; 
	GXSTD::cerr << "\n";
	return 0;
      }
      string_to_replace = arg+2;
      inserting_string = 1;
      replacing_line = 0;  
      replacing_string = 0;  
      find_string = 0;
      find_all = 0;
      break;

    case 'W':
      if(arg[2] == '\0') {
	GXSTD::cerr << "\n";
	GXSTD::cerr << "You must enter a string following the -W option." 
		    << "\n";
	GXSTD::cerr << "Example usage: " << ProgramName << " -W\"string\"" 
		    << "\n"; 
	GXSTD::cerr << "\n";
	return 0;
      }

      if(string_to_replace == 0) {
	GXSTD::cerr << "\n";
	GXSTD::cerr << "The -W option must be used with the -R, -L, or -I \
option." << "\n";
	GXSTD::cerr << "Usage: -R\"string\" -W\"replacement\"" << "\n";
	GXSTD::cerr << "\n";
	return 0;
      }

      string_to_insert = arg+2;
      break;
      
    default:
      GXSTD::cerr << "\n";
      GXSTD::cerr << "Unknown switch " << arg << "\n";
      GXSTD::cerr << "Exiting..." << "\n";
      GXSTD::cerr << "\n";
      return 0;
  }
  arg[0] = '\0';
  return 1; // All command line arguments were valid
}

int ProcessTextFile(DiskFileB &iofile, GXSTD::ostream &stream)
{
  char LineBuffer[df_MAX_LINE_LENGTH];
  UString LineData;
  UString appendlineBuffer;
  gxList<UString> list;
  int rv, offset = 0;
  unsigned long line_number = 0;
  
  while(!iofile.df_EOF()) {
   if(iofile.df_GetLine(LineBuffer, df_MAX_LINE_LENGTH, '\n', 1) != 
      DiskFileB::df_NO_ERROR) {
      break; // Error reading from the disk file
    }
    offset = 0; // Reset string offset
    LineData.Clear(); // Reset the string buffer
    line_number++;
        
    // Detect backslashes used to mark the continuation of a line
    if((LineBuffer[strlen(LineBuffer)-1] == '\\') && (append_lines == 1)) {
      appendlineBuffer += LineBuffer;
      appendlineBuffer += '\n'; // Insert LF after the backslash 
      continue;  // Goto the top of loop and append next line
    }
    if(appendlineBuffer.length() > 0) { // Append the next line
      appendlineBuffer += LineBuffer;
      LineData = appendlineBuffer;
      appendlineBuffer.DeleteAt(0, appendlineBuffer.length());
    }
    else
      LineData = LineBuffer;
    
    // Routine to cat file
    // ----------------------------------------------------------- 
    if(cat_file) stream << LineData.c_str() << "\n";
    
    // Routine to find a string
    // ----------------------------------------------------------- 
    if(string_to_find != 0 && find_string != 0) {
      if(find_all) {
	while(1) {
	  if(check_case)
	    offset = LineData.Find(string_to_find, offset);
	  else
	    offset = LineData.IFind(string_to_find, offset);
	  if(offset == -1) break;
	  num_matches++; num_processed++;
	  offset++;
	}
      }
      else {
	if(check_case)
	  rv = LineData.Find(string_to_find);
	else
	  rv = LineData.IFind(string_to_find);
	if(rv != -1) {
	  num_matches++; num_processed++;
	  GXSTD::cout << "\n";
	  GXSTD::cout << "File name: " << open_file << " String: " 
	       << string_to_find << "\n";
	  GXSTD::cout << "Found match on line: #" << line_number << "\n";
	}
      }
    }

    // Routine to replace a string
    // ----------------------------------------------------------- 
    if(replacing_string) {
      while(1) {
	if(check_case)
	  offset = LineData.Find(string_to_replace, offset);
	else
	  offset = LineData.IFind(string_to_replace, offset);
	if(offset == -1) break;
	LineData.DeleteAt(offset, (strlen(string_to_replace)));
	LineData.InsertAt(offset, string_to_insert);
	num_replaced++;  // Update global replacement counter
	num_processed++; // Record number of string processed for this file
	offset += strlen(string_to_insert);
      }
      list.Add(LineData); // Store all the lines
    }
    
    // Routine to replace a line
    // ----------------------------------------------------------- 
    if(replacing_line) {
      if(check_case)
	offset = LineData.Find(string_to_replace, offset);
      else
	offset = LineData.IFind(string_to_replace, offset);

      if(offset != -1) {
	offset += strlen(string_to_replace);
	LineData.DeleteAt(offset, (LineData.length() - offset));
	LineData.InsertAt(offset, string_to_insert);
	num_replaced++;  // Update global replacement counter
	num_processed++; // Record number of string processed for this file
      }
      list.Add(LineData); // Store all the lines 
    }
    
    // Routine to insert a line
    // ----------------------------------------------------------- 
    if(inserting_string) {
      if(check_case)
	offset = LineData.Find(string_to_replace, offset);
      else
	offset = LineData.IFind(string_to_replace, offset);

      if(offset != -1) {
	offset += strlen(string_to_replace);
	LineData.InsertAt(offset, string_to_insert);
	num_replaced++;  // Update global replacement counter
	num_processed++; // Record number of string processed for this file
      }
      list.Add(LineData); // Store all the lines 
    }
  } 

  // If any strings were replaced in the file, rewrite the file
  // ----------------------------------------------------------- 
  if(num_replaced > 0) {
    char *EndOfLineSequence;
    iofile.df_Close(); 
    iofile.df_Create(open_file);
#if defined(__DOS__) || defined (__WIN32__)
    EndOfLineSequence = "\r\n"; 
#else 
    EndOfLineSequence = "\n"; 
#endif
    
    if(!iofile) {
      GXSTD::cerr << "\n";
      GXSTD::cerr << "Cannot write to: " << open_file << "\n";
      GXSTD::cerr << "Exiting..." << "\n";
      GXSTD::cerr << "\n";
      exit(0);
    }

    gxListNode<UString> *list_ptr = list.GetHead();
    while(list_ptr) {
      if(iofile.df_Write(list_ptr->data.c_str(), list_ptr->data.length()) !=
	 DiskFileB::df_NO_ERROR) {
	break;
      }
      if(iofile.df_Write(EndOfLineSequence, strlen(EndOfLineSequence)) != 
	 DiskFileB::df_NO_ERROR) {
	break;
      }
      list_ptr = list_ptr->next; 
    }

    if(iofile.df_CheckError()) {
      GXSTD::cerr << "\n";
      GXSTD::cerr << "Error writing to: " << open_file << "\n";
      GXSTD::cerr << iofile.df_ExceptionMessage() << "\n";
      GXSTD::cerr << "Exiting..." << "\n";
      GXSTD::cerr << "\n";
      iofile.df_Close();
      exit(0);
    }

    iofile.df_Close();
  }

  list.ClearList();
  return 1;
}

// Program's main thread of execution.
// ----------------------------------------------------------- 
int main(int argc,     // Number of strings in array argv.
	 char *argv[], // Array of command-line argument strings.
	 char *envp[]) // Array of environment variable strings.
// NOTE: None of the MSVC compilers will expand wildcard characters
// used in command-line arguments unless linked with the setargv.obj
// library. All the UNIX compliers will expand wildcard characters
// by default.
{
#ifdef __MSVC_DEBUG__
  InitLeakTest();
#endif

  // If no arguments are given print usage message to the screen 
  if(argc < 3) {
    HelpMessage(ProgramName, FFindVersionNumber);
    return 0;
  }

  // Process command ling arguments and files 
  int narg;
  char *arg = argv[narg = 1];
  while (narg < argc) {
    if (arg[0] != '\0') {
      if (arg[0] == '-') { // Look for command line arguments
	if(!ProcessArgs(arg)) return 0; // Exit if argument is not valid
      }
      else { 
	if(futils_isdirectory((const char *)arg)) {
	  // Do not process directories
	  arg = argv[++narg];
	  continue;
	}
	open_file = arg; // Update the open file name pointer
	DiskFileB iofile(open_file);
	if(!iofile) { // Skip over directory names
	  if(narg < argc) {
	    arg = argv[++narg];
	    iofile.df_Close();
	    continue; 
	  }
	  else { // Cannot open the specified file
	    GXSTD::cerr << "\n";
	    GXSTD::cerr << "Cannot open file: " << open_file << "\n";
	    GXSTD::cerr << "Exiting..." << "\n";
	    GXSTD::cerr << "\n";
	    return 0;
	  }
	}
	num_files++;

	// Echo the name of the file and search operation
	// ----------------------------------------------------------- 
	if(replacing_string == 1) {
	  if(string_to_insert == 0) {
	    GXSTD::cerr << "\n";
	    GXSTD::cerr << "The -R option must be used with the -W option." 
			<< "\n";
	    GXSTD::cerr << "Usage: -R\"string\" -W\"replacement\"" << "\n";
	    GXSTD::cerr << "\n";
	    return 0;
	  }
	  num_processed = 0; // Number of string processed for this file
	  GXSTD::cout << "\n";
	  GXSTD::cout << "Opening file: " << open_file << "\n";
	  GXSTD::cout << "Replacing: " << string_to_replace << "\n";
	  GXSTD::cout << "With: " << string_to_insert << "\n";
	}

	if(replacing_line) {
	  if(string_to_insert == 0) {
	    GXSTD::cerr << "\n";
	    GXSTD::cerr << "The -L option must be used with the -W option." 
			<< "\n";
	    GXSTD::cerr << "Usage: -L\"string\" -W\"replacement\"" << "\n";
	    return 0;
	  }
	  num_processed = 0; // Number of string processed for this file
	  GXSTD::cout << "\n";
	  GXSTD::cout << "Opening file: " << open_file << "\n";
	  GXSTD::cout << "Replacing line after: " << string_to_replace << "\n";
	  GXSTD::cout << "With: " << string_to_insert << "\n";
	}

	if(inserting_string ) {
	  if(string_to_insert == 0) {
	    GXSTD::cerr << "\n";
	    GXSTD::cerr << "The -I option must be used with the -W option." 
			<< "\n";
	    GXSTD::cerr << "Usage: -I\"string\" -W\"insert\"" << "\n";
	    GXSTD::cerr << "\n";
	    return 0;
	  }
	  num_processed = 0; // Number of string processed for this file
	  GXSTD::cout << "\n";
	  GXSTD::cout << "Opening file: " << open_file << "\n";
	  GXSTD::cout << "Inserting: " << string_to_replace << "\n";
	  GXSTD::cout << "After: " << string_to_insert << "\n";
	}

	if(find_string) {
	  num_processed = 0; // Number of string processed for this file
	  // Do not echo file status to the console
	}

	// Process the test file
	num_matches = 0; num_replaced = 0; num_processed = 0;
	ProcessTextFile(iofile, GXSTD::cout);
	if(num_files) iofile.df_Close();
	
	// Report the total number of strings replaced/inserted 
	// ----------------------------------------------------------- 
	if(replacing_string) {
	  GXSTD::cout << "Replaced " << num_processed << " occurrence(s)." 
		      << "\n";
	  GXSTD::cout << "\n";
	}

	if(replacing_line) {
	  GXSTD::cout << "Replaced " << num_processed << " occurrence(s)." 
		      << "\n";
	  GXSTD::cout << "\n";
	}

	if(inserting_string ) {
	  GXSTD::cout << "Inserted " << num_processed << " line(s)." << "\n";
	  GXSTD::cout << "\n";
	}

	if(find_string) {
	  if(find_all) {
	    if(num_processed > 0) {
	      GXSTD::cout << "\n";
	      GXSTD::cout << "File name: " << open_file << " String: " 
		   << string_to_find << "\n";
	      if(num_processed > 1) 
		GXSTD::cout << "Found " << num_processed << " matches" << "\n";
	      else
		GXSTD::cout << "Found 1 match" << "\n";
	    }
	  }
	  if(num_matches == 0) { // Used by -f command if search fails
	    // Do not echo results to the console
	  }
	}
      }
      arg = argv[++narg];
    }
  }
  
  if(num_files == 0) {
    GXSTD::cerr << "\n";
    GXSTD::cerr << "You must enter a file name." << "\n";
    GXSTD::cerr << "Exiting..." << "\n";
    GXSTD::cerr << "\n";
    return 0;
  }
  
  // Return the total number of strings replaced/inserted 
  // ----------------------------------------------------------- 
  if(replacing_string) return num_replaced;
  if(replacing_line) return num_replaced;
  if(inserting_string ) return num_replaced;
  if(find_string) return num_matches;

  return 0;
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
