// ACIS V6.3
// This program creates a axis and saves it to a file .sat and .wrml
#include <stdio.h>                             // File I/O functions
#include <stdlib.h>
#include <string>
#include "boolean/kernapi/api/boolapi.hxx" // Declares boolean API's
#include "constrct/kernapi/api/cstrapi.hxx"    // Declares constructor api's
#include "cover/kernapi/api/coverapi.hxx"   // Declares api_cover_wires
#include "operator/kernapi/api/operapi.hxx"  
#include "blend/kernapi/api/blendapi.hxx"   // Declares blending API's 
#include "faceter/api/af_api.hxx"
#include "kernel/acis.hxx"                 // Declares system wide parameters
#include "kernel/acis3dt.h"
#include "kernel/kernapi/api/kernapi.hxx"      // Declares kernel api's
#include "kernel/kernapi/api/api.hxx"
#include "kernel/kerndata/lists/lists.hxx"     // Declares ENTITY_LIST class
#include "kernel/kerndata/top/body.hxx" //Declara a classe BODY
#include "kernel/kerndata/savres/fileinfo.hxx" // Declares fileinfo class
#include "kernel/kerndata/data/entity.hxx"
#include "kernel/kerndata/lists/lists.hxx"
#include "kernel/kerndata/geometry/getbox.hxx" 
#include "kernel/kerndata/geom/transfrm.hxx"
#include "baseutil/vector/transf.hxx"  // Declares position
#include "baseutil/vector/vector.hxx"
#include "baseutil/vector/unitvec.hxx"
#include "baseutil/vector/position.hxx"  // Declares position
#include "kernel/kerndata/top/alltop.hxx"   // Declares Topology Classes
#include "VrmlMM.hxx"
#include "VrmlMM.cxx"
#include "cgiparser.h"

using namespace rude; // include the rude namespace
using std ::string;

inline position p(double p1,double p2,double p3)
{ return position (p1,p2,p3); }

inline double max(double a, double b)
{ return  (a > b) ? a: b; }

void save_ent(char*, ENTITY*);  // Function prototype
void save_wrl(char *, char *);  // Function prototype

int main() {   
	
  double comprimento[20], diam_e[20], diam_d[20], raio_d[20], raio_e[20];
	double blend_e[20], vblend_e[20], blend_d[20], vblend_d[20];
	double dfurod, profurod;
  int numcil, codfuros, seccao;
	
	CGIParser *parser = CGIParser::instance();
		
	//Pega as strings com os valores concatenados que vem da pgina jsp...
	//Ex: string comp = "12-34-56-78-"
	string comp(parser->value("comp"));
	string diamE(parser->value("diam_e"));
	string diamD(parser->value("diam_d"));
	string blendE(parser->value("blend_e"));
	string vblendE(parser->value("vblend_e"));
	string blendD(parser->value("blend_d"));
	string vblendD(parser->value("vblend_d"));
	
	//conversao para inteiro
	numcil = (int) atoi(parser->value("numsol"));
	codfuros = (int) atoi(parser->value("codfuros"));
	seccao = (int) atoi(parser->value("seccao"));
	
  int i = 0;
  int digitos = 0;
	string temp;
  
	//Converte as substrings da string comp em valores double
  for(int j = 0; j < comp.length(); j++){
			digitos++;
			if(comp[j] == '-'){
			  temp.assign(comp, (j - (digitos-1)), digitos-1);
				comprimento[i] = atof(temp.data());
				digitos = 0;
				i++;
			}
	}
  
  //Converte as substrings da string diamE em valores double
	i = 0;
	digitos = 0;
	for(j = 0; j < diamE.length(); j++){
			digitos++;
			if(diamE[j] == '-'){
			  temp.assign(diamE, (j - (digitos-1)), digitos-1);
				diam_e[i] = atof(temp.data());
				raio_e[i] = 0.5 * diam_e[i];
				digitos = 0;
				i++;
			}
	}

	//Converte as substrings da string diamD em valores double
  i = 0;
	digitos = 0;
	for(j = 0; j < diamD.length(); j++){
			digitos++;
			if(diamD[j] == '-'){
			  temp.assign(diamD, (j - (digitos-1)), digitos-1);
				diam_d[i] = atof(temp.data());
				raio_d[i] = 0.5 * diam_d[i];
				digitos = 0;
				i++;
			}
	}
	
	//Converte as substrings da string blendE em valores double
	i = 0;
	digitos = 0;
	for(j = 0; j < blendE.length(); j++){
			digitos++;
			if(blendE[j] == '-'){
			  temp.assign(blendE, (j - (digitos-1)), digitos-1);
				blend_e[i] = atof(temp.data());
				digitos = 0;
				i++;
			}
	}

	//Converte as substrings da string vBlendE em valores double
	i = 0;
	digitos = 0;
	for(j = 0; j < vblendE.length(); j++){
			digitos++;
			if(vblendE[j] == '-'){
			  temp.assign(vblendE, (j - (digitos-1)), digitos-1);
				vblend_e[i] = atof(temp.data());
				digitos = 0;
				i++;
			}
	}

	//Converte as substrings da string blendD em valores double
	i = 0;
	digitos = 0;
	for(j = 0; j < blendD.length(); j++){
			digitos++;
			if(blendD[j] == '-'){
			  temp.assign(blendD, (j - (digitos-1)), digitos-1);
				blend_d[i] = atof(temp.data());
				digitos = 0;
				i++;
			}
	}

	//Converte as substrings da string vblendD em valores double
	i = 0;
	digitos = 0;
	for(j = 0; j < vblendD.length(); j++){
			digitos++;
			if(vblendD[j] == '-'){
			  temp.assign(vblendD, (j - (digitos-1)), digitos-1);
				vblend_d[i] = atof(temp.data());
				digitos = 0;
				i++;
			}
	}
						
	
	api_start_modeller(0);
  api_initialize_booleans();
	
	BODY *cilin[20];
	double inicio = 0.0;
	
  //Construindo o eixo...
	if (numcil!=0) {
	  api_solid_cylinder_cone(p(0,0,0),p(0,0,comprimento[0]),raio_e[0],raio_e[0],raio_d[0],NULL,*cilin);

	  inicio += comprimento[0];
	
	  for (int j=1; j<numcil; j++) {

		  api_solid_cylinder_cone(p(0,0,inicio),p(0,0,inicio+comprimento[j]),raio_e[j],raio_e[j],raio_d[j],NULL,*(cilin+j));
	    inicio+=comprimento[j];
		  api_unite(*(cilin+j-1),*(cilin+j));
	  
		}

	  api_initialize_blending();  // Initialize highest level lib

		VERTEX *v;
	  ENTITY_LIST edges;
    ENTITY_LIST edges_to_blend;
		double pos = 0.0;
		
		//Aplicando chanfros e raios de concordncia...
		for (int z=0; z<numcil; z++) {
		
			if (blend_e[z] == 1) { //Chanfro  esquerda
			
			  api_find_vertex(*(cilin+numcil-1), position(raio_e[z],0,pos), v);
	      api_q_edges_around_vertex(v, &edges);  //Procura a edge correta para aplicar o chanfro
        api_chamfer_edges(edges, vblend_e[z]); //Definio do raio do chanfro
	      edges_to_blend.add((EDGE*)edges[0]);
	      edges.clear();
						
			}

			else if (blend_e[z] == 2) { //Raio de concordncia  esquerda 
			
				api_find_vertex(*(cilin+numcil-1), position(raio_e[z],0,pos), v);
	      api_q_edges_around_vertex(v, &edges);
        api_blend_edges(edges, vblend_e[z]);
	      edges_to_blend.add((EDGE*)edges[0]);
	      edges.clear();
			
			}
		
		  pos += comprimento[z];

			if (blend_d[z] == 1) { //Chanfro  direita
			
			  api_find_vertex(*(cilin+numcil-1), position(raio_d[z],0,pos), v);
	      api_q_edges_around_vertex(v, &edges);  //Procura a edge correta para aplicar o chanfro
        api_chamfer_edges(edges, vblend_d[z]); //Definio do raio do chanfro
	      edges_to_blend.add((EDGE*)edges[0]);
	      edges.clear();
			
			}

			else if (blend_d[z] == 2) { //Raio de concordncia  direita 
			
			  api_find_vertex(*(cilin+numcil-1), position(raio_d[z],0,pos), v);
	      api_q_edges_around_vertex(v, &edges);  //Procura a edge correta para aplicar o chanfro
        api_blend_edges(edges, vblend_d[z]); //Definio do raio do chanfro
	      edges_to_blend.add((EDGE*)edges[0]);
	      edges.clear();
			
			}
		
		
		}
		
		api_fix_blends(edges_to_blend);

    api_terminate_blending();

		//FILE *myfile;
    //myfile = fopen ("check.txt", "w");
    //fprintf (myfile, "codfuros = %d\n", codfuros);
		//fprintf (myfile, "seccao = %d\n", seccao);
		//fclose (myfile);

    //codfuros = 2 -> H um furo  DIREITA
	  if (codfuros == 2) {

		  dfurod = atof(parser->value("dfuro"));
		  profurod = atof(parser->value("profuro"));
		  double rfurod = dfurod/2.0;

		  BODY *cilinfuro[1];

		  api_solid_cylinder_cone(p(0,0,inicio-profurod),p(0,0,inicio),rfurod,rfurod,rfurod,NULL,*cilinfuro);

		  api_subtract(*cilinfuro,*(cilin+numcil-1));

	  }

    //codfuros = 3 -> H um furo  ESQUERDA
	  else if (codfuros == 3) {

		  dfurod = atof(parser->value("dfuro"));
		  profurod = atof(parser->value("profuro"));
		  double rfurod = dfurod/2.0;

      BODY *cilinfuro[1];

		  api_solid_cylinder_cone(p(0,0,0),p(0,0,profurod),rfurod,rfurod,rfurod,NULL,*cilinfuro);

		  api_subtract(*cilinfuro,*(cilin+numcil-1));
	  
		}

   //codfuros = 4 -> H um furo PASSANTE
	  else if (codfuros == 4) {

		  dfurod = atof(parser->value("dfuro"));
		  double rfurod = dfurod/2.0;

      BODY *cilinfuro[1];

		  api_solid_cylinder_cone(p(0,0,0),p(0,0,inicio),rfurod,rfurod,rfurod,NULL,*cilinfuro);

		  api_subtract(*cilinfuro,*(cilin+numcil-1));
	  
		}

    //Determinao da seo transversal do eixo escalonado.
	  if (seccao > 1) {
		   
			api_initialize_covering();

		  // Get a bounding box around the model
 		  box shaft_size = get_body_box(*(cilin+numcil-1));

		  //Find out the length of its diagonal
	    position bottom = shaft_size.low();
	    position top = shaft_size.high();
	    vector diagonal  = top - bottom;
      double len = diagonal.len();

	    //Create a circular edge
	    position cen(0,0,inicio/2.0); // point on section
	    vector direction(1,0,0); // Determines direction of face
      unit_vector norm = normalise(direction);
      vector radius(0,0,len*0.6); // Box size plus a bit
      EDGE *slicer_e;
	    api_mk_ed_ellipse(cen,norm,radius,1,0,0,slicer_e);

	    //Turn the edge into a WIRE Body
      EDGE *e_array[1];
	    e_array[0] = slicer_e;
	    BODY *slicer_wb;
	    api_make_ewire(1,e_array,slicer_wb);

      //Cover the WIRE to make a planar FACE
      ENTITY_LIST slicer_faces;
      api_cover_wires(slicer_wb,*(surface*)NULL,slicer_faces);

	    //Reverse the direction of the cutter
      FACE *sf = (FACE*)slicer_faces[0];
      api_reverse_face(sf);

	    //Carry-out the section
      api_boolean(slicer_wb,*(cilin+numcil-1),SUBTRACTION);

      // Preparar para seccionar o eixo uma segunda vez.
		  if (seccao == 3) {

	      //Create a second circular edge
				position cen2(0,0,inicio/2.0); // point on section
				vector direction2(0,1,0); // Determines direction of face
				unit_vector norm2 = normalise(direction2);
				vector radius2(0,0,len*0.6); // Box size plus a bit
				EDGE *slicer_e2;
				api_mk_ed_ellipse(cen2,norm2,radius2,1,0,0,slicer_e2);

	      //Turn the edge into a WIRE Body
				EDGE *e_array2[1];
				e_array2[0] = slicer_e2;
				BODY *slicer_wb2;
				api_make_ewire(1,e_array2,slicer_wb2);

        //Cover the WIRE to make a planar FACE
				ENTITY_LIST slicer_faces2;
				api_cover_wires(slicer_wb2,*(surface*)NULL,slicer_faces2);

	      //Reverse the direction of the cutter
				FACE *sf2 = (FACE*)slicer_faces2[0];
				api_reverse_face(sf2);

	      //Carry-out the section
				api_boolean(slicer_wb2,*(cilin+numcil-1),SUBTRACTION);

			} // Fim do cdigo para a determinao da segunda seo transversal.

	    api_terminate_covering();

		}// Fim do cdigo para a determinao da seo transversal.

      save_ent("..\\htdocs\\acis1\\sats\\eixo.sat",*(cilin+numcil-1));
      save_wrl("..\\htdocs\\acis1\\sats\\eixo.sat","..\\htdocs\\acis1\\wrls\\eixo.wrl");
 }

    printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhaeixo.shtml\n\n");

    api_terminate_booleans();
    api_stop_modeller();

	return 1;
}


void save_ent(char *filename, ENTITY *ent)
{
		FileInfo info;  // Create FileInfo Object
		info.set_product_id("UFSC");  // set info's data
		info.set_units(10.0);  // Millimeters 

      // Sets header info to be written to sat file 
		api_set_file_info(FileId | FileUnits, info);

        FILE *fp = fopen(filename, "w");
        if (fp != NULL) {
                ENTITY_LIST savelist; //= new ENTITY_LIST;
                savelist.add(ent);
                api_save_entity_list(fp,TRUE,savelist);
                //delete savelist;
				fclose(fp); 
         }
         else
		 {   printf("Unable to open file!\n");
		     exit(1);
		 }
		
        
}

void save_wrl(char *filein, char *fileout)
{ //char *path;
  outcome rc;
  BODY *TheBody = NULL;
  FILE *pSatFile = NULL;
  FILE *pOutFile = NULL;
  ENTITY_LIST EntList;
  REFINEMENT* ref = NULL;
  int index = 0;
  int refFlag = 0;


	rc = api_start_modeller(0);
  
  pSatFile = fopen( filein, "r" );
  
  rc = api_restore_entity_list(pSatFile, TRUE, EntList);
  fclose(pSatFile);
  
  pOutFile = fopen(fileout, "w");
  
  rc = api_initialize_faceter();
  
  VRML_MESH_MANAGER *MM = new VRML_MESH_MANAGER();
  rc = api_set_mesh_manager( (MESH_MANAGER *) MM);

  rc = api_get_default_refinement( ref );
  
  rc = api_set_default_refinement( ref );
  
  MM->set_output_file(pOutFile);
  MM->header();

  //find the scale and apply any transforms already there
  EntList.init();
  box bounds;
  index = 1;
  while( TheBody = (BODY*)EntList.next() ) {
    if( !is_REFINEMENT( (ENTITY*)TheBody ) ) {
      api_change_body_trans( TheBody, NULL );

      if( index == 1 ) {
	bounds = get_body_box( TheBody, NULL );
	index = 0;
      }
      else {
	bounds = bounds | get_body_box( TheBody, NULL );
      }
    }
  }

  //finds the scale so that the bodies to fit in a 
  //100*x*y box where x <= 100 && y <= 100
  //it then centers all the bodies at (50, 50, 50)
  double max_dim = labs( bounds.high().x() - bounds.low().x() );
  max_dim = max( max_dim,
		 labs( bounds.high().y() - bounds.low().y()) );
  max_dim = max( max_dim,
		 labs( bounds.high().z() - bounds.low().z()) );
  //make scale transform
  double scale = 100.0/max_dim;
  transf trans = scale_transf( scale );
  //make translation transform
  vector vec =  position(50.0, 50.0, 50.0) - bounds.mid() * scale;
  transf tranl = translate_transf( vec );

  index = 0;
  EntList.init();
  while(TheBody = (BODY*)EntList.next()) {
    if( !is_REFINEMENT( TheBody ) ) {
      index++;
      
      //apply scale transform
      api_apply_transf( (ENTITY *)TheBody, trans );
      rc = api_change_body_trans( TheBody, NULL );
      
      //apply translation transform
      api_apply_transf( (ENTITY *)TheBody, tranl );
      rc = api_change_body_trans( TheBody, NULL );
      
      rc = api_facet_entity( TheBody );
      
    }
  }
  
 
  MM->footer();
  fclose(pOutFile);
  delete MM;

  EntList.init();
  while(TheBody = (BODY*)EntList.next())
    api_del_entity(TheBody);
  
  api_terminate_faceter();
	api_stop_modeller();


}









