/******************************************************************************

STOPSERVER.C - Robot's stop server through the INETD main server
 Sergio Roberto Gonsalves Tourino - tourino@graco.unb.br - 10/05/2000

Before using this program, edit the 'inetd.conf' and the 'services' files:

INETD.CONF:
 stopserver  stream  tcp     nowait  root /home/tourino/stopserver stopserver

SERVICES:
 stopserver  8084/tcp

After these modifications, restart the inetd server:
 kill -1 <PID of inetd - see with ps>

*******************************************************************************
/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>

/*-----------------3/29/00 5:17PM-------------------
 * This main routine parses commands from stdin, which
 * corresponds to a socket connection from a remote client
 * connected to this server. Its outputs are sent to the
 * stdout stream, back to the client machine. The local
 * tests are made with the stderr stream. This program is
 * to be used with the INETD main server.
 * --------------------------------------------------*/

#define TRUE    (1==1)
#define FALSE   (1==0)

/****************************************************************************/

void goodbye (void) {
        int result;

        result = system ("rm /home/tourino/stopserver.lock");
}

/****************************************************************************/

int main (void) {

    const char delimiters[] = "=, ";    /* delimiters for the commands */
    char command[] = "direction=front, distance=500, nothing important, directi
on=left, goodbye";
    char *token, parameters[10], commandline[100];
    char direction[10];                 /* direction for the movement */
    
    char sensors[3];                    /* state of sensors */
    char get[10];

    int open_connection = TRUE;
    int result, pid;
    char spid[5];
    int dir, distance = 500, speed=100, teta=524, omega=314;    /* movement con
figuration */

    FILE *lock, *robotserver;

    /* verify the lock file */

    lock = fopen ("/home/tourino/stopserver.lock","r");

    if (lock == NULL) {                 /* the lock does not exist */
        lock = fopen ("/home/tourino/stopserver.lock","w+");
        fclose (lock);
    }
    else {                              /* the lock exists */
        fclose (lock);
        fprintf (stderr,"Lock file found. Program aborted.\n");
        exit (0);                       /* closes the program */
    }

    atexit (goodbye);           /* defines the 'goodbye' function to be
                                        called at program termination */

    while (open_connection) {           /* loop until 'open_connection' is fals
e */

        fscanf (stdin,"%s",command);    /* reads commands from stdin */

        token = strtok(command,delimiters);  /* strtok parses the commands from
 'command' variable */

        while ((token != NULL) && open_connection) {

            if (strcmp(token,"stop") == 0) {
                strcpy (get,strtok(NULL,delimiters));
                if (strcmp (get,"on") == 0) {
                        printf ("stop=on!\n");
                        robotserver = fopen ("/home/tourino/robserver.pid","r")
;
                        fscanf (robotserver,"%s",spid);
                        fclose (robotserver);
                        pid = atoi (spid);
                        printf ("Sending signal to the %d process.\n",pid);
                        kill (pid,SIGUSR1);   /* send the signal to the server 
*/
                }
            }

            if (strcmp(token,"goodbye") == 0) {
                open_connection = FALSE;  /* terminates all of the loops */
            }

            token = strtok (NULL,delimiters);
        }
    }

    fprintf (stderr,"Goodbye. Connection closed.\n");

    return 0;
}


