#include "senir.h"
#include <stdio.h>
#include <stdlib.h>
/**
*
*	@title   Library C++ XR4000 - Class InfraRed Sensor
*	@author  Mrcio Moreira, Paulo Roberto Corra Dutra e Gabriel
*	Last update: 24/11/2002 - Paulo
*
**/

/* --------------------------------------------------------------
 * Function: 	SenIr::SenIr and SenIr::~SenIr
 * Purpose: 	The constructor and destructor of SenIr class
 * Arguments:	none
 * Return:	an instance of SenIr class
 * --------------------------------------------------------------*/

SenIr::SenIr(){}
SenIr::~SenIr(){}


/* --------------------------------------------------------------
 * Function: 	SenIr::Start
 * Purpose: 	start all infrared sensors of nomad
 * Arguments:	state: the Robot State
 * Return:	int : Error code of start infrared sensors action
		0 if OK and another value when error ocurred
 * --------------------------------------------------------------*/

int SenIr::Start(N_RobotState *state)
{
	struct N_InfraredSet *ninfrared_set;
	struct N_InfraredController *ninfrared_controller;

	ninfrared_controller = &(state->InfraredController);
	ninfrared_controller->InfraredPaused=FALSE;

	for (int i=0; i<6 ; i++ )
	{
		ninfrared_set = &(state->InfraredController.InfraredSet[i]);
		ninfrared_set->DataActive = TRUE;
		ninfrared_set->TimeStampActive = TRUE;
	}


	return 0;

}

/* --------------------------------------------------------------
 * Function: 	SenIr::Stop
 * Purpose: 	stop all infrared sensors of nomad
 * Arguments:	state: the Robot State
 * Return:	int : Error code of start infrared sensors action
		0 if OK and another value when error ocurred
 * --------------------------------------------------------------*/
int SenIr::Stop(N_RobotState *state)
{
	struct N_InfraredController *ninfrared_controller;

	ninfrared_controller = &(state->InfraredController);
	ninfrared_controller->InfraredPaused=TRUE;

	return 0;
}

/* --------------------------------------------------------------
 * Function: 	SenIr::Read
 * Purpose: 	return the minimum converted read of a given
 		infrared index in distance
 * Arguments:	*state : the Robot State
 		ir_index: the index of infrared to check
 * Return:	long : the minimum of the values from top infrared
 		       ring and the bottom infrared ring, at the
		       given index
 * --------------------------------------------------------------*/
long SenIr::Read(N_RobotState *state, int ir_index)
{
	int set, ir;
	long r1, r2, result;

	//N_GetInfrared(1);

	set = sensor_circle[ir_index][0];
	ir  = sensor_circle[ir_index][1];

	r1 = state->InfraredController.InfraredSet[set].Infrared[ir].Reading;
	r2 = state->InfraredController.InfraredSet[set+1].Infrared[ir].Reading;

	result = r1 >? r2; // MAX(r1,r2)

	if (result < 10) return LONG_MAX; // --> Desconsiderando valores muito distantes

	return (long)(400-1.46*(result - 10));
}



