#include "sensonar.h"
#include <stdio.h>

/* --------------------------------------------------------------
 * Function: 	Sensonar::Sensonar and Sensonar::~Sensonar
 * Purpose: 	The constructor and destructor of Sensonar class
 * Arguments:	none
 * Return:	an instance of Sensonar class
 * --------------------------------------------------------------*/

SenSonar::SenSonar(){}
SenSonar::~SenSonar(){}

/* --------------------------------------------------------------
 * Function: 	Sensonar::Start
 * Purpose: 	start firing all sonar sensors of given set
 * Arguments:	*state : Robot State
 		sonarSet: the set to start fire
 		fireOrder: the Order of the sensor will be fired
		fireDelay: the delay time of fires in miliseconds
 * Return:	int : Error code of stop firing action
		0 if OK and another value when error ocurred
 * --------------------------------------------------------------*/

int SenSonar::Start(N_RobotState *state, int sonarSet, int fireOrder[8], long fireDelay)
{

	struct N_SonarSet *nsonar_set;

	nsonar_set = &(state->SonarController.SonarSet[sonarSet]);
	N_GetSonarConfiguration(1);

	nsonar_set->DataActive = TRUE;
	nsonar_set->TimeStampActive = TRUE;
	nsonar_set->FiringDelay = fireDelay;

	for (int i=0; i < 8 ; i++)
	{
		nsonar_set->FiringOrder[i]= fireOrder[i];
	}
	nsonar_set->FiringOrder[8]=N_END_SONAR_FIRING_ORDER;

	return N_SetSonarConfiguration(1);

}

/* --------------------------------------------------------------
 * Function: 	Sensonar::Stop
 * Purpose: 	stop firing all sonar sensors of a given set
 * Arguments:	*state : Robot State
 		sonarSet : the set to stop fire
 * Return:	int : Error code of stop firing action
		0 if OK and another value when error ocurred
 * --------------------------------------------------------------*/

int SenSonar::Stop(N_RobotState *state,int sonarSet)
{
	struct N_SonarSet *nsonar_set;

	nsonar_set = &(state->SonarController.SonarSet[sonarSet]);
	N_GetSonarConfiguration(1);

	nsonar_set->DataActive = TRUE;
	nsonar_set->TimeStampActive = TRUE;
	nsonar_set->FiringOrder[0]=N_END_SONAR_FIRING_ORDER;

	return N_SetSonarConfiguration(1);

}

/* --------------------------------------------------------------
 * Function: 	Sensonar::StopAll
 * Purpose: 	stop firing all sonar sensors
 * Arguments:	*state: Robot State
 * Return:	int : Error code of stop firing action
		0 if OK and another value when error ocurred
 * --------------------------------------------------------------*/

int SenSonar::StopAll(N_RobotState *state)
{

	struct N_SonarSet *nsonar_set;

	N_GetSonarConfiguration(1);

	for (int i=0; i<6; i++)
	{
		nsonar_set = &(state->SonarController.SonarSet[i]);
		nsonar_set->DataActive = TRUE;
		nsonar_set->TimeStampActive = TRUE;
		nsonar_set->FiringOrder[0]=N_END_SONAR_FIRING_ORDER;
	}


	return N_SetSonarConfiguration(1);


}

/* --------------------------------------------------------------
 * Function: 	Sensonar::Read
 * Purpose: 	return the minimum read of a given sonar index
 * Arguments:	*state : Robot State
 		sonar: the index of sonar to check
 * Return:	long : the minimum of the values from topo sonar
 		       ring and the bottom sonar ring, at the
		       given index
 * --------------------------------------------------------------*/

long SenSonar::Read(N_RobotState *state, int sonar)
{
	int set, sn;
	long r1, r2;
	
	//N_GetSonar(1);

	set = sensor_circle[sonar][0];
	sn = sensor_circle[sonar][1];

	r1 = state->SonarController.SonarSet[set].Sonar[sn].Reading;
	r2 = state->SonarController.SonarSet[set+1].Sonar[sn].Reading;

	return r1 <? r2;
}





