// <APPLET CODE=ImageControl.class width=400 height=200></APPLET>

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.net.*;
import java.io.*;

//
// Main class for control of the camera of the XR4000 robot
//

public class ImageControl extends Applet {

static String[]
  stringButtons = {"UpLeft","Up","Up Right","Left","Center","Right","DownLeft","Down","DownRight"},
  actionButtons = {"upleft","up","upright","left","zero","right","downleft","down","downright"};

Panel
  controlPanel;

Button[]
  controlButtons = new Button[9];

Label
  controlLabel;

Image
	image;

ImagePanel
	imagePanel;

AnimatePanel
  animatePanel;

URL
  server;

SocketIO
  connection;

TextField
  xchange, ychange, xabsolute, yabsolute;

// Init method for the applet. Initializes the applet and creates the panel and the
// buttons for its use.

public void init() {

  try {
  	connection = new SocketIO (getCodeBase().getHost(),8086); // opens the connection with the camera's remote server
  }
  catch (UnknownHostException e) {
	System.err.println ("Error in URL of camera server.");
  }
  catch (IOException e) {
	System.err.println ("Error connecting camera server.");
  }

  controlPanel = new Panel (new GridLayout (6,3));
  controlLabel = new Label ("");
  xchange = new TextField ("1000");
  ychange = new TextField ("400");
  xabsolute = new TextField ("0");
  yabsolute = new TextField ("0");

  int i;

  for (i=0;i<9;i++) {                   // creates all buttons
    controlButtons[i]=new Button (stringButtons[i]);
    controlButtons[i].addActionListener (new EventButton (i));
    controlPanel.add (controlButtons[i]);
  }

  controlPanel.add (new Label (""));
  controlPanel.add (new Label ("Absolute"));
  controlPanel.add (new Label ("Change"));
  controlPanel.add (new Label ("X"));
  controlPanel.add (xabsolute);
  controlPanel.add (xchange);
  controlPanel.add (new Label ("Y"));
  controlPanel.add (yabsolute);
  controlPanel.add (ychange);

  setLayout(new GridLayout(1,2));

   try {
      server = new URL ("http://upper.graco.unb.br:8087");  // for final use
//      server = new URL (getCodeBase(),"intro.jpg");  // for tests
      image = getImage (server);        // gets the first image from server
   }
   catch(MalformedURLException e) {
     System.err.println ("Error connecting server.");
   }

  System.err.println (server.toString());

  imagePanel = new ImagePanel (image);  // starts the panel

//  add (imagePanel);
  add (controlPanel);
}

// Start method for the applet. Initiates the display of images from server

public void start () {
  animatePanel = new AnimatePanel (this, server, imagePanel);  // creates the animation
  animatePanel.start();                 // starts animation
}

// Stop method for the applet. Stops the display of images.

public void stop () {
  animatePanel.stopdisplaying ();
}

// Destroy method for the applet. Stops the socket connection with the servers.

public void destroy () {
  try {
  	connection.close();
  } catch (IOException e) {
	System.err.println ("Error closing connection with camera server.");
  }
}


// Class used to catch the events created by the buttons.

class EventButton implements ActionListener {

	int index;

  EventButton (int index) {
		this.index = index;
  }

	public void actionPerformed (ActionEvent e) {
		String command;

      command = "direction=" + actionButtons[index] + ", start";

      try {
        connection.send (command);
      }
      catch(Exception exception) {
        System.err.println ("Error connecting server. Command: " + command);
	exception.printStackTrace();
      }
  }

}

}

// Class who displays images in a panel

class ImagePanel extends Panel {
    Image image;
    Dimension dimension;

    public ImagePanel(Image image) {
        this.image = image;
    }

    public void paint(Graphics g) {
        dimension = getSize ();
        g.drawImage(image, 0, 0, (int) dimension.width, (int)
dimension.height, this); //85x62 image
    }
}

// Class who opens and displays images got from a server in a specified panel

class AnimatePanel extends Thread {

    URL server;
    Image image;
    ImagePanel panel;
    Applet applet;
    boolean displaying = true;
    int imagenumber = 1;

    // Constructor for the class

    public AnimatePanel (Applet applet, URL imageserver, ImagePanel imagepanel) {
        this.applet = applet;
        server = imageserver;
        panel = imagepanel;
    }

    // The run method for the thread who displays the image

    public void run () {
        while (displaying) {
	    try {
                image = applet.getImage (new
URL("http://upper.graco.unb.br:8087")); // loads image from server
	    }
	    catch(MalformedURLException e) {
     		System.err.println ("Error connecting server.");
   	    }

            panel = new ImagePanel (image);  // displaying image
            System.out.println ("Image "+imagenumber+" loaded.");
            imagenumber++;
            try {
              sleep (5000);      // 5 seconds for each image
            }
            catch (InterruptedException e) {
            }
        }
    }

    // Interrupts the display of image got from server

    public void stopdisplaying () {
        displaying = false;
    }

}
