/************************************************************************/
/*              Program Identification                                  */
/************************************************************************/

/* Ram - Mobile Autonomous Robot Program Controller
   Blind Autonomous RoboT

	Programmer: Srgio Roberto Gonsalves Tourino
    Last update: 01/04/1999

	Contacts by email: tourino@graco.unb.br
*/

/************************************************************************/
/*              Things to do						*/
/************************************************************************/

/*
	Consertar as rotinas turnrobot e spinrobot quanto ao angulo a ser
feito pelo robo para girar um dado angulo (refazer as figuras do robo e a
analise de distancias percorridas).

*/

/************************************************************************/
/*              How to compile this program                             */
/************************************************************************/

/* Compile this program under Linux (or another UNIX) with the following
   command line:

   gcc ram.c -o ram -O1 -lm

   Under DOS or Windows this program must be modified before compilation.
   See 'Operational system definition' below.
*/

/************************************************************************/
/*              Operational system definition                           */
/************************************************************************/

/* #define DOS */
#define LINUX */

/************************************************************************/
/*              "Include" files                                         */
/************************************************************************/

/* Operational system dependents */

#ifdef LINUX
    #include <asm/io.h>
#endif

#ifdef DOS
    #include <dos.h>
#endif

/* Generic operational system */

#include <stdio.h>
#include <math.h>

/************************************************************************/
/*          Definitions and Variables                                   */
/************************************************************************/

/* Message output definition */

#define MESSAGES

/* Debug definition */

#define DEBUG

/* Port definitions (default) */

#define DATA        0x3BC       /* LPT 1 in my machine...*/
#define STATUS		DATA+1
#define CONTROL     DATA+2

/* Excitation type definitions */

#define E_SIMPLES		0		/* Single step excitation */
#define E_DUPLA         1       /* Double step excitation */

/* Motors definitions */

#define MOTOR_A_SO  -1				/* Runs both motors in opposite directions */
#define MOTOR_A_MS	0				/* Runs both motors in same direction */
#define MOTOR_LEFT   1				/* Runs only first motor */
#define MOTOR_RIGHT  2				/* Runs only second motor */
#define MOTOR_PRECISION    1.8      /* Motor's resolution */

/* Sensors definitions */

/* Speeds definitions */

#define SLOW          30
#define MEDIUM        20
#define FAST          10

/* Routine's return values */

#define OK		 0
#define ERROR	-1

/* Robot's dimensions definitions */

#define DWHEEL      70          /* Diameter of robot's wheel in mm */
#define DELTAX		300			/* Distance in x axis between wheels in mm */
#define DELTAY		100			/* Distance in y axis between center of robot
											and the x axis of wheels */
#define TRATIO		0.1			/* Transmission ratio from motor to wheels */

/* Global Variables */

char    confname[80] = "ram.conf";      /* configuration file (not used yet)*/
int     data_port;                      /* parallel port address */

/**************************************************************************/
/*          Interface Motor-Paralell Port                                 */
/**************************************************************************/

/* You must have the root access on Unix to use the following implementation */
/* Alternatively you can set the file's permition with:
	chmod 4755 file
	...logged as root	to use this program without the root access. */

void InitPort (int dataport);      /* Opens access to the paralell port */
void EndPort (void);               /* Closes access to the paralell port */
void send (int n);                 /* Sends the value to the DATA port */
void sendc (int n);                /* Sends the value to the CONTROL port */
int get (void);                    /* Gets data from STATUS port */
int getcontrol (int value);        /* Gets data from CONTROL port */
void waiting (int t);              /* Sleeps the machine */

int porta;			/* global variable for the parallel port */

void InitPort (int dataport)

/* opens access for the parallel port */

{
	porta = dataport;

#ifdef LINUX
	if (ioperm (porta,3,1))
		{
		perror ("ioperm");
		exit (1);
		}
#endif
}

void EndPort (void)

/* closes access for parallel port */

{

#ifdef LINUX
	if (ioperm (porta,3,0))
		{
		perror ("ioperm");
		exit (1);
		}
#endif
}


void send (int n)

/* sends data for the data byte in parallel port */

{
#ifdef LINUX
	outb (n,porta);
#endif

#ifdef DOS
    outportb (n,porta);
#endif
}

void sendc (int n)

/* sends data for the control byte in parallel port */

{
#ifdef LINUX
    outb (n,porta+2);
#endif

#ifdef DOS
    outportb (n,porta+2);
#endif

}

int get (void)

/* gets data from status byte in parallel port */

{
#ifdef LINUX
    return inb (porta+1);
#endif

#ifdef DOS
    return 0;
#endif
}

int getcontrol (int value)

/* gets data from control byte in parallel port */

{
#ifdef LINUX
    outb (porta+2,value);
    return inb (porta+2);
#endif

#ifdef DOS
	return 0;
#endif
}


void waiting (int t)

/* makes a t milisseconds interruption */

{
#ifdef LINUX
    usleep (t*1000);
#endif

#ifdef DOS
    delay (t);
#endif
}

/************************************************************************/
/*              Motor controls                                          */
/************************************************************************/

int motor1_s [4] = {0x01,0x02,0x04,0x08},
    motor2_s [4] = {0x10,0x20,0x40,0x80},
    motor1_d [4] = {0x09,0x03,0x06,0x0C},
    motor2_d [4] = {0x90,0x30,0x60,0xC0};

int	numero,intervalo,
		  tipo_excitacao,
	excitacao [4],
	ultima_excitacao;
float	precisao;

void    set (int n);                    /* defines the motor to be used */
void    precision (float p);            /* defines motor's precision in degrees */
void    sdelay (int tempo);             /* defines excitation delay */
int     getsensor (int id);             /* reads data from status port */
float   turn (float angle,int nsensor);   /* turns stepper motor by angle (in radians)*/
void    wait (int time);                /* stops motor at a specified time */
void    sstep (void);                   /* defines single step */
void    dstep (void);                   /* defines double step */
void    off (void);                     /* turn off the motors */

void set (int n)

{	numero = n;

	if (numero == 1)
		ultima_excitacao = 0xF0 & ultima_excitacao;
	if (numero == 2)
		ultima_excitacao = 0x0F & ultima_excitacao;

	if (tipo_excitacao)
		dstep ();
	else
		sstep ();
}

void sstep (void)

{
	if (numero == -1) {
		excitacao [0] = motor1_s [0] | motor2_s [3];
		excitacao [1] = motor1_s [1] | motor2_s [2];
		excitacao [2] = motor1_s [2] | motor2_s [1];
		excitacao [3] = motor1_s [3] | motor2_s [0]; }
	if (numero == 0) {
		excitacao [0] = motor1_s [0] | motor2_s [0];
		excitacao [1] = motor1_s [1] | motor2_s [1];
		excitacao [2] = motor1_s [2] | motor2_s [2];
		excitacao [3] = motor1_s [3] | motor2_s [3]; }
	if (numero == 1) {
		excitacao [0] = motor1_s [0] | ultima_excitacao;
		excitacao [1] = motor1_s [1] | ultima_excitacao;
		excitacao [2] = motor1_s [2] | ultima_excitacao;
		excitacao [3] = motor1_s [3] | ultima_excitacao; }
	if (numero == 2) {
		excitacao [0] = motor2_s [0] | ultima_excitacao;
		excitacao [1] = motor2_s [1] | ultima_excitacao;
		excitacao [2] = motor2_s [2] | ultima_excitacao;
		excitacao [3] = motor2_s [3] | ultima_excitacao; }
	tipo_excitacao = E_SIMPLES;
}

void dstep (void)

{
	if (numero == -1) {
		excitacao [0] = motor1_d [0] | motor2_d [3];
		excitacao [1] = motor1_d [1] | motor2_d [2];
		excitacao [2] = motor1_d [2] | motor2_d [1];
		excitacao [3] = motor1_d [3] | motor2_d [0];  }
	if (numero == 0) {
		excitacao [0] = motor1_d [0] | motor2_d [0];
		excitacao [1] = motor1_d [1] | motor2_d [1];
		excitacao [2] = motor1_d [2] | motor2_d [2];
		excitacao [3] = motor1_d [3] | motor2_d [3];  }
	if (numero == 1) {
		excitacao [0] = motor1_d [0] | ultima_excitacao;
		excitacao [1] = motor1_d [1] | ultima_excitacao;
		excitacao [2] = motor1_d [2] | ultima_excitacao;
		excitacao [3] = motor1_d [3] | ultima_excitacao; }
	if (numero == 2) {
		excitacao [0] = motor2_d [0] | ultima_excitacao;
		excitacao [1] = motor2_d [1] | ultima_excitacao;
		excitacao [2] = motor2_d [2] | ultima_excitacao;
		excitacao [3] = motor2_d [3] | ultima_excitacao;}
	tipo_excitacao = E_DUPLA;
}


void off (void)

{
	ultima_excitacao = 0x00;
	send (0x00);
}

void precision (float p)

{
	precisao = p;
}

void sdelay (int tempo)

{
	intervalo = tempo;
}


/*  Reads the status port */

int getsensor (int id)

{
int valor;

	valor = get ();
/*    printf ("%d\n",valor); */
	return (valor & id);
}


float turn (float angle,int nsensor)

{
	int ciclos,sensor,temp,altera;
	int sensibilidade = 1;
	int girando = 0;

    angle *= 180 / PI;                  /* makes angle from radians to degrees */
    altera = 0;                         /* since 'precisao' is in degrees */
	sensor = getsensor (nsensor);
	ciclos = (int) (angle / precisao) + girando;

		/* + girando pois girando pode ser != 0... */

	if (ciclos > 0) {
		for (;girando < ciclos;girando++) {
			send (excitacao [girando % 4]);

		/* girando % 4 e' um numero entre 0 e 3... */

			waiting (intervalo);
			temp = getsensor (nsensor);
			if (temp != sensor)
				altera++;
			else
				altera = 0;
			if (altera == sensibilidade) {
                return girando * precisao * (PI / 180);  /* returns angle in radians */
				}
			sensor = temp;
			}
		ultima_excitacao = excitacao [girando % 4];  /* grava a ultima excitacao */
		girando = girando % 4;		/* atualiza o valor de girando */
		}
	else {
		for (;girando < (-ciclos);girando++) {
			send (excitacao [3 - girando % 4]);
			waiting (intervalo);
			temp = getsensor (nsensor);
			if (temp != sensor)
				altera++;
			else
				altera = 0;
			if (altera == sensibilidade) {
                return girando * precisao * (PI / 180);
				}
			sensor = temp;
			}
		ultima_excitacao = excitacao [3 - girando % 4];
		girando = girando % 4;
		};
	return angle * (PI/180);
}

void wait (int time)

{
	send (ultima_excitacao);		/* envia a ultima excitacao */
                                    /* usada no motor */
	waiting (time);
}

/************************************************************************/
/*              Generic Routines                                        */
/************************************************************************/

void InitMotors (void);			/* Initializes motors */
void EndMotors (void);			/* Stops motors */

void InitMotors (void)

{
	InitPort (DATA);
	off ();
	precision (MOTOR_PRECISION);
	sdelay (MEDIUM);
	sstep ();
}

void EndMotors (void)

{
	off ();
	EndPort ();
}

/************************************************************************/
/*              Walking Routines                                        */
/************************************************************************/

void WalkRobot (float *x,float *y,float *t,float s,float v);
    /* Walks robot in straigth line */
void TurnRobot (float *x,float *y,float *t,float a,float v);
    /* Turns robot with one wheel only */
void SpinRobot (float *x,float *y,float *t,float a,float v);
    /* Turns robot with both wheels */

/* Variables:

x,y     actual position of robot
t	actual orientation of robot
s       distance to be walked by robot (in milimeters)
a       angle to be turned by robot (in radians)
v	velocity of translation of robot at movements

Note: the above routines send back to user the position and the orientation
		after the movement in the variables x,y and t

*/

void WalkRobot (float *x,float *y,float *t,float s,float v)

{
int sensor,speed;
float angle;

#ifdef MESSAGES
	printf ("Walking...\n");
#endif

angle = 2 * s / (DWHEEL * TRATIO);    /* calculates angle to be turned by motors */
speed = FAST;                               /* here will be calculated the velocity... */

InitMotors ();
set (MOTOR_A_MS);                           /* uses both motors */
dstep ();
sdelay (speed);
angle = turn (angle,sensor);                /* make motors turning */
EndMotors ();

s = angle * DWHEEL * TRATIO / 2;            /* calculates the real movement done */
*x += s * cos (*t);                         /* calculates new position after movement */
*y += s * sin (*t);                         /* the value of t is left unchanged */

}

void TurnRobot (float *x,float *y,float *t,float a,float v)

{
int sensor,speed,motor;
float angle,turned,dx,dy;

#ifdef MESSAGES
	printf ("Turning...\n");
#endif

a *= PI / 180;			/* from degrees to radians */
angle = abs (2 * DELTAX * a / (DWHEEL * TRATIO));
speed = FAST;

#ifdef DEBUG
	printf ("%f\n",angle);
#endif

InitMotors ();
if (a > 0)		/* choose the motor to be used in turning the robot */
	{
	set (MOTOR_LEFT);
	motor = MOTOR_LEFT;
	}
else
	{
	set (MOTOR_RIGHT);
	motor = MOTOR_RIGHT;
	}

dstep ();
sdelay (speed);
angle = turn (angle,sensor);
EndMotors ();

turned = DWHEEL * angle * TRATIO / (4 * DELTAX);
*t -= turned;
dx = DELTAX - DELTAX * cos (turned) - DELTAY * sin (turned);

if (motor = MOTOR_RIGHT)
	dx = -dx;
dy = DELTAY + DELTAX * sin (turned) - DELTAY * cos (turned);

*x += dx;
*y += dy;
}

void SpinRobot (float *x,float *y,float *t,float a,float v)

{
int sensor,speed,motor;
float angle,turned,dx,dy;

#ifdef MESSAGES
	printf ("Spinning...\n");
#endif

a *= PI / 180;			/* from degrees to radians */
angle = abs (DELTAX * a / (DWHEEL * TRATIO));
speed = FAST;

#ifdef DEBUG
	printf ("%f\n",angle);
#endif

InitMotors ();
set (MOTOR_A_SO);				/* uses both motors in opposite directions */
dstep ();
sdelay (speed);
angle = turn (angle,sensor);
EndMotors ();

turned = DWHEEL * angle * TRATIO / (2 * DELTAX);
*t += turned;
dx = - DELTAY * sin (turned);
dy = DELTAY - DELTAY * cos (turned);

*x += dx;
*y += dy;
}

/************************************************************************/
/*              Main routine                                            */
/************************************************************************/


int main (int argc,char *argv [])

{
float x,y,t;

	x = 0;
	y = 0;
	t = PI / 2;

 	printf ("Starting robot...\n");
/*	printf ("The values are: %f, %f, %f\n",x,y,t);
    	WalkRobot (&x,&y,&t,200,1); 
	printf ("The values are: %f, %f, %f\n",x,y,t); */
    	TurnRobot (&x,&y,&t,90,1); 
	printf ("The values are: %f, %f, %f\n",x,y,t);
/*    	SpinRobot (&x,&y,&t,-90,1);
	printf ("The values are: %f, %f, %f\n",x,y,t);  */
    	printf ("All job done!\n");
	return 0;
}


