// ACIS V6.3
// This program creates a prism and saves it to a file
#include <stdio.h>                             // File I/O functions
#include <stdlib.h>
#include "boolean/kernapi/api/boolapi.hxx" // Declares boolean API's
#include "constrct/kernapi/api/cstrapi.hxx"    // Declares constructor api's
#include "kernel/acis.hxx"                 // Declares system wide parameters
#include "kernel/acis3dt.h"
#include "kernel/kernapi/api/kernapi.hxx"      // Declares kernel api's
#include "kernel/kernapi/api/api.hxx"
#include "kernel/kerndata/top/body.hxx" //Declara a classe BODY
#include "kernel/kerndata/savres/fileinfo.hxx" // Declares fileinfo class
#include "kernel/kerndata/geometry/getbox.hxx" 
#include "kernel/kerndata/geom/transfrm.hxx"
#include "baseutil/vector/transf.hxx"  // Declares position
#include "baseutil/vector/vector.hxx"
#include "baseutil/vector/position.hxx"  // Declares position
#include "cover/kernapi/api/coverapi.hxx"   // Declares api_cover_wires
#include "VrmlMM.hxx"
#include "VrmlMM.cxx"
#include "cgiparser.h"

// include the rude namespace
using namespace rude;

inline position p(double p1,double p2,double p3)
{ return position (p1,p2,p3); }

inline double max(double a, double b)
{ return  (a > b) ? a: b; }

// Functions prototype
void cria_paralelepipedo(double larg, double comp, double alt);
void cria_cilindro(double alt, double radius1, double radius2, double radius3);
void cria_cone(double altu, double rmai1, double rmai2, double rmen);
void cria_esfera(double radius);
void cria_torus(double rmai, double rmen);
void cria_prisma_eliptico(double altu, double rmaip, double rmenp, double nlados);
void cria_piramide_eliptica(double altu, double rmaip, double rmenp, double rtop, double nlados);
void cria_cruz(double ccin1, double dcin1, double ccin2, double dcin2);
void cria_cunha(double larg, double comp, double altu);
void save_ent(char*, ENTITY*);  
void save_wrl(char *, char *);

char diam_s[20], altu_s[20], comp_s[20], larg_s[20], dmai_s[20], rmai_s[20], rmen_s[20];
char dmen_s[20], nlados_s[20], rtop_s[20], dcin1_s[20], dcin2_s[20], ccin1_s[20], ccin2_s[20];


int main()
{
		
		double diam, radius, altu, comp, larg;
		double dmai, dmen, rmai, rmen, dcin1, dcin2, ccin1, ccin2;
		double rmaip, rmenp, rtop; 
		int tipo, nlados;

		CGIParser *parser = CGIParser::instance();
		
		diam = atof(parser->value("diam_s"));
		radius = 0.5 * diam;
		
		comp = atof(parser->value("comp_s"));
		larg = atof(parser->value("larg_s"));
		altu = atof(parser->value("altu_s"));

		dmai = atof(parser->value("dmai_s"));
		rmai = 0.5 * dmai;
		dmen = atof(parser->value("dmen_s"));
		rmen = 0.5 * dmen;

		//Referentes a prisma elptico
		nlados = atoi(parser->value("nlados_s"));
		rmaip = atof(parser->value("rmai_s"));
		rmenp = atof(parser->value("rmen_s"));
		rtop = atof(parser->value("rtop_s"));

		//Referentes a cruz
		dcin1 = atof(parser->value("dcin1_s"));
		dcin2 = atof(parser->value("dcin2_s"));
		ccin1 = atof(parser->value("ccin1_s"));
		ccin2 = atof(parser->value("ccin2_s"));

		//conversao  p/ inteiro
		tipo = (int) atof(parser->value("tipo"));

		//construo do slido
		api_start_modeller(0);
		api_initialize_constructors();   

    switch(tipo)
		{ 
			
		  case 0:
    	  cria_paralelepipedo(larg, comp, altu);
			break;

		  case 1:
		 		cria_cilindro(altu, radius, radius, radius);
			break;

		  case 2:
				cria_cone(altu, rmai, rmai, rmen);
		  break;

			case 3:
				cria_esfera(radius);
		  break;

		  case 4:
				cria_torus(rmai, rmen);
		  break;

		  case 5:
				cria_prisma_eliptico(altu, rmaip, rmenp, nlados);
		  break;

		  case 6:
				cria_piramide_eliptica(altu, rmaip, rmenp, rtop, nlados);
		  break;
		
		  case 7:
     		cria_cruz(ccin1, dcin1, ccin2, dcin2);
			break;

			case 8:
        cria_cunha(larg, comp, altu);
			break;

		}//Fim do switch

    api_terminate_constructors();
    api_stop_modeller();

		return 1;

}//Fim do main

void cria_paralelepipedo(double larg, double comp, double alt)
{
		BODY *bloco;
	  api_make_cuboid(larg, comp, alt, bloco);
	  save_ent("..\\htdocs\\acis1\\sats\\paralelepipedo.sat",bloco);
		save_wrl("..\\htdocs\\acis1\\sats\\paralelepipedo.sat", "..\\htdocs\\acis1\\wrls\\paralelepipedo.wrl");
    printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhaparalelepipedo.shtml\n\n");
}

void cria_cilindro(double altu, double radius1, double radius2, double radius3)
{
		BODY *cilindro;
		api_make_frustum(altu, radius1, radius2, radius3, cilindro);
	  save_ent("..\\htdocs\\acis1\\sats\\cilindro.sat", cilindro);
  	save_wrl("..\\htdocs\\acis1\\sats\\cilindro.sat", "..\\htdocs\\acis1\\wrls\\cilindro.wrl");
		printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhacilindro.shtml\n\n");
}

void cria_cone(double altu, double rmai1, double rmai2, double rmen)
{
  BODY *cone;
	api_make_frustum(altu, rmai1, rmai2, rmen, cone);
	save_ent("..\\htdocs\\acis1\\sats\\cone.sat", cone);
	save_wrl("..\\htdocs\\acis1\\sats\\cone.sat", "..\\htdocs\\acis1\\wrls\\cone.wrl");
	printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhacone.shtml\n\n");
}

void cria_esfera(double radius)
{
		BODY *esfera;				
		api_make_sphere(radius, esfera);
		save_ent("..\\htdocs\\acis1\\sats\\esfera.sat",esfera);
		save_wrl("..\\htdocs\\acis1\\sats\\esfera.sat", "..\\htdocs\\acis1\\wrls\\esfera.wrl");
		printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhaesfera.shtml\n\n");
}

void cria_torus(double rmai, double rmen)
{
	  BODY *torus;
		api_make_torus(rmai, rmen, torus);
		save_ent("..\\htdocs\\acis1\\sats\\torus.sat",torus);
		save_wrl("..\\htdocs\\acis1\\sats\\torus.sat", "..\\htdocs\\acis1\\wrls\\torus.wrl");
		printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhatorus.shtml\n\n");
}

void cria_prisma_eliptico(double altu, double rmaip, double rmenp, double nlados)
{
    BODY *prisma;
		api_make_prism(altu, rmaip, rmenp, nlados, prisma);
		save_ent("..\\htdocs\\acis1\\sats\\prisma.sat", prisma);
		save_wrl("..\\htdocs\\acis1\\sats\\prisma.sat", "..\\htdocs\\acis1\\wrls\\prisma.wrl");
		printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhaprisma.shtml\n\n");
}

void cria_piramide_eliptica(double altu, double rmaip, double rmenp, double rtop, double nlados)
{
    BODY *piramide;
	  api_make_pyramid(altu, rmaip, rmenp, rtop, nlados, piramide);
		save_ent("..\\htdocs\\acis1\\sats\\piramide.sat", piramide);
		save_wrl("..\\htdocs\\acis1\\sats\\piramide.sat", "..\\htdocs\\acis1\\wrls\\piramide.wrl");
		printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhapiramide.shtml\n\n");
}

void cria_cruz(double ccin1, double dcin1, double ccin2, double dcin2)
{
		BODY *cyl1, *cyl2;
		api_initialize_booleans();
    			
		// make two cylinders 
    api_make_frustum(ccin1, dcin1*0.5, dcin1*0.5, dcin1*0.5, cyl1);
    api_make_frustum(ccin2, dcin2*0.5, dcin2*0.5, dcin2*0.5, cyl2);

		// Rotate cylinder 90 degrees about x-axis
    transf rotX = rotate_transf(M_PI/2,vector(1,0,0));
    api_apply_transf(cyl1, rotX);

		// Unite the two cylinders
    api_unite(cyl1, cyl2);   
		save_ent("..\\htdocs\\acis1\\sats\\cruz.sat",cyl2);
		save_wrl("..\\htdocs\\acis1\\sats\\cruz.sat", "..\\htdocs\\acis1\\wrls\\cruz.wrl");
		printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhacruz.shtml\n\n");

		api_terminate_booleans();
}

void cria_cunha(double larg, double comp, double altu)
{
		BODY *cunha;		
		api_initialize_covering();
  
		// Create a cube
	  api_make_cuboid(larg, comp, altu, cunha);
  
		// Get a bounding box around the model
	  box solido_size = get_body_box(cunha);
 
	  // Find out the length of its diagonal
	  position bottom = solido_size.low();
	  position top = solido_size.high();
	  vector diagonal  = top - bottom;
    double len = diagonal.len();
	
	  // Create a circular edge
	  position cen(0,0,0);         // point on section
	  vector direction(0,0,1);     // Determines direction of face (corta no eixo Z)
    unit_vector norm = normalise(direction);
    vector radius(len*0.6,0,0);  // Box size plus a bit
  
	  EDGE *plano_e;
	  api_mk_ed_ellipse(cen,norm,radius,1,0,0,plano_e);

	  //Turn the edge into a WIRE Body
    EDGE *e_array[1];
	  e_array[0] = plano_e;
	  BODY *plano_wb;
	  api_make_ewire(1,e_array,plano_wb);

    //Cover the WIRE to make a planar FACE
    ENTITY_LIST plano_faces;
    api_cover_wires(plano_wb,*(surface*)NULL,plano_faces);

		//Girando o plano para cortar o solido
    double ang = atan(altu/larg);
	  transf rot = rotate_transf(ang+M_PI, vector(0,1,0));
	  api_apply_transf(plano_wb, rot);
	
	  //Carry-out the section
    api_boolean(plano_wb, cunha, SUBTRACTION);
			
		save_ent("..\\htdocs\\acis1\\sats\\cunha.sat", cunha);
		save_wrl("..\\htdocs\\acis1\\sats\\cunha.sat", "..\\htdocs\\acis1\\wrls\\cunha.wrl");
		printf("Location: http://einstein.grucon.ufsc.br/acis1/shtml/desenhacunha.shtml\n\n");

 	  api_terminate_covering();
}

//CRIA O ARQUIVO .SAT
void save_ent(char *filename, ENTITY *ent)
{
		FileInfo info;  // Create FileInfo Object
		info.set_product_id("UFSC");  // set info's data
		info.set_units(10.0);  // Millimeters 

      // Sets header info to be written to sat file 
		api_set_file_info(FileId | FileUnits, info);

        FILE *fp = fopen(filename, "w");
        if (fp != NULL) {
                ENTITY_LIST savelist; //= new ENTITY_LIST;
                savelist.add(ent);
                api_save_entity_list(fp,TRUE,savelist);
                //delete savelist;
				fclose(fp); 
         }
         else
		 {   printf("Unable to open file!\n");
		     exit(1);
		 }
		
        
}


//CRIA O ARQUIVO .VRL A PARTIR DO ARQUIVO .SAT GERADO
void save_wrl(char *filein, char *fileout)
{ //char *path;
  outcome rc;
  BODY *TheBody = NULL;
  FILE *pSatFile = NULL;
  FILE *pOutFile = NULL;
  ENTITY_LIST EntList;
  REFINEMENT* ref = NULL;
  int index = 0;
  int refFlag = 0;


	rc = api_start_modeller(0);
  
  pSatFile = fopen( filein, "r" );
  
  rc = api_restore_entity_list(pSatFile, TRUE, EntList);
  fclose(pSatFile);
  
  pOutFile = fopen(fileout, "w");
  
  rc = api_initialize_faceter();
  
  VRML_MESH_MANAGER *MM = new VRML_MESH_MANAGER();
  rc = api_set_mesh_manager( (MESH_MANAGER *) MM);

  rc = api_get_default_refinement( ref );
  
  /*if( refFlag ) {
    setRefine( ref );
  }
  */
  rc = api_set_default_refinement( ref );
  
  MM->set_output_file(pOutFile);
  MM->header();

  //find the scale and apply any transforms already there
  EntList.init();
  box bounds;
  index = 1;
  while( TheBody = (BODY*)EntList.next() ) {
    if( !is_REFINEMENT( (ENTITY*)TheBody ) ) {
      api_change_body_trans( TheBody, NULL );

      if( index == 1 ) {
	bounds = get_body_box( TheBody, NULL );
	index = 0;
      }
      else {
	bounds = bounds | get_body_box( TheBody, NULL );
      }
    }
  }

  //finds the scale so that the bodies to fit in a 
  //100*x*y box where x <= 100 && y <= 100
  //it then centers all the bodies at (50, 50, 50)
  double max_dim = labs( bounds.high().x() - bounds.low().x() );
  max_dim = max( max_dim,
		 labs( bounds.high().y() - bounds.low().y()) );
  max_dim = max( max_dim,
		 labs( bounds.high().z() - bounds.low().z()) );
  //make scale transform
  double scale = 100.0/max_dim;
  transf trans = scale_transf( scale );
  //make translation transform
  vector vec =  position(50.0, 50.0, 50.0) - bounds.mid() * scale;
  transf tranl = translate_transf( vec );

  index = 0;
  EntList.init();
  while(TheBody = (BODY*)EntList.next()) {
    if( !is_REFINEMENT( TheBody ) ) {
      index++;
      
      //apply scale transform
      api_apply_transf( (ENTITY *)TheBody, trans );
      rc = api_change_body_trans( TheBody, NULL );
      
      //apply translation transform
      api_apply_transf( (ENTITY *)TheBody, tranl );
      rc = api_change_body_trans( TheBody, NULL );
      
      rc = api_facet_entity( TheBody );
      
    }
  }
  
 
  MM->footer();
  fclose(pOutFile);
  delete MM;

  EntList.init();
  while(TheBody = (BODY*)EntList.next())
    api_del_entity(TheBody);
  
  api_terminate_faceter();
	api_stop_modeller();


}









